/************************************************************************/

//			Skeleton for gsfdm plug-in

//				E-mail : j.a.l @ free.fr

/************************************************************************/

#include	<gnome.h>
#include	<glib.h>
#include	<stdio.h>
#include	<signal.h>
#include	<sys/wait.h>
#include	<fcntl.h>
#include	"./gsfdm_plugin_shared.h"
#include	"./main.h"

/************************************************************************/
//	debug switch
/************************************************************************/

#define	DEBUG_UPDATE
#define DEBUG_PARSE_DATA_FILE

/************************************************************************/
//	structure and define definition
/************************************************************************/
#include "your plug in icon.xpm"

#define SOFT			"asfrecord3"
#define PLUGIN_NAME		"Your plug-in name"
#define VERSION			"plug in version"
#define GSFDM_VERSION	"Version of gsfdm this plug least need"
#define COMMENT			"A description..."
static gchar const		*PROTOCOL[] = { "a least one protocol:", NULL };
static gchar const		*EXTENSION[] = { ".ext this plug can parse", NULL };
	
static gchar const		*AUTHORS[] = { "Unknow", NULL };

/************************************************************************/
//	functions definition
/************************************************************************/
// this is the list of all functions defined. all MUST be present, otherwise, the plugin loader
// will cancel use of the library

#define CHILD_BUFFER_LENGTH 500			// length of a line... must be ok !

// buffer used for reading a line from child stream....
static gchar			data[ CHILD_BUFFER_LENGTH ];

static gboolean		plug_initialized = FALSE;
static GHashTable	*hash = NULL;			// a list of job, if you prefer you own solution...

/************************************************************************/
//	first call ( after loading through g_module_open and symbols resolving )
// here you do things the plug need at startup
// return TRUE on success
gboolean	plug_init( void )
{
	// only need a table to stock open descriptor to child
	if( !plug_initialized )
	{	// look if found the soft
		// but this bore me, so suppose it's on path...
		
		// create the hash for open job, key is pid
		if( hash ) g_hash_table_destroy( hash );
			
		if( ( hash = g_hash_table_new( g_direct_hash, g_direct_equal ) ) )
			plug_initialized = TRUE;
		
		
		// load config switch
		gnome_config_push_prefix( "/"PLUGIN_NAME"/std/" );
		// ..............

		gnome_config_pop_prefix();	
	}
	
	return( plug_initialized );
}

/************************************************************************/
// need more explaination ???
// if safe=TRUE; ask plugin, if could,  not stop current work : push it at backgroud
gboolean	plug_stop( gboolean safe )
{
	static void safe_mode( gpointer key, gpointer value, gpointer data )
	{
		// may i connect  back to stdin and stdout ???
		st_gsfdm_job_destroy( value );		
	}
	
	static void kill_mode( gpointer key, gpointer value, gpointer data )
	{
		kill( st_gsfdm_job_get_pid( value ), 9 );
		st_gsfdm_job_destroy( value );
	}
	
	/************************************************************************/
	// kill 
	if( safe )
	{	g_hash_table_foreach( hash, (GHFunc)safe_mode, NULL );
	} else {
		g_hash_table_foreach( hash, (GHFunc)kill_mode, NULL );
	}
	
	if( hash )
		g_hash_table_destroy( hash );
		
	plug_initialized = FALSE;
	
	//...................................
	
	// save config
	gnome_config_push_prefix( "/"PLUGIN_NAME"/std/" );
	
	// .................................
	
	gnome_config_pop_prefix();	
	gnome_config_sync();

	return( TRUE );
}

/************************************************************************/
// return a static string for the full plug name, used at display
gchar const *plug_get_name( void )
{	return( PLUGIN_NAME );
}

/************************************************************************/
// version number ; typ return( VERSION );
gchar const	*plug_get_version( void )
{	return( VERSION );
}

/************************************************************************/
// return array to const gchar * of all authors names
gchar const	**plug_get_author( void )
{	return( AUTHORS );
}

/************************************************************************/
// data for and icon or ( NULL if nome ), this is the variable 
// created when you included an XPM icon
gchar const	**plug_get_icon( void )
{	return( ( gchar const **)icon_xpm );
}

/************************************************************************/
// return version of gsfdm this plug was created for ;
// perhaps usefull further if the interface changed...
gchar const *plug_get_gsfdm_version( void )
{	return( GSFDM_VERSION );
}

/************************************************************************/
// a comment ; description of plug
gchar const *plug_get_comment( void )
{	return( COMMENT );
}

/************************************************************************/
// return null ending array of supported protocol [] = { "http:", "https:", NULL };
gchar const	**plug_get_protocol_list( void )
{	return( PROTOCOL );
}

/************************************************************************/
// this is the list of HD FILE the plug is able to parse.
// exemple : .asf that enclose stream info
// null if none
gchar const	**plug_get_ext_list( void )
{	return( EXTENSION );
}

/************************************************************************/
// TRUE if provide a gnome dlg for configuration
gboolean	plug_have_gnome_config( void )
{	return( TRUE );
}

/************************************************************************/
// return the top widget of the dlg in a form of gpointer
// this is for compatibility if no gnome support
gpointer	plug_get_gnome_dlg( void )
{	
GnomeDialog	*dlg = NULL;
	
	#ifdef DEBUG_GSFDM_PLUGIN_TEST_CONFIG
	g_print( DEBUG_ID"opening config dlg" );
	#endif

	//	create the dialog at first call, keep it through the plugin life
	// this is useful if dlg is big and you want to keep 
	// of course could use a global static instead local, but in this case...
	if( !dlg )
	{	dlg = GNOME_DIALOG( gnome_dialog_new( PLUGIN_NAME, GNOME_STOCK_BUTTON_OK, NULL ) );
		gnome_dialog_close_hides( dlg, TRUE );
		gnome_dialog_set_close( dlg, TRUE );
		
		
		//..................................
		
		
		gtk_widget_show_all( dlg->vbox );
	}
	
	return( (gpointer)(dlg) );
}

/************************************************************************/
// start dowloading this job, return pid of child running it ( or 0 on error )
guint	plug_start_job( st_gsfdm_url_entry_shared *s, st_gsfdm_config_proxy *proxy )
{
st_gsfdm_job	*j = st_gsfdm_job_new();
pid_t			pid = 0, ppid = getpid();
int				fd[2];
	
	// prevent zombie
	signal( SIGCHLD, SIG_IGN );

	// NOTA : using popen would be a great idea. Infortunly, with this function
	// there no way ( i think ) to get the pid process owning the streamed data...
	// must use the boring-classic solution
	
	if( pipe( fd ) )
	{	g_warning( "error starting asf command... abort" );
		st_gsfdm_job_destroy( j );
		return( 0 );
	}
	
	if( !( pid = fork() ) )
	{	// child process
		static gchar	*list[30];					// must be enough
		guint			count = 0;
		
		// use one pipe from child ( asfrecord3 ) to parent ( plugin
		dup2( fd[ STDOUT_FILENO ], STDOUT_FILENO );
		close( fd[ STDIN_FILENO ] );

		// start the list of arg
			list[ count++ ] = SOFT;
		
		//..........................
		
		// QUESTION HOW I SPECIFY USER AND PASS FOR PROXY ????
		// "-a" IS IT FOR PROXY OR FOR REMOTE AUTHORISATION ????
		if( gsfdm_proxy_st_get_use_proxy( proxy ) )
		{	
			//......................	
		}	
		
		//............................
		
		list[ count ] = NULL;
		
		// set to background directly, if gsfdm crash, still download...
		setpgid( 0, ppid );
		setsid();

		// disable local support, comme back to eng.
		putenv( "LC_ALL=C" );
		chdir( gsfdm_url_st_shared_get_path( s ) );
		
		// execute
		execvp( list[ 0 ], list );
		exit( -1 );							// exit on error
	}		
	
	// error in fork ???
	if( pid < 0 )
	{	g_warning( "error starting asf command... abort" );
		st_gsfdm_job_destroy( j );
		return( 0 );		
	}
	
	// parent process
	// out for child is read for parent...
	close( fd[ STDOUT_FILENO ] );
	st_gsfdm_job_get_read( j ) = fd[ STDIN_FILENO ];
	fcntl( st_gsfdm_job_get_read( j ), F_SETFL, O_NONBLOCK );
	st_gsfdm_job_get_write( j ) = 0;			// don't write to child, just read
	st_gsfdm_job_get_stream( j ) = fdopen( fd[ STDIN_FILENO ], "r" );	

	
	//.......................................
	
	
	// add now to list ( of running process )
	st_gsfdm_job_get_pid( j ) = pid;
	g_hash_table_insert( hash, (gpointer)pid, (gpointer)j );
	gsfdm_url_st_shared_get_pid( s ) = pid;
	
	return( pid );
}

/************************************************************************/
// request abort downloading of this job - identify it with pid...
// TRUE if successful ; you have to take care of kill child pid by yourself
gboolean	plug_stop_job( st_gsfdm_url_entry_shared *s )
{
st_gsfdm_job	*j = ST_GSFDM_JOB( g_hash_table_lookup( hash, (gpointer)gsfdm_url_st_shared_get_pid( s ) ) );
gboolean		val = FALSE;			// unknow pid by default...

	if( j )
	{	// found, now stop
		if( !kill( st_gsfdm_job_get_pid( j ), 9 ) )
			val = TRUE;
		
		//......................................
		
		g_hash_table_remove( hash, (gpointer)st_gsfdm_job_get_pid( j ) );
		st_gsfdm_job_destroy( j );
	} 
	
	return( val );
}

/************************************************************************/
// update infos ( percent, size ) and return the state ( after update )
GsfdmUrlState plug_update_job( st_gsfdm_url_entry_shared *s )
{
st_gsfdm_job	*j = ST_GSFDM_JOB( g_hash_table_lookup( hash, (gpointer)gsfdm_url_st_shared_get_pid( s ) ) );
GsfdmUrlState	state = GSFDM_URL_STATE_RUNNING;	// if here, gsfdm ask update for - so far - a ruuning process

	// test for stuct... no really the right state but it's the closer
	if( !j )
		return( GSFDM_URL_STATE_PARTIAL );	
	
	// read data from stream
	while( fgets( data, CHILD_BUFFER_LENGTH, st_gsfdm_job_get_stream( j ) ) )
	{
		
		//...................................
		
	}
	
	// perhaps there no new data available : this is not an error, just don't care...

	//............................................
	
	return( state );
}

/************************************************************************/
// when loading from disk, gsfdm request the plug to parse the
// description file ( like m3u, asx... also html)
// return true on success
// read data fron input stream
// write '\n' ending string for each url to add
// string into output ( flush after each write ), 
// s : struct that own the data or NULL if unavailable
// extension of the file to parse if you need it...
gboolean	plug_parse_file( FILE *input, FILE *output, st_gsfdm_url_entry_shared const *s, gchar const *ext )
{
gboolean			state = FALSE;

	#ifdef DEBUG_PARSE_DATA_FILE
	g_print( "ASF - start parsing\n" );
	#endif

	if( !input || !output )
		return( FALSE );	

	
	//..........................................
	
	
	return( state );
}

