/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		Header asf plug

/************************************************************************/

#include	<gnome.h>
#include	<glib.h>
#include	<stdio.h>
#include	<signal.h>
#include	<sys/wait.h>
#include	<fcntl.h>
#include	<regex.h>
#include	"./gsfdm_plugin_shared.h"
#include	"./header_skeleton.h"

#include	"../../config.h"

/************************************************************************/
//	debug switch
/************************************************************************/

#ifdef HAVE_DEBUG
#define	DEBUG_UPDATE
//#define	PINGPONG_TEST
#define DEBUG_PARSE_DATA_FILE
#endif

/************************************************************************/
//	structure and define definition
/************************************************************************/
#include "icon.xpm"

#define SOFT			"asfrecord3"
#define PLUGIN_NAME		"Gsfdm_ASF_Downloader"
#define VERSION			"0.1.2"
#define GSFDM_VERSION	"0.2"
#define COMMENT			"Gsfdm Plug-in, for downloading of Micro$oft ASF streamed files"
static gchar const		*PROTOCOL[] = { "mms:", "mmst:", "mmsu:", NULL };
static gchar const		*EXTENSION[] = { ".asf", ".afs", ".asx", ".wax", ".wma", ".wvx", ".wmv",".wma", NULL };
	
static gchar const		*AUTHORS[] = { "Unknow", NULL };

#define CHILD_BUFFER_LENGTH 500			// length of a line... must be ok !
//guint					length = CHILD_BUFFER_LENGTH ;

static gchar			data[ CHILD_BUFFER_LENGTH ];
//gchar					*stream_buffer = NULL;

/************************************************************************/
//	functions definition
/************************************************************************/
// this is the list of all functions defined. all MUST be present, otherwise, the plugin loader
// will cancel use of the library

static gboolean		plug_initialized = FALSE;
static GHashTable	*hash = NULL;

// if this switch is on, after download successful, the plug
// check wether it was a defininition and automaticaly download data
static gboolean		auto_dl = FALSE;

// write also a raw file ???
static gboolean		raw_file = FALSE;

/************************************************************************/
//	first call ( after loading through g_module_open and symbols resolving )
// here you do things the plug need at startup
// return TRUE on success
gboolean	plug_init( void )
{
	// only need a table to stock open descriptor to child
	if( !plug_initialized )
	{	// look if found the soft
		// but this bore me, so suppose it's on path...
		
		// create the hash for open job, key is pid
		if( hash ) g_hash_table_destroy( hash );
			
		if( ( hash = g_hash_table_new( g_direct_hash, g_direct_equal ) ) )
			plug_initialized = TRUE;
		
		// initial buffer for reading data
//		if( !data )
//			data = g_new0( char, length );
		
//		if( !stream_buffer )
//			stream_buffer = g_new0( char, BUFSIZ );
		
		// load config switch
		gnome_config_push_prefix( "/"PLUGIN_NAME"/std/" );
		raw_file = gnome_config_get_bool_with_default( "raw=FALSE", NULL );
		gnome_config_pop_prefix();	
	}
	
	return( plug_initialized );
}

/************************************************************************/
// need more explaination ???
// if safe=TRUE; ask plugin, if could,  not stop current work : push it at backgroud
gboolean	plug_stop( gboolean safe )
{
	static void safe_mode( gpointer key, gpointer value, gpointer data )
	{
		// may i connect  back to stdin and stdout ???
		st_gsfdm_job_destroy( value );		
	}
	
	static void kill_mode( gpointer key, gpointer value, gpointer data )
	{
		kill( st_gsfdm_job_get_pid( value ), 9 );
		st_gsfdm_job_destroy( value );
	}
	
	/************************************************************************/
	// kill 
	if( safe )
	{	g_hash_table_foreach( hash, (GHFunc)safe_mode, NULL );
	} else {
		g_hash_table_foreach( hash, (GHFunc)kill_mode, NULL );
	}
	
	if( hash )
		g_hash_table_destroy( hash );
	
//	g_free( data );
	
	plug_initialized = FALSE;
	
	// save config
	gnome_config_push_prefix( "/"PLUGIN_NAME"/std/" );
	raw_file = gnome_config_get_bool_with_default( "raw=FALSE", NULL );
	gnome_config_pop_prefix();	
	gnome_config_sync();

	return( TRUE );
}

/************************************************************************/
// return a static string for the full plug name, used at display
gchar const *plug_get_name( void )
{	return( PLUGIN_NAME );
}

/************************************************************************/
// version number ; typ return( VERSION );
gchar const	*plug_get_version( void )
{	return( VERSION );
}

/************************************************************************/
// return array to const gchar * of all authors names
gchar const	**plug_get_author( void )
{	return( AUTHORS );
}

/************************************************************************/
// data for and icon or ( NULL if nome ), this is the variable 
// created when you included an XPM icon
gchar const	**plug_get_icon( void )
{	return( ( gchar const **)icon_xpm );
}

/************************************************************************/
// return version of gsfdm this plug was created for ;
// perhaps usefull further if the interface changed...
gchar const *plug_get_gsfdm_version( void )
{	return( GSFDM_VERSION );
}

/************************************************************************/
// a comment ; description of plug
gchar const *plug_get_comment( void )
{	return( COMMENT );
}

/************************************************************************/
// return null ending array of supported protocol [] = { "http:", "https:", NULL };
gchar const	**plug_get_protocol_list( void )
{	return( PROTOCOL );
}

/************************************************************************/
// this is the list of HD FILE the plug is able to parse.
// exemple : .asf that enclose stream info
// null if none
gchar const	**plug_get_ext_list( void )
{	return( EXTENSION );
}

/************************************************************************/
// TRUE if provide a gnome dlg for configuration
gboolean	plug_have_gnome_config( void )
{	return( TRUE );
}

/************************************************************************/
static void	click_check_bp_callback( GtkToggleButton *c, gpointer d )
{
	*((gboolean *)d) = gtk_toggle_button_get_active( c );
}

/************************************************************************/
// return the top widget of the dlg in a form of gpointer
// this is for compatibility if no gnome support
gpointer	plug_get_gnome_dlg( void )
{	
GnomeDialog	*dlg = NULL;
GtkBox		*b, *b2;
GtkWidget	*c;
	
	#ifdef DEBUG_GSFDM_PLUGIN_TEST_CONFIG
	g_print( DEBUG_ID"opening config dlg" );
	#endif
	//	create the dialog at first call, keep it through the plugin life
	// this is useful if dlg is big and you want to keep 
	// of course could use a global static instead local, but in this case...
	if( !dlg )
	{	dlg = GNOME_DIALOG( gnome_dialog_new( _( "GSFDM AsfRecord3 Configuration" ), GNOME_STOCK_BUTTON_OK, NULL ) );
		gnome_dialog_close_hides( dlg, TRUE );
		gnome_dialog_set_close( dlg, TRUE );
			
		gtk_box_pack_start( GTK_BOX( dlg->vbox ), GTK_WIDGET( b = GTK_BOX( gtk_hbox_new( FALSE, 0 ) ) ) , FALSE, FALSE, 5 );
		gtk_box_pack_start( b, gnome_pixmap_new_from_xpm_d( icon_xpm ), FALSE, FALSE, 0 );
		gtk_box_pack_start( b, gtk_vseparator_new(), FALSE, FALSE, 5 );
		
		gtk_box_pack_start( b, GTK_WIDGET( b2 = GTK_BOX( gtk_vbox_new( FALSE, 0 ) ) ), FALSE, FALSE, 5 );
		gtk_box_pack_start( b2, GTK_WIDGET( c = gtk_check_button_new_with_label( _( "Write Raw stream file" ) ) ), FALSE, FALSE, 0 );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ), raw_file );
		gtk_signal_connect_after( GTK_OBJECT( c ), "clicked", GTK_SIGNAL_FUNC( click_check_bp_callback ), (gpointer)( &raw_file ) );
		
		gtk_widget_show_all( dlg->vbox );
	}
	
	return( (gpointer)(dlg) );
}

/************************************************************************/
// start dowloading this job, return pid of child running it ( or 0 on error )
guint	plug_start_job( st_gsfdm_url_entry_shared *s, st_gsfdm_config_proxy *proxy )
{
st_gsfdm_job	*j = st_gsfdm_job_new();
pid_t			pid = 0, ppid = getpid();
int				fd[2];
	
	// prevent zombie
	signal( SIGCHLD, SIG_IGN );

	// NOTA : using popen would be a great idea. Infortunly, with this function
	// there no way ( i think ) to get the pid process owning the streamed data...
	// must use the boring-classic solution
	
	if( pipe( fd ) )
	{	g_warning( "error starting asf command... abort" );
		st_gsfdm_job_destroy( j );
		return( 0 );
	}
	
	if( !( pid = fork() ) )
	{	// child process
		static gchar	*list[30];					// must be enough
		guint			count = 0;
		
		// use one pipe from child ( asfrecord3 ) to parent ( plugin
		dup2( fd[ STDOUT_FILENO ], STDOUT_FILENO );
		close( fd[ STDIN_FILENO ] );

		// start the list of arg
		#ifdef PINGPONG_TEST
			list[ count++ ] = "pingpong";
			list[ count++ ] = "FILE";
			list[ count++ ] = "1500";
			list[ count++ ] = "1";
			list[ count++ ] = PACKAGE_SOURCE_DIR"/plugin/gsfdm_asf/pingpong.data";
		#else
			list[ count++ ] = SOFT;
		#endif
		
		
		// QUESTION HOW I SPECIFY USER AND PASS FOR PROXY ????
		// "-a" IS IT FOR PROXY OR FOR REMOTE AUTHORISATION ????
		if( gsfdm_proxy_st_get_use_proxy( proxy ) )
		{	list[ count++ ] =  "-P";
			list[ count++ ] = gsfdm_proxy_st_get_proxy_url( proxy );
			list[ count++ ] = "-a";
			list[ count++ ] = g_strconcat( gsfdm_proxy_st_get_user( proxy ), ":", gsfdm_proxy_st_get_pass( proxy ), NULL );
		}	
		
		// use steam raw file format
		if( raw_file )
		{	list[ count++ ] = "-r";
		}
		
		// destination name
		list[ count++ ] = "-f";
		list[ count++ ] = gsfdm_url_st_shared_get_name( s );

		// and source
		list[ count++ ] = gsfdm_url_st_shared_get_url( s );
		list[ count ] = NULL;
		
		// set to background directly, if gsfdm crash, still download...
		setpgid( 0, ppid );
		setsid();

		// disable local support, comme back to eng.
		putenv( "LC_ALL=C" );
		chdir( gsfdm_url_st_shared_get_path( s ) );
		
		// execute
		execvp( list[ 0 ], list );
		exit( -1 );							// exit on error
	}		
	
	// error in fork ???
	if( pid < 0 )
	{	g_warning( "error starting asf command... abort" );
		st_gsfdm_job_destroy( j );
		return( 0 );		
	}
	
	// parent process
	// out for child is read for parent...
	close( fd[ STDOUT_FILENO ] );
	st_gsfdm_job_get_read( j ) = fd[ STDIN_FILENO ];
	fcntl( st_gsfdm_job_get_read( j ), F_SETFL, O_NONBLOCK );
	st_gsfdm_job_get_write( j ) = 0;			// don't write to child, just read
	st_gsfdm_job_get_stream( j ) = fdopen( fd[ STDIN_FILENO ], "r" );	
	
//	setvbuf( st_gsfdm_job_get_stream( j ), stream_buffer, _IOLBF, BUFSIZ );

	// add now to list ( of running process )
	st_gsfdm_job_get_pid( j ) = pid;
	g_hash_table_insert( hash, (gpointer)pid, (gpointer)j );
	gsfdm_url_st_shared_get_pid( s ) = pid;
	gsfdm_url_st_shared_get_total( s ) = 0;			// asfrecord doesn't provide total size
	
	return( pid );
}

/************************************************************************/
// request abort downloading of this job - identify it with pid...
// TRUE if successful ; you have to take care of kill child pid by yourself
gboolean	plug_stop_job( st_gsfdm_url_entry_shared *s )
{
st_gsfdm_job	*j = ST_GSFDM_JOB( g_hash_table_lookup( hash, (gpointer)gsfdm_url_st_shared_get_pid( s ) ) );
gboolean		val = FALSE;			// unknow pid by default...

	if( j )
	{	// found, now stop
		if( !kill( st_gsfdm_job_get_pid( j ), 9 ) )
			val = TRUE;
		
		g_hash_table_remove( hash, (gpointer)st_gsfdm_job_get_pid( j ) );
		st_gsfdm_job_destroy( j );
	} 
	
	return( val );
}

/************************************************************************/
// update infos ( percent, size ) and return the state ( after update )
GsfdmUrlState plug_update_job( st_gsfdm_url_entry_shared *s )
{
st_gsfdm_job	*j = ST_GSFDM_JOB( g_hash_table_lookup( hash, (gpointer)gsfdm_url_st_shared_get_pid( s ) ) );
GsfdmUrlState	state = GSFDM_URL_STATE_RUNNING;	// if here, gsfdm ask update for - so far - a ruuning process
gint			answer;

	// test for stuct... no really the right state but it's the closer
	if( !j )
		return( GSFDM_URL_STATE_PARTIAL );	
	
//	clearerr( st_gsfdm_job_get_stream( j ) );
	// now read from stream, parse all line
	while( fgets( data, CHILD_BUFFER_LENGTH, st_gsfdm_job_get_stream( j ) ) )
	{	guint	a, c, d, e;
		gfloat	b;

		// data don't match ??? don't care of this message...
		#ifdef DEBUG_UPDATE
			g_print( "ASF PLUG DEBUG : state %d -- data || %s\n", state, data );
		#endif

		
		// get a line, parse
		// get size data
		 if( sscanf( data, "%u kB ( %f%%), HDR: $%x, part: $%x, %u", &a, &b, &c, &d, &e ) == 5 )
		 {	gsfdm_url_st_shared_get_size( s ) = gsfdm_url_st_shared_get_size( s ) + e;
			b /= 100.0;
			if( gsfdm_url_st_shared_get_percent( s ) < b )
				gsfdm_url_st_shared_get_percent( s ) = b;
			
			state |= GSFDM_URL_CHANGE_MASK;
			continue;
		 }
		
		// found host
		if( strstr( data, "connecting to: " ) )
		{	if( gsfdm_url_st_shared_get_host( s ) )
				g_free( gsfdm_url_st_shared_get_host( s ) );
			
			gsfdm_url_st_shared_get_host( s ) = g_strndup( &data[14], ( strlen( data ) - 15 ) );
			state |= GSFDM_URL_CHANGE_MASK;
			continue;
		}
	
		// get end of file : ok
		if( strstr( data, "Transfer complete." ) )
		{	plug_stop_job( s );					// clean data...
			return( GSFDM_URL_STATE_DONE | GSFDM_URL_CHANGE_MASK );
		}
		
		// get end before end : partial
		if( strstr( data, "Connection reset" ) ||
			strstr( data, "transfer cancelled." ) ||
			strstr( data, "maxtime reached." ) )
		{	plug_stop_job( s );					// clean data...
			return( GSFDM_URL_STATE_PARTIAL | GSFDM_URL_CHANGE_MASK );
		}
		
		// case where we can't get data
		if( strstr( data, "Authorization failed!" ) ||
			strstr( data, "Unable to capture this stream" ) ||
			strstr( data, "Unable to parse this ASF header!" ) ||
			strstr( data, "Authorization cancelled!" ) ||
			strstr( data, "Unable to resolve host" ) ||
 			strstr( data, "socket() failed" ) ||
			strstr( data, "connect() failed:" ) ||
			strstr( data, "Length confirmation doesn't match!" ) ||
			strstr( data, "Buffer too small. Chunk is" ) ||
			strstr( data, "Chunk has no body!" ) ||
			strstr( data, "Invalid URL format!" ) ||
			strstr( data, "Unable to resolve host" ) )
		{	plug_stop_job( s );					// clean data...
			return( GSFDM_URL_STATE_UNABLE | GSFDM_URL_CHANGE_MASK );
		}
		
		// now all errors
		if( strstr( data, "socket() failed" ) ||
			strstr( data, "connect() failed:" ) ||
			strstr( data, "reply: 404 - Not Found" ) ||
			strstr( data, "send() failed:" ) ||
			strstr( data, "Illegal server reply!" ) ||
			strstr( data, "Authorization cancelled!" ) ||
			strstr( data, "Authorization failed!" ) ||
			strstr( data, "Length confirmation doesn't match!" ) ||
			strstr( data, "Buffer too small. Chunk is" ) ||
			strstr( data, "unknown content-type:" ) ||
			strstr( data, "Chunk has no body!" ) )
		{	plug_stop_job( s );					// clean data...
			return( GSFDM_URL_STATE_DEAD | GSFDM_URL_CHANGE_MASK );
		}
		
		
		
		// data don't match ??? don't care of this message...
		#ifdef DEBUG_UPDATE
			g_print( "ASF PLUG DEBUG : state %d -- data || %s\n", state, data );
		#endif
	}
	
	// in case of segfault, kill, every else, getline return "EOF", ok for parsing, and
	// also set the error flag to true, MUST check every time whether this is 
	// really EOF or error
/*	if( !feof( st_gsfdm_job_get_stream( j ) ) )
	{	state = GSFDM_URL_STATE_PARTIAL;
		plug_stop_job( s );					// clean data...
		
		#ifdef DEBUG_UPDATE
		g_print( "ASF PLUG_DEBUG : STOP ON ERROR\n" );
		#endif
	}
*/	
	// test after : if data on buffer, get then before...
	if( waitpid( st_gsfdm_job_get_pid( j ), &answer, WUNTRACED | WNOHANG ) == -1 )
	{	plug_stop_job( s );					// clean data...
		
		if( state == GSFDM_URL_STATE_RUNNING )
			state = GSFDM_URL_STATE_PARTIAL	| GSFDM_URL_CHANGE_MASK;
		
		#ifdef DEBUG_UPDATE
		g_print( "ASF PLUG_DEBUG : STOP ON ERROR\n" );
		#endif
	}
	
	// perhaps there no new data available : this is not an error, just don't care...
	return( state );
}


/* exemple of a asx ( almost asf ) definition file
<ASX VERSION="3.0">
 <ENTRY>
  <REF HREF="mms://a992.v21895.c2189.g.vm.akamaistream.net/7/992/2189/v0001/zap2it.download.akamai.com/2189/archive/wb/0202/wb_angel_020218_hi.asf" />
 </ENTRY>
</ASX>
*/

// don't care of 'version field'
#define ASX_MAIN_TOKEN "(.*)<[[:blank:]]*ASX[^<>]*>(.*)<[[:blank:]]*/ASX[[:blank:]]*>[[:blank:]]*"
// do as could have several entry field.. perhaps...
#define ASX_ENTRY_TOKEN "(.*)<[[:blank:]]*ENTRY[[:blank:]]*>(.*)<[[:blank:]]*/ENTRY[[:blank:]]*>[[:blank:]]*"
// do as could have several ref on each entry
#define ASX_REF_TOKEN "(.*)<[[:blank:]]*REF[[:blank:]]+HREF[[:blank:]]*=[[:blank:]]*\"[[:blank:]]*(.*)\"[[:blank:]]*/[[:blank:]]*>[[:blank:]]*"

/************************************************************************/
// when loading from disk, gsfdm request the plug to parse the
// description file ( like m3u, asx... also html)
// return true on success
// read data fron input stream
// write '\n' ending string for each url to add
// string into output ( flush after each write ), 
// s : struct that own the data or NULL if unavailable
// extension of the file to parse if you need it...
gboolean	plug_parse_file( FILE *input, FILE *output, st_gsfdm_url_entry_shared const *s, gchar const *ext )
{
	// the job is a simple reg exp matching. though the job is really
	// simple, i could have used few "while" and "scanf" to match
	// data; result would have been same but with regexp function, it's 
	// faster ( it is ?? ) and funnier...
static regex_t		*main_tag = NULL, *entry_tag, *ref_tag;
static regmatch_t	main_m[4], entry_m[4], ref_m[4];
gchar				*str, *entry_str, *work_str;
gboolean			state = FALSE;

	#ifdef DEBUG_PARSE_DATA_FILE
	g_print( "ASF - start parsing\n" );
	#endif

	if( !input || !output )
		return( FALSE );	

	// first, compile... NOTA : don't perform test on compilation, no reason for error...
	if( !main_tag )
	{	main_tag = g_new0( regex_t, 1 );
		regcomp( main_tag, ASX_MAIN_TOKEN, REG_ICASE | REG_EXTENDED );
		entry_tag = g_new0( regex_t, 1 );
		regcomp( entry_tag, ASX_ENTRY_TOKEN, REG_ICASE | REG_EXTENDED );
		ref_tag = g_new0( regex_t, 1 );
		regcomp( ref_tag, ASX_REF_TOKEN, REG_ICASE | REG_EXTENDED );
	}	

	// load all data...
	{ size_t s;
	
		fseek( input, 0, SEEK_END ); 
		str = g_new0( gchar, ( s = ftell( input ) ) + 1 );
		fseek( input, 0, SEEK_SET );
		fread( str, sizeof( gchar ), s, input );
	}
	
	while( !regexec( main_tag, str, 4, main_m, 0 ) )
	{	 entry_str = g_strndup( str + main_m[2].rm_so, main_m[2].rm_eo - main_m[2].rm_so );

		while( !regexec( entry_tag, entry_str, 4, entry_m, 0 ) )
		{	work_str = g_strndup( entry_str + entry_m[2].rm_so, entry_m[2].rm_eo - entry_m[2].rm_so );

			while( !regexec( ref_tag, work_str, 4, ref_m, 0 ) )
			{	
				#ifdef DEBUG_PARSE_DATA_FILE
				g_print( "ASF - parse URL :: %s\n", g_strndup( work_str + ref_m[2].rm_so, ref_m[2].rm_eo - ref_m[2].rm_so ) );
				#endif
								
				fwrite( work_str + ref_m[2].rm_so, sizeof( gchar ), ref_m[2].rm_eo - ref_m[2].rm_so, output );
				fputc( '\n', output );

				work_str[ ref_m[1].rm_eo + 1 ] = 0;
				state = TRUE;
			}
			
			g_free( work_str );
			entry_str[ entry_m[1].rm_eo + 1 ] = 0;			
		}
		g_free( entry_str );
		str[ main_m[1].rm_eo + 1 ] = 0;		
	}

	fputc( 0, output );
	fflush( output );
	g_free( str );
	return( state );
}

