/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		Header http plug

// ok, this is almost the same code as http plugin

/************************************************************************/

#include	<gnome.h>
#include	<glib.h>
#include	<stdio.h>
#include	<signal.h>
#include	<fcntl.h>
#include	<sys/wait.h>
#include	"./gsfdm_plugin_shared.h"
#include	"./header_skeleton.h"

#include	"../../config.h"

/************************************************************************/
//	debug switch
/************************************************************************/
#ifdef HAVE_DEBUG
#define	DEBUG_UPDATE
#endif

/************************************************************************/
//	structure and define definition
/************************************************************************/
#include "icon.xpm"

#define SOFT			"wget"
#define PLUGIN_NAME		"Gsfdm_FTP_Downloader"
#define VERSION			"0.2"
#define GSFDM_VERSION	"0.2"
#define COMMENT			"Gsfdm Plug-in, for ftp connection"
static gchar const		*PROTOCOL[] = { "ftp:", "ftps:", NULL };
static gchar const		*EXTENSION[] = { NULL };
	
static gchar const	*AUTHORS[] = { "J.A.L @ Free.fr", NULL };

#define CHILD_BUFFER_LENGTH 500			// length of a line... must be ok !
static gchar			data[ CHILD_BUFFER_LENGTH ];

static gboolean		plug_initialized = FALSE;
static GHashTable	*hash = NULL;

static gboolean		passive_ftp = TRUE;
static gboolean		use_globbing = FALSE;

/************************************************************************/
//	functions definition
/************************************************************************/
//	first call ( after loading through g_module_open and symbols resolving )
// here you do things the plug need at startup
// return TRUE on success
gboolean	plug_init( void )
{
	// only need a table to stock open descriptor to child
	if( !plug_initialized )
	{	// look if found the soft
		
		
		// create the hash for open job, key is pid
		if( hash ) g_hash_table_destroy( hash );
			
		if( ( hash = g_hash_table_new( g_direct_hash, g_direct_equal ) ) )
			plug_initialized = TRUE;
		
		// load config switch
		gnome_config_push_prefix( "/"PLUGIN_NAME"/std/" );
		passive_ftp = gnome_config_get_bool_with_default( "passive_ftp=TRUE", NULL );
		use_globbing = gnome_config_get_bool_with_default( "use_globbing=FALSE", NULL );
		gnome_config_pop_prefix();	
	}
	
	return( plug_initialized );
}

/************************************************************************/
// need more explaination ???
// if safe=TRUE; ask plugin, if could,  not stop current work : push it at backgroud
gboolean	plug_stop( gboolean safe )
{
	static void safe_mode( gpointer key, gpointer value, gpointer data )
	{
		// may i connect  back to stdin and stdout ???
		st_gsfdm_job_destroy( value );		
	}
	
	static void kill_mode( gpointer key, gpointer value, gpointer data )
	{
		kill( st_gsfdm_job_get_pid( value ), 9 );
		st_gsfdm_job_destroy( value );
	}
	
	/************************************************************************/
	// kill 
	if( safe )
	{	g_hash_table_foreach( hash, (GHFunc)safe_mode, NULL );
	} else {
		g_hash_table_foreach( hash, (GHFunc)kill_mode, NULL );
	}

	if( hash )
		g_hash_table_destroy( hash );
	
	plug_initialized = FALSE;
	
	// save config
	gnome_config_push_prefix( "/"PLUGIN_NAME"/std/" );
	gnome_config_set_bool( "passive_ftp", passive_ftp );
	gnome_config_set_bool( "use_globbing", use_globbing );
	gnome_config_pop_prefix();	
	gnome_config_sync();

	return( TRUE );
}

/************************************************************************/
// return a static string for the full plug name, used at display
gchar const *plug_get_name( void )
{	return( PLUGIN_NAME );
}

/************************************************************************/
// version number ; typ return( VERSION );
gchar const	*plug_get_version( void )
{	return( VERSION );
}

/************************************************************************/
// return array to const gchar * of all authors names
gchar const	**plug_get_author( void )
{	return( AUTHORS );
}

/************************************************************************/
// data for and icon or ( NULL if nome ), this is the variable 
// created when you included an XPM icon
gchar const	**plug_get_icon( void )
{	return( ( gchar const **)icon_xpm );
}

/************************************************************************/
// return version of gsfdm this plug was created for ;
// perhaps usefull further if the interface changed...
gchar const *plug_get_gsfdm_version( void )
{	return( GSFDM_VERSION );
}

/************************************************************************/
// a comment ; description of plug
gchar const *plug_get_comment( void )
{	return( COMMENT );
}

/************************************************************************/
// return null ending array of supported protocol [] = { "http:", "https:", NULL };
gchar const	**plug_get_protocol_list( void )
{	return( PROTOCOL );
}

/************************************************************************/
// this is the list of HD FILE the plug is able to parse.
// exemple : .asf that enclose stream info
// null if none
gchar const	**plug_get_ext_list( void )
{	return( EXTENSION );
}

/************************************************************************/
// TRUE if provide a gnome dlg for configuration
gboolean	plug_have_gnome_config( void )
{	return( TRUE );
}

/************************************************************************/
static void	click_check_bp_callback( GtkToggleButton *c, gpointer d )
{
	*((gboolean *)d) = gtk_toggle_button_get_active( c );
}

/************************************************************************/
// return the top widget of the dlg in a form of gpointer
// this is for compatibility if no gnome support
gpointer	plug_get_gnome_dlg( void )
{	
GnomeDialog	*dlg = NULL;
GtkBox		*b, *b2;
GtkWidget	*c;
	
	#ifdef DEBUG_GSFDM_PLUGIN_TEST_CONFIG
	g_print( DEBUG_ID"opening config dlg" );
	#endif
	//	create the dialog at first call, keep it through the plugin life
	// this is useful if dlg is big and you want to keep 
	// of course could use a global static instead local, but in this case...
	if( !dlg )
	{	dlg = GNOME_DIALOG( gnome_dialog_new( _( "GSFDM Http Configuration" ), GNOME_STOCK_BUTTON_OK, NULL ) );
		gnome_dialog_close_hides( dlg, TRUE );
		gnome_dialog_set_close( dlg, TRUE );
			
		gtk_box_pack_start( GTK_BOX( dlg->vbox ), GTK_WIDGET( b = GTK_BOX( gtk_hbox_new( FALSE, 0 ) ) ) , FALSE, FALSE, 5 );
		gtk_box_pack_start( b, gnome_pixmap_new_from_xpm_d( icon_xpm ), FALSE, FALSE, 0 );
		gtk_box_pack_start( b, gtk_vseparator_new(), FALSE, FALSE, 5 );
		
		gtk_box_pack_start( b, GTK_WIDGET( b2 = GTK_BOX( gtk_vbox_new( FALSE, 0 ) ) ), FALSE, FALSE, 5 );
		gtk_box_pack_start( b2, GTK_WIDGET( c = gtk_check_button_new_with_label( _( "Use passive connection" ) ) ), FALSE, FALSE, 0 );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ), passive_ftp );
		gtk_signal_connect_after( GTK_OBJECT( c ), "clicked", GTK_SIGNAL_FUNC( click_check_bp_callback ), (gpointer)( &passive_ftp ) );
		gtk_box_pack_start( b2, GTK_WIDGET( c = gtk_check_button_new_with_label( _( "Turn On name globbing" ) ) ), FALSE, FALSE, 0 );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ), use_globbing );
		gtk_signal_connect_after( GTK_OBJECT( c ), "clicked", GTK_SIGNAL_FUNC( click_check_bp_callback ), (gpointer)( &use_globbing ) );

		gtk_widget_show_all( dlg->vbox );
	}
	
	return( (gpointer)(dlg) );
}

/************************************************************************/
// start dowloading this job, return pid of child running it ( or 0 on error )
guint	plug_start_job( st_gsfdm_url_entry_shared *s, st_gsfdm_config_proxy *proxy )
{
st_gsfdm_job	*j = st_gsfdm_job_new();
pid_t			pid = 0, ppid = getpid();
int				fd[2];
	
	// prevent zombie
	signal( SIGCHLD, SIG_IGN );

	// NOTA : using popen would be a great idea. Infortunly, with this function
	// there no way ( i think ) to get the pid process owning the streamed data...
	// must use the boring-classic solution
	
	if( pipe( fd ) )
	{	g_warning( "error starting http command... abort" );
		st_gsfdm_job_destroy( j );
		return( 0 );
	}
	
	if( !( pid = fork() ) )
	{	// child process
		gchar	*list[30];					// must be enough
		guint	count = 0;
		
		#ifdef PINGPONG_TEST
		dup2( fd[ STDOUT_FILENO ], STDOUT_FILENO );
		#else
		dup2( fd[ STDOUT_FILENO ], STDERR_FILENO );
		#endif
		close( fd[ STDIN_FILENO ] );

		// start the list of arg
		#ifdef PINGPONG_TEST
			list[ count++ ] = "pingpong";
			list[ count++ ] = "FILE";
			list[ count++ ] = "1500";
			list[ count++ ] = "1";
			list[ count++ ] = PACKAGE_SOURCE_DIR"/plugin/gsfdm_http/pingpong.data";
		#else
			list[ count++ ] = SOFT;
		#endif

		list[ count++ ] = "-v";
		list[ count++ ] = "-t";
		list[ count++ ] = "1";
				
		if( gsfdm_proxy_st_get_use_proxy( proxy ) )
		{	list[ count++ ] =  "-Y";
			list[ count++ ] =  "on";
			list[ count++ ] = g_strconcat( "http_proxy=", gsfdm_proxy_st_get_proxy_url( proxy ), NULL );
			list[ count++ ] = g_strconcat( "--proxy-user=", gsfdm_proxy_st_get_user( proxy ), NULL );
			list[ count++ ] = g_strconcat( "--proxy-passwd=", gsfdm_proxy_st_get_pass( proxy ), NULL );
		}	
		
		if( passive_ftp )
			list[ count++ ] = "--passive-ftp";
		
		if( use_globbing )
		{	list[ count++ ] = "-g";
			list[ count++ ] = "on";
		}
		
		// destination name
		list[ count++ ] = "-O";
		list[ count++ ] = gsfdm_url_st_shared_get_name( s );

		// and source
		list[ count++ ] = gsfdm_url_st_shared_get_url( s );
		list[ count ] = NULL;

		// set to background directly, if gsfdm crash, still download...
		setpgid( 0, ppid );
		setsid();
		
		// disable local support, comme back to eng.
		putenv( "LC_ALL=C" );
		chdir( gsfdm_url_st_shared_get_path( s ) );

		// execute
		execvp( list[ 0 ], list );
		exit( -1 );							// exit on error
	}		
	
	// error in fork ???
	if( pid < 0 )
	{	g_warning( "error starting http command... abort" );
		st_gsfdm_job_destroy( j );
		return( 0 );		
	}

	// parent process
	// out for child is read for parent...
	close( fd[ STDOUT_FILENO ] );
	st_gsfdm_job_get_read( j ) = fd[ STDIN_FILENO ];
	fcntl( st_gsfdm_job_get_read( j ), F_SETFL, O_NONBLOCK );
	st_gsfdm_job_get_write( j ) = 0;			// don't write to child, just read
	st_gsfdm_job_get_stream( j ) = fdopen( fd[ STDIN_FILENO ], "r" );	
	
//	setvbuf( st_gsfdm_job_get_stream( j ), stream_buffer, _IOLBF, BUFSIZ );

	// add now to list ( of running process )
	st_gsfdm_job_get_pid( j ) = pid;
	g_hash_table_insert( hash, (gpointer)pid, (gpointer)j );
	gsfdm_url_st_shared_get_pid( s ) = pid;
	gsfdm_url_st_shared_get_percent( s ) = 0.0;

	return( pid );
}

/************************************************************************/
// request abort downloading of this job - identify it with pid...
// TRUE if successful ; you have to take care of kill child pid by yourself
gboolean	plug_stop_job( st_gsfdm_url_entry_shared *s )
{
st_gsfdm_job	*j = ST_GSFDM_JOB( g_hash_table_lookup( hash, (gpointer)gsfdm_url_st_shared_get_pid( s ) ) );
gboolean		val = FALSE;			// unknow pid by default...

	if( j )
	{	// found, now stop
		if( !kill( st_gsfdm_job_get_pid( j ), 9 ) )
			val = TRUE;

		g_hash_table_remove( hash, (gpointer)st_gsfdm_job_get_pid( j ) );
		st_gsfdm_job_destroy( j );
	} 
	
	return( val );
}

#define BUFFER_SIZE 1000				// length of a line... must be ok !

/************************************************************************/
// update infos ( percent, size ) and return the state ( after update )
GsfdmUrlState plug_update_job( st_gsfdm_url_entry_shared *s )
{
st_gsfdm_job	*j = ST_GSFDM_JOB( g_hash_table_lookup( hash, (gpointer)gsfdm_url_st_shared_get_pid( s ) ) );
GsfdmUrlState	state = GSFDM_URL_STATE_RUNNING;	// if here, gsfdm ask update for - so far - a ruuning process
gint			answer;

	// test for stuct... no really the right state but it's the closer
	if( !j )
		return( GSFDM_URL_STATE_PARTIAL );	
	
//	clearerr( st_gsfdm_job_get_stream( j ) );
	// now read from stream, parse all line
	while( fgets( data, CHILD_BUFFER_LENGTH, st_gsfdm_job_get_stream( j ) ) )
	{	guint	a, f;
		gfloat	c;
		gchar	b[60], d[10], e[25], *i;

		// data don't match ??? don't care of this message...
		#ifdef DEBUG_UPDATE
			g_print( "HTTP PLUG DEBUG : state %d -- data || %s\n", state, data );
		#endif

		// get a line, parse
		// get size data, there 3 solutions that match a size info
		// size + count
		if( sscanf( data, "%dK ->%60[. ]", &a, &b ) == 2 )
		{	for( i = b ; *i != '0' ; i++ )
			{	if( *i == '.' )
					a++;
			}
			
			gsfdm_url_st_shared_get_size( s ) = a * 1024;

			if( gsfdm_url_st_shared_get_total( s ) )
			{	gsfdm_url_st_shared_get_percent( s ) = 
					(gfloat)gsfdm_url_st_shared_get_size( s ) / (gfloat)gsfdm_url_st_shared_get_total( s );
			}

			state |= GSFDM_URL_CHANGE_MASK;
			continue;
		}
		
		if( sscanf( data, "%60[. ]", &b ) == 1 )
		{	for( i = b ; *i != '0' ; i++ )
			{	if( *i == '.' )
					gsfdm_url_st_shared_get_size( s ) += 1024;
			}
			
			if( gsfdm_url_st_shared_get_total( s ) )
			{	gsfdm_url_st_shared_get_percent( s ) = 
					(gfloat)gsfdm_url_st_shared_get_size( s ) / (gfloat)gsfdm_url_st_shared_get_total( s );
			}

			state |= GSFDM_URL_CHANGE_MASK;
			continue;
		}
		
		if( sscanf( data, "%60[. ] [ %d%%]", &b, &a ) == 2 )
		 {	for( i = b ; *i != '0' ; i++ )
			{	if( *i == '.' )
					gsfdm_url_st_shared_get_size( s ) += 1024;
			}

			gsfdm_url_st_shared_get_percent( s ) = (gfloat)a / 100.0;
			state |= GSFDM_URL_CHANGE_MASK;
			continue;
		 }

		// found the file size information, use it to create
		// a ratio, used to update 'percent' at every data reception
		// nota : parse only file greaser than 1Kb, if lower, don't
		// case : file will end before any update...
		if( !strncmp( data, "Length: ", 8 ) )
		{	gchar	*ii;
			// wget write size in 3 digits blocks
			ii = data+8;
			if( ( i = strchr( ii, '[' ) ) )
				*i = 0;
			
			st_gsfdm_job_get_data( j ) = 0;
			while( ( a = strtol( ii, &i, 10  ) ), i != ii )
			{	gsfdm_url_st_shared_get_total( s ) *= 1000;
				gsfdm_url_st_shared_get_total( s ) += a;
				ii = i+1;
			}
			
			state |= GSFDM_URL_CHANGE_MASK;
			continue;		 			
		}
		 
		 // get end of file : ok
		if( strstr( data, "FINISHED" ) ||
			( sscanf( data, "%8c (%f KB/s) - `%60[^'`]' saved %25c", &d, &c, &b, &e ) == 4 ) )
		{	plug_stop_job( s );					// clean data...
			gsfdm_url_st_shared_get_percent( s ) = 1.0;
			return( GSFDM_URL_STATE_DONE | GSFDM_URL_CHANGE_MASK );
		}
		
		// found host
		if( sscanf( data, "Connecting to %s... connected!", &b ) == 1 )
		{	if( gsfdm_url_st_shared_get_host( s ) )
				g_free( gsfdm_url_st_shared_get_host( s ) );
			
			gsfdm_url_st_shared_get_host( s ) = g_strdup( b );
			state |= GSFDM_URL_CHANGE_MASK;
			continue;
		}
	
		// get end before end : partial
		if( strstr( data, "Data transfer aborted" ) ||
			strstr( data, "Giving up" ) )
		{	plug_stop_job( s );					// clean data...
			return( GSFDM_URL_STATE_PARTIAL | GSFDM_URL_CHANGE_MASK );
		}
		
		// case where we can't get data
		if( strstr( data, "Error in Set-Cookie, field" ) ||
			strstr( data, "Syntax error in Set-Cookie" ) ||
			strstr( data, "Cannot open cookies" ) ||
			strstr( data, "Error writing to" ) ||
			strstr( data, "Error closing" ) ||
			strstr( data, "Unsupported listing type" ) ||
			strstr( data, "Error in server" ) ||
			strstr( data, "Login incorrect" ) ||
			strstr( data, "(unauthoritative)" ) ||
			strstr( data, "Authorization failed" ) ||
			strstr( data, "ERROR 403: Forbidden." )	)
		{	plug_stop_job( s );					// clean data...
			return( GSFDM_URL_STATE_UNABLE | GSFDM_URL_CHANGE_MASK );
		}
		
		// now all errors
		if( strstr( data, "HTTP request sent, awaiting response... 404 Not Found" ) ||
			strstr( data, "couldn't connect to" ) ||
			strstr( data, "Server error" ) ||
			strstr( data, "No such file" ) ||
			strstr( data, "Host not found" ) ||
			strstr( data, "No data received" ) ||
			strstr( data, "Unknown authentication scheme" )
		)
		{	plug_stop_job( s );					// clean data...
			return( GSFDM_URL_STATE_DEAD | GSFDM_URL_CHANGE_MASK );
		}
	
			
		// data don't match ??? don't care of this message...
	}
	
	// test after : if data on buffer, get then before...
	if( waitpid( st_gsfdm_job_get_pid( j ), &answer, WUNTRACED | WNOHANG ) == -1 )
	{	plug_stop_job( s );					// clean data...
		
		if( state == GSFDM_URL_STATE_RUNNING )
			state = GSFDM_URL_STATE_PARTIAL | GSFDM_URL_CHANGE_MASK;
		
		#ifdef DEBUG_UPDATE
		g_print( "HTTP PLUG_DEBUG : STOP ON ERROR\n" );
		#endif
	}
	
	// perhaps there no new data available : this is not an error, just don't care...
	return( state );
}
	
/************************************************************************/
// when loading from disk, gsfdm request the plug to parse the
// description file ( like m3u, asx... also html)
// return true on success
// read data fron input stream
// write '\n' ending string for each url to add
// string into output ( flush after each write ), 
// s : struct that own the data or NULL if unavailable
// extension of the file to parse if you need it...
gboolean	plug_parse_file( FILE *input, FILE *output, st_gsfdm_url_entry_shared const *s, gchar const *ext )
{
	
	
	return( FALSE );
}