#include "GSFDM.h"
#include	"gsfdm_config.h"

/*** App-specific servant structures ***/
typedef struct
{
   POA_GSFDM_Url servant;
   PortableServer_POA poa;

}
impl_POA_GSFDM_Url;

typedef struct
{
   POA_GSFDM_Action servant;
   PortableServer_POA poa;

}
impl_POA_GSFDM_Action;

/*** Implementation stub prototypes ***/
static void impl_GSFDM_Url__destroy(impl_POA_GSFDM_Url * servant,
				    CORBA_Environment * ev);

static CORBA_boolean
impl_GSFDM_Url_add_url(impl_POA_GSFDM_Url * servant,
		       CORBA_char * url, CORBA_Environment * ev);

static CORBA_boolean
impl_GSFDM_Url_add_url_full(impl_POA_GSFDM_Url * servant,
			    CORBA_char * url,
			    CORBA_boolean auto_dl,
			    CORBA_boolean auto_rn,
			    CORBA_boolean dlg, CORBA_Environment * ev);

static CORBA_boolean
impl_GSFDM_Url_remove_url(impl_POA_GSFDM_Url * servant,
			  CORBA_char * url,
			  CORBA_boolean force, CORBA_Environment * ev);

static void
impl_GSFDM_Url_stop_url(impl_POA_GSFDM_Url * servant,
			CORBA_char * url, CORBA_Environment * ev);

static CORBA_short
impl_GSFDM_Url_copy_to_clipboard(impl_POA_GSFDM_Url * servant,
				 CORBA_Environment * ev);

static CORBA_short
impl_GSFDM_Url_paste_from_clipboard(impl_POA_GSFDM_Url * servant,
				    CORBA_boolean auto_dl,
				    CORBA_boolean dlg,

				    CORBA_Environment * ev);

static void impl_GSFDM_Action__destroy(impl_POA_GSFDM_Action * servant,
				       CORBA_Environment * ev);

static GSFDM_Action_return_value
impl_GSFDM_Action_exit(impl_POA_GSFDM_Action * servant,
		       GSFDM_Action_exit_value mode,
		       CORBA_boolean current_list, CORBA_Environment * ev);

/*** epv structures ***/
static PortableServer_ServantBase__epv impl_GSFDM_Url_base_epv = {
   NULL,			/* _private data */
   NULL,			/* finalize routine */
   NULL,			/* default_POA routine */
};
static POA_GSFDM_Url__epv impl_GSFDM_Url_epv = {
   NULL,			/* _private */

   (gpointer) & impl_GSFDM_Url_add_url,

   (gpointer) & impl_GSFDM_Url_add_url_full,

   (gpointer) & impl_GSFDM_Url_remove_url,

   (gpointer) & impl_GSFDM_Url_stop_url,

   (gpointer) & impl_GSFDM_Url_copy_to_clipboard,

   (gpointer) & impl_GSFDM_Url_paste_from_clipboard,

};
static PortableServer_ServantBase__epv impl_GSFDM_Action_base_epv = {
   NULL,			/* _private data */
   NULL,			/* finalize routine */
   NULL,			/* default_POA routine */
};
static POA_GSFDM_Action__epv impl_GSFDM_Action_epv = {
   NULL,			/* _private */

   (gpointer) & impl_GSFDM_Action_exit,

};

/*** vepv structures ***/
static POA_GSFDM_Url__vepv impl_GSFDM_Url_vepv = {
   &impl_GSFDM_Url_base_epv,
   &impl_GSFDM_Url_epv,
};
static POA_GSFDM_Action__vepv impl_GSFDM_Action_vepv = {
   &impl_GSFDM_Action_base_epv,
   &impl_GSFDM_Action_epv,
};

/*** Stub implementations ***/
GSFDM_Url impl_GSFDM_Url__create(PortableServer_POA poa, CORBA_Environment * ev)
{
   GSFDM_Url retval;
   impl_POA_GSFDM_Url *newservant;
   PortableServer_ObjectId *objid;

   newservant = g_new0(impl_POA_GSFDM_Url, 1);
   newservant->servant.vepv = &impl_GSFDM_Url_vepv;
   newservant->poa = poa;
   POA_GSFDM_Url__init((PortableServer_Servant) newservant, ev);
   objid = PortableServer_POA_activate_object(poa, newservant, ev);
   CORBA_free(objid);
   retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

   return retval;
}

static void
impl_GSFDM_Url__destroy(impl_POA_GSFDM_Url * servant, CORBA_Environment * ev)
{
   PortableServer_ObjectId *objid;

   objid = PortableServer_POA_servant_to_id(servant->poa, servant, ev);
   PortableServer_POA_deactivate_object(servant->poa, objid, ev);
   CORBA_free(objid);

   POA_GSFDM_Url__fini((PortableServer_Servant) servant, ev);
   g_free(servant);
}

/************************************************************************/
static CORBA_boolean impl_GSFDM_Url_add_url(impl_POA_GSFDM_Url * servant,
		       CORBA_char * url, CORBA_Environment * ev)
{
CORBA_boolean retval = TRUE;

	if( gsfdm_config_open_popup_on_dnd() )
	{	// this function don't ( and can't ) return a value
		gsfdm_url_popup_url( url );
	} else {
		retval = gsfdm_url_push_url( url, 
			gsfdm_config_get_auto_download(), gsfdm_config_get_auto_rename() );
		
		if( !retval )
		{	// will add several types of error, for now, just "error"
			GSFDM_Url_error	*exception;
			exception = GSFDM_Url_error__alloc();
			
			strcpy( exception->why, "error" );
			CORBA_exception_set( &ev, CORBA_USER_EXCEPTION, ex_GSFDM_Url_error, (void *)exception );
		}
	}

return( retval );	
}

/************************************************************************/
static CORBA_boolean impl_GSFDM_Url_add_url_full(impl_POA_GSFDM_Url * servant,
	CORBA_char * url, CORBA_boolean auto_dl, CORBA_boolean auto_rn,
	CORBA_boolean dlg, CORBA_Environment * ev)
{
CORBA_boolean retval = TRUE;

	if( dlg )
	{	// this function don't ( and can't ) return a value
		gsfdm_url_popup_url( url );
	} else {
		retval = gsfdm_url_push_url( url, auto_dl, auto_rn );
		
		if( !retval )
		{	// will add several types of error, for now, just "error"
			GSFDM_Url_error	*exception;
			exception = GSFDM_Url_error__alloc();
			
			strcpy( exception->why, "error" );
			CORBA_exception_set( &ev, CORBA_USER_EXCEPTION, ex_GSFDM_Url_error, (void *)exception );
		}
	}

return( retval );	
}

/************************************************************************/
static CORBA_boolean impl_GSFDM_Url_remove_url(impl_POA_GSFDM_Url * servant, CORBA_char * url,
	CORBA_boolean force, CORBA_Environment * ev)
{
CORBA_boolean retval;
	
	// nothing more...
	retval = gsfdm_url_remove( url, force );
	
	if( !retval )
	{	GSFDM_Url_error	*exception;
		exception = GSFDM_Url_error__alloc();
			
		strcpy( exception->why, "unknow url" );
		CORBA_exception_set( &ev, CORBA_USER_EXCEPTION, ex_GSFDM_Url_error, (void *)exception );
	}
	
return( retval );
}

/************************************************************************/
static void impl_GSFDM_Url_stop_url(impl_POA_GSFDM_Url * servant,
	CORBA_char * url, CORBA_Environment * ev)
{
CORBA_boolean retval;
	
	retval = gsfdm_url_stop_url( url );
	
	if( !retval )
	{	GSFDM_Url_error	*exception;
		exception = GSFDM_Url_error__alloc();
			
		strcpy( exception->why, "error" );
		CORBA_exception_set( &ev, CORBA_USER_EXCEPTION, ex_GSFDM_Url_error, (void *)exception );
	}

	return( retval );

}

/************************************************************************/
static CORBA_short impl_GSFDM_Url_copy_to_clipboard(impl_POA_GSFDM_Url * servant, CORBA_Environment * ev)
{
   CORBA_short retval;

	
	
	
	
	
	
	
   return retval;
}

/************************************************************************/
static CORBA_short impl_GSFDM_Url_paste_from_clipboard(impl_POA_GSFDM_Url * servant,
	CORBA_boolean auto_dl, CORBA_boolean dlg, CORBA_Environment * ev)
{
   CORBA_short retval;

	
	
	
	
	
   return retval;
}

/************************************************************************/
GSFDM_Action impl_GSFDM_Action__create(PortableServer_POA poa, CORBA_Environment * ev)
{
   GSFDM_Action retval;
   impl_POA_GSFDM_Action *newservant;
   PortableServer_ObjectId *objid;

   newservant = g_new0(impl_POA_GSFDM_Action, 1);
   newservant->servant.vepv = &impl_GSFDM_Action_vepv;
   newservant->poa = poa;
   POA_GSFDM_Action__init((PortableServer_Servant) newservant, ev);
   objid = PortableServer_POA_activate_object(poa, newservant, ev);
   CORBA_free(objid);
   retval = PortableServer_POA_servant_to_reference(poa, newservant, ev);

   return retval;
}

static void
impl_GSFDM_Action__destroy(impl_POA_GSFDM_Action * servant,
			   CORBA_Environment * ev)
{
   PortableServer_ObjectId *objid;

   objid = PortableServer_POA_servant_to_id(servant->poa, servant, ev);
   PortableServer_POA_deactivate_object(servant->poa, objid, ev);
   CORBA_free(objid);

   POA_GSFDM_Action__fini((PortableServer_Servant) servant, ev);
   g_free(servant);
}

/************************************************************************/
static GSFDM_Action_return_value impl_GSFDM_Action_exit(impl_POA_GSFDM_Action * servant,
	GSFDM_Action_exit_value mode, CORBA_boolean current_list, CORBA_Environment * ev)
{
   GSFDM_Action_return_value retval;

	
	
	
	
	
   return retval;
}
