/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//			Code for all configuration operations ( dlg, load, save... )

/************************************************************************/

#include	<gnome.h>
#include	<config.h>
#include	"gsfdm_config.h"
#include	"gsfdm_log.h"
#include	"gsfdm_historic.h"
#include	"gsfdm_url.h"

/************************************************************************/
//	debug
/************************************************************************/
#ifdef HAVE_DEBUG
#define	DEBUG_CONFIG_CREATE_STD_BOX
#define	DEBUG_CONFIG_CREATE_DOWNLOAD_BOX
#endif

/************************************************************************/
//	static variable definition
/************************************************************************/
// var for main op
static gboolean		main_warm_empty_job_stack = TRUE;
static gboolean		main_prompt_before_removing = FALSE;
static gboolean		main_auto_remove_when_done = FALSE;
static gboolean		main_open_popup_on_dnd = FALSE;
static guint		main_update_time = 1;
static gchar		*main_pppd = "ppp0";

static gboolean		url_auto_download_by_default = TRUE;
static gboolean		url_auto_rename_by_default = TRUE;
static guint		url_max_simultaneous = 5;
static guint		url_timeout = 180;
static gchar		*url_save_dir = NULL;
static gboolean		url_auto_parse = TRUE;

st_gsfdm_config_proxy	*proxy_data = NULL;

#include	"pixmaps/running_low.xpm"
#include	"pixmaps/conf_low.xpm"
#include	"pixmaps/log_low.xpm"
#include	"pixmaps/hist_low.xpm"
#include	"pixmaps/proxy_low.xpm"

enum {
	PAGE_STD = 0,
	PAGE_DOWNLOAD = 1,
	PAGE_LOG = 2,
	PAGE_HIST = 3,
	PAGE_PROXY = 4
};

/************************************************************************/
//	functions definition
/************************************************************************/
static GtkWidget	*gsfdm_config_std_get_page( void );
static GtkWidget	*gsfdm_config_download_get_page( void );
static GtkWidget	*gsfdm_config_proxy_get_page( void );

// just need to inform directly of some change ( timer.. )
static void			exit_callback( GnomePropertyBox *dlg );

static void			dialog_help_callback( GnomePropertyBox *property_box, gint page_num );

/************************************************************************/
void st_gsfdm_config_proxy_destroy( st_gsfdm_config_proxy *s )
{
	if( gsfdm_proxy_st_get_user( s ) )
		g_free( gsfdm_proxy_st_get_user( s ) );
	if( gsfdm_proxy_st_get_pass( s ) )
		g_free( gsfdm_proxy_st_get_pass( s ) );
	if( gsfdm_proxy_st_get_proxy_url( s ) )
		g_free( gsfdm_proxy_st_get_proxy_url( s ) );
	
	g_free( s );	
}

/************************************************************************/
// just need to inform directly of some change ( timer.. )
/************************************************************************/
static void	exit_callback( GnomePropertyBox *dlg )
{	gsfdm_url_set_update_time( main_update_time );
	gtk_widget_hide( GTK_WIDGET( dlg ) );
}

/************************************************************************/
// open help on this on current page
/************************************************************************/
static void	dialog_help_callback( GnomePropertyBox *property_box, gint page_num )
{
static GnomeHelpMenuEntry help_page = { PACKAGE, NULL };
        
	switch( page_num )
	{	case PAGE_STD :
			help_page.path = "configuration.html#standard";
			break;
		case PAGE_DOWNLOAD :
			help_page.path = "configuration.html#download";
			break;
		case PAGE_LOG :
			help_page.path = "configuration.html#log";
			break;
		case PAGE_HIST :
			help_page.path = "configuration.html#historic";
			break;
		case PAGE_PROXY :
			help_page.path = "configuration.html#proxy";
			break;
		default :
			return;
	}
	
	gnome_help_display( NULL, &help_page );
}

/************************************************************************/
// display, and create, the configuration box
/************************************************************************/
void	gsfdm_config_open( void )
{
static GnomePropertyBox	*dlg = NULL;
GtkWidget			*name;			// widget for page name

	if( !dlg )
	{	dlg = GNOME_PROPERTY_BOX( gnome_property_box_new() );
    	gtk_widget_set_events( GTK_WIDGET( dlg ), gtk_widget_get_events( GTK_WIDGET( dlg ) ) | GDK_BUTTON_PRESS_MASK );
		gnome_dialog_close_hides( GNOME_DIALOG( dlg ), TRUE );
		gtk_signal_connect( GTK_OBJECT( dlg ), "delete_event", GTK_SIGNAL_FUNC( exit_callback ), NULL );
		gnome_dialog_button_connect_object( GNOME_DIALOG( dlg ), 0, GTK_SIGNAL_FUNC( exit_callback ), GTK_OBJECT( dlg ) );
		gtk_signal_connect_object( GTK_OBJECT( dlg->ok_button ), "clicked", GTK_SIGNAL_FUNC( exit_callback ), GTK_OBJECT( dlg ) );		
		gtk_signal_connect_object( GTK_OBJECT( dlg->help_button ), "clicked", GTK_SIGNAL_FUNC( dialog_help_callback ), GTK_OBJECT( dlg ) );
		
		gtk_window_set_title( GTK_WINDOW( dlg ), _( "Properties Box" ) );
		
		// create label and add std / general page
		gtk_box_pack_start( GTK_BOX( ( name = gtk_hbox_new( FALSE, 0 ) ) ),
			gnome_pixmap_new_from_xpm_d( conf_low_xpm ), FALSE, 0, 0 );
		gtk_box_pack_start( GTK_BOX( name ), gtk_label_new( _( " General" ) ), FALSE, 0, 0 );
		gtk_widget_show_all( name );
		gnome_property_box_append_page( (GnomePropertyBox *)dlg, gsfdm_config_std_get_page(), name );
	
		// create label and add download page
		gtk_box_pack_start( GTK_BOX( ( name = gtk_hbox_new( FALSE, 0 ) ) ),
			gnome_pixmap_new_from_xpm_d( running_low_xpm ), FALSE, 0, 0 );
		gtk_box_pack_start( GTK_BOX( name ), gtk_label_new( _( " Download" ) ), FALSE, 0, 0 );
		gtk_widget_show_all( name );
		gnome_property_box_append_page( (GnomePropertyBox *)dlg, gsfdm_config_download_get_page(), name );
	
		// create label and add log page
		gtk_box_pack_start( GTK_BOX( ( name = gtk_hbox_new( FALSE, 0 ) ) ),
			gnome_pixmap_new_from_xpm_d( log_low_xpm ), FALSE, 0, 0 );
		gtk_box_pack_start( GTK_BOX( name ), gtk_label_new( _( " Log File" ) ), FALSE, 0, 0 );
		gtk_widget_show_all( name );
		gnome_property_box_append_page( (GnomePropertyBox *)dlg, gsfdm_log_get_config_page(), name );
	
		// create label and add historic page
		gtk_box_pack_start( GTK_BOX( ( name = gtk_hbox_new( FALSE, 0 ) ) ),
			gnome_pixmap_new_from_xpm_d( hist_low_xpm ), FALSE, 0, 0 );
		gtk_box_pack_start( GTK_BOX( name ), gtk_label_new( _( " History" ) ), FALSE, 0, 0 );
		gtk_widget_show_all( name );
		gnome_property_box_append_page( (GnomePropertyBox *)dlg, gsfdm_historic_get_config_page(), name );
	
		// create label and add log page
		gtk_box_pack_start( GTK_BOX( ( name = gtk_hbox_new( FALSE, 0 ) ) ),
			gnome_pixmap_new_from_xpm_d( proxy_low_xpm ), FALSE, 0, 0 );
		gtk_box_pack_start( GTK_BOX( name ), gtk_label_new( _( " Proxy" ) ), FALSE, 0, 0 );
		gtk_widget_show_all( name );
		gnome_property_box_append_page( (GnomePropertyBox *)dlg, gsfdm_config_proxy_get_page(), name );
	}

	gtk_widget_show_all( GTK_WIDGET( dlg ) );
}

/************************************************************************/
// TO CONNECT AFTER
static	void		gsfdm_config_check_bp_switcher( GtkToggleButton *bp, gpointer to_switch )
{	*( (gboolean *)to_switch ) = gtk_toggle_button_get_active( bp );
}

/************************************************************************/
static	void		gsfdm_config_entry_switcher( GtkEntry *e, gpointer to_change )
{	if( *( (gchar **)to_change ) )
		g_free( *( (gchar **)to_change ) );
	
	*( (gchar **)to_change ) = g_strdup( gtk_entry_get_text( e ) );
}

/************************************************************************/
static	void		gsfdm_config_spin_guint_switcher( GtkSpinButton *spin, gpointer to_change )
{	*( (gint *)to_change ) = gtk_spin_button_get_value_as_int( spin );
}

/************************************************************************/
static void main_pppd_callback( GtkEntry *e, gpointer user_data )
{	
	g_free( *((gchar **)user_data) );
	*((gchar **)user_data) = g_strdup( gtk_entry_get_text( e ) );
}

/************************************************************************/
static GtkWidget	*gsfdm_config_std_get_page( void )
{
GtkBox		*b, *bh;
GtkWidget	*w;

	#ifdef	DEBUG_CONFIG_CREATE_DOWNLOAD_BOX
	g_print( "debug : going to create the std page\n" );
	#endif
	
	b = GTK_BOX( gtk_vbox_new( FALSE, 5 ) );
	
	gtk_box_pack_start( b, GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 0 );
	gtk_box_pack_start( bh, gtk_label_new( _( "ppp daemon to use :" ) ), FALSE, FALSE, 5 );
	gtk_box_pack_end( bh, ( w = gtk_entry_new() ), FALSE, FALSE, 5 ); 
	gtk_entry_set_text( GTK_ENTRY( w ), main_pppd );
	gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( main_pppd_callback ), (gpointer)(&main_pppd) );
	
	gtk_box_pack_start( b, ( w = gtk_check_button_new_with_label( _( "Warm when job stack is empty" ) ) ), FALSE, FALSE, 0 );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), main_warm_empty_job_stack );
	gtk_signal_connect( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &main_warm_empty_job_stack ) );
	
	gtk_box_pack_start( b, ( w = gtk_check_button_new_with_label( _( "Prompt before removing" ) ) ), FALSE, FALSE, 0 );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), main_prompt_before_removing );
	gtk_signal_connect( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &main_prompt_before_removing ) );
		
	gtk_box_pack_start( b, ( w = gtk_check_button_new_with_label( _( "Automatically remove job when done" ) ) ), FALSE, FALSE, 0 );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ),main_auto_remove_when_done );
	gtk_signal_connect( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &main_auto_remove_when_done ) );
	
	gtk_box_pack_start( b, ( w = gtk_check_button_new_with_label( _( "Open popup when adding url" ) ) ), FALSE, FALSE, 0 );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ),main_open_popup_on_dnd );
	gtk_signal_connect( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &main_open_popup_on_dnd ) );
	
	// a spin button for update timer
	w = gtk_spin_button_new( GTK_ADJUSTMENT( gtk_adjustment_new( 
		(gfloat )main_update_time, 1.0, 3600.0, 1.0, 5.0, 0.0 ) ), 0, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( w ), TRUE );	
	gtk_spin_button_set_update_policy( GTK_SPIN_BUTTON( w ), GTK_UPDATE_IF_VALID );
	gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( gsfdm_config_spin_guint_switcher ), 
		(gpointer)( &main_update_time ) );
	
	gtk_box_pack_start( b, GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 0 );
	gtk_box_pack_start( bh, gtk_label_new( _( "Update stack status every (sec)" ) ), FALSE, FALSE, 5 );
	gtk_box_pack_end( bh, w, FALSE, FALSE, 5 );
	
	return( GTK_WIDGET( b ) );
}

/************************************************************************/
static void	dirsave_ok_callback( GtkWidget *dlg )
{
gchar	*file = gtk_file_selection_get_filename( GTK_FILE_SELECTION( dlg ) );
//gchar	**ptr = gtk_object_get_data( GTK_OBJECT( dlg ), "config_entry_data" );
	
	gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( dlg ), "config_entry_widget" ) ), file );
	if( url_save_dir )
		g_free( url_save_dir );
	url_save_dir = g_strdup( file );
	gtk_widget_hide( dlg );
}

/************************************************************************/
static void	new_dirsave_callback( GtkWidget *parent, gpointer data )
{
static GtkFileSelection	*dlg = NULL;
	
	if( !dlg )
	{	dlg = GTK_FILE_SELECTION( gtk_file_selection_new( _( "New destination directory" ) ) );
		gtk_widget_set_sensitive( dlg->file_list, FALSE );
 		gtk_window_set_position( GTK_WINDOW( dlg ), GTK_WIN_POS_MOUSE );
	
		gtk_object_set_data( GTK_OBJECT( dlg ), "config_entry_widget", (gpointer)parent );
		gtk_object_set_data( GTK_OBJECT( dlg ), "config_entry_data", data );
		gtk_signal_connect_object( GTK_OBJECT( dlg->ok_button ), "clicked", GTK_SIGNAL_FUNC( dirsave_ok_callback ), GTK_OBJECT( dlg ) );
		gtk_signal_connect_object( GTK_OBJECT( dlg->cancel_button ), "clicked", GTK_SIGNAL_FUNC(  gtk_widget_hide ), GTK_OBJECT( dlg ) );
		gtk_signal_connect_object( GTK_OBJECT( dlg ), "delete_event", GTK_SIGNAL_FUNC( gtk_widget_hide ), GTK_OBJECT( dlg ) );
	}

	gtk_widget_show( GTK_WIDGET( dlg ) );
}

/************************************************************************/
static GtkWidget	*gsfdm_config_download_get_page( void )
{
GtkBox		*bh, *bv;
GtkWidget	*w;
	
	#ifdef	DEBUG_CONFIG_CREATE_DOWNLOAD_BOX
	g_print( "debug : going to create the download page\n" );
	#endif
	
	gtk_box_pack_start( ( bv = GTK_BOX( gtk_vbox_new( FALSE, 5 ) ) ), 
		GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 0 );
	
	// widget for user spec save directory
	gtk_box_pack_start( bh, gtk_label_new( _( "Save in :" ) ), FALSE, FALSE, 5 ); 
	gtk_box_pack_end( bh, ( w = gtk_entry_new() ), FALSE, FALSE, 5 ); 

	gtk_entry_set_text( GTK_ENTRY( w ), url_save_dir );
	gtk_signal_connect( GTK_OBJECT( w ), "button-press-event", 
		GTK_SIGNAL_FUNC( new_dirsave_callback ), (gpointer)(&url_save_dir) );

	// max auto dl simultaneous
	w = gtk_spin_button_new( GTK_ADJUSTMENT( gtk_adjustment_new( 
		(gfloat )url_max_simultaneous, 0.0, 1000.0, 1.0, 5.0, 0.0 ) ), 0, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( w ), TRUE );	
	gtk_spin_button_set_update_policy( GTK_SPIN_BUTTON( w ), GTK_UPDATE_IF_VALID );
	gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( gsfdm_config_spin_guint_switcher ), 
		(gpointer)( &url_max_simultaneous ) );
	
	gtk_box_pack_start( bv, GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 0 );
	gtk_box_pack_start( bh, gtk_label_new( _( "Max auto download simultaneous" ) ), FALSE, FALSE, 5 );
	gtk_box_pack_end( bh, w, FALSE, FALSE, 5 );
	
	// timeout
	w = gtk_spin_button_new( GTK_ADJUSTMENT( gtk_adjustment_new( 
		(gfloat )url_timeout, 0.0, 3600.0, 5.0, 60.0, 0.0 ) ), 0, 0 );
	gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( w ), TRUE );	
	gtk_spin_button_set_update_policy( GTK_SPIN_BUTTON( w ), GTK_UPDATE_IF_VALID );
	gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( gsfdm_config_spin_guint_switcher ), 
		(gpointer)( &url_timeout ) );
	
	gtk_box_pack_start( bv, GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 0 );
	gtk_box_pack_start( bh, gtk_label_new( _( "Timeout before cancelling a frozen job (sec)" ) ), FALSE, FALSE, 5 );
	gtk_box_pack_end( bh, w, FALSE, FALSE, 5 );
	
	gtk_box_pack_start( bv, ( w = gtk_check_button_new_with_label( _( "Automaticaly parse newly downloaded files" ) ) ), FALSE, FALSE, 0 );
	gtk_signal_connect_after( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &url_auto_parse ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), url_auto_parse );
	
	gtk_box_pack_start( bv, gtk_hseparator_new(), FALSE, FALSE, 0 );
		
	// auto dl switch
	gtk_box_pack_start( bv, ( w = gtk_check_button_new_with_label( _( "Auto Download per default" ) ) ), FALSE, FALSE, 0 );
	gtk_signal_connect_after( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &url_auto_download_by_default ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), url_auto_download_by_default );

	// auto rename switch
	gtk_box_pack_start( bv, ( w = gtk_check_button_new_with_label( _( "Auto Rename per default" ) ) ), FALSE, FALSE, 0 );
	gtk_signal_connect_after( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &url_auto_rename_by_default ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), url_auto_rename_by_default );

	gtk_widget_show_all( GTK_WIDGET( bv ) );
	
	return( GTK_WIDGET( bv ) );
}

/************************************************************************/
static GtkWidget	*gsfdm_config_proxy_get_page( void )
{
GtkBox		*b;
GtkTable	*t;
GtkWidget	*w;
	
	gtk_box_pack_start( ( b = GTK_BOX( gtk_vbox_new( FALSE, 5 ) ) ), 
		( w = gtk_check_button_new_with_label( _( "Use a proxy" ) ) ), FALSE, FALSE, 0 );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), gsfdm_proxy_st_get_use_proxy( proxy_data ) );
	gtk_signal_connect( GTK_OBJECT( w ), "toggled", 
		GTK_SIGNAL_FUNC( gsfdm_config_check_bp_switcher ), (gpointer)( &gsfdm_proxy_st_get_use_proxy( proxy_data ) ) );

	gtk_box_pack_start( b, GTK_WIDGET( t = GTK_TABLE( gtk_table_new( 2, 3, FALSE ) ) ), FALSE, FALSE, 0 );
	
	gtk_table_attach( t, gtk_label_new( _( "User : " ) ), 0, 1, 0, 1, 0,0,0,0 );
	gtk_table_attach( t, gtk_label_new( _( "Pass : " ) ), 0, 1, 1, 2, 0,0,0,0 );
	gtk_table_attach( t, gtk_label_new( _( "URL : " ) ), 0, 1, 2, 3, 0,0,0,0 );
	
	gtk_table_attach( t, ( w = gtk_entry_new( ) ), 1, 2, 0, 1, GTK_FILL | GTK_EXPAND,0,0,0 );
	gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( gsfdm_config_entry_switcher ), 
		(gpointer)( &gsfdm_proxy_st_get_user( proxy_data ) ) );
	gtk_entry_set_text( GTK_ENTRY( w ), gsfdm_proxy_st_get_user( proxy_data ) );
	
	gtk_table_attach( t, ( w = gtk_entry_new( ) ), 1, 2, 1, 2, GTK_FILL | GTK_EXPAND,0,0,0 );
	gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( gsfdm_config_entry_switcher ), 
		(gpointer)( &gsfdm_proxy_st_get_pass( proxy_data ) ) );
	gtk_entry_set_text( GTK_ENTRY( w ), gsfdm_proxy_st_get_pass( proxy_data ) );
	
	gtk_table_attach( t, ( w = gtk_entry_new( ) ), 1, 2, 2, 3, GTK_FILL | GTK_EXPAND,0,0,0 );
	gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( gsfdm_config_entry_switcher ), 
		(gpointer)( &gsfdm_proxy_st_get_proxy_url( proxy_data ) ) );
	gtk_entry_set_text( GTK_ENTRY( w ), gsfdm_proxy_st_get_proxy_url( proxy_data ) );
	
	return( GTK_WIDGET( b ) );
}

/************************************************************************/
// load the global config of the gui
/************************************************************************/
void	gsfdm_config_load( void )
{
	proxy_data = st_gsfdm_config_proxy_new();
	
	gnome_config_push_prefix( "/"PACKAGE"/proxy/" );
	gsfdm_proxy_st_get_use_proxy( proxy_data ) = gnome_config_get_bool( "use_proxy=FALSE" );
	gsfdm_proxy_st_get_user( proxy_data ) = gnome_config_get_string( "user_proxy=user" );
	gsfdm_proxy_st_get_pass( proxy_data ) = gnome_config_get_string( "pass_proxy=pass" );
	gsfdm_proxy_st_get_proxy_url( proxy_data ) = gnome_config_get_string( "url_proxy=http://proxy:8080" );
	gnome_config_pop_prefix();

	gnome_config_push_prefix( "/"PACKAGE"/std/" );
	main_warm_empty_job_stack = gnome_config_get_bool( "warm_empty_job_stack=TRUE" );
	main_prompt_before_removing = gnome_config_get_bool( "prompt_before_removing=FALSE" );
	main_auto_remove_when_done = gnome_config_get_bool( "auto_remove_when_done=FALSE" );
	main_open_popup_on_dnd = gnome_config_get_bool( "open_popup=FALSE" );
	main_update_time = gnome_config_get_int( "update_time=1" );
	main_pppd = gnome_config_get_string( "pppd=ppp0" );
	gnome_config_pop_prefix();

	gnome_config_push_prefix( "/"PACKAGE"/url_conf/" );
	url_auto_download_by_default = gnome_config_get_bool( "auto_by_default=TRUE" );
	url_auto_rename_by_default = gnome_config_get_bool( "auto_rename_by_default=FALSE" );
	url_max_simultaneous = gnome_config_get_int( "max=5" );
	url_timeout = gnome_config_get_int( "timeout=180" );
	url_save_dir = gnome_config_get_string( "default_dir=/tmp" );
	url_auto_parse = gnome_config_get_bool( "url_auto_parse=TRUE" );
	gnome_config_pop_prefix();
	
//	gsfdm_historic_load_conf();
}

/************************************************************************/
// save the global config of the gui
/************************************************************************/
void	gsfdm_config_save( void )
{
	gnome_config_push_prefix( "/"PACKAGE"/proxy/" );
	gnome_config_set_bool( "use_proxy", gsfdm_proxy_st_get_use_proxy( proxy_data ) );
	gnome_config_set_string( "user_proxy", gsfdm_proxy_st_get_user( proxy_data ) );
	gnome_config_set_string( "pass_proxy", gsfdm_proxy_st_get_pass( proxy_data ) );
	gnome_config_set_string( "url_proxy", gsfdm_proxy_st_get_proxy_url( proxy_data ) );
	gnome_config_pop_prefix();

	gnome_config_push_prefix( "/"PACKAGE"/std/" );
	gnome_config_set_bool( "warm_empty_job_stack", main_warm_empty_job_stack );
	gnome_config_set_bool( "prompt_before_removing", main_prompt_before_removing );
	gnome_config_set_bool( "auto_remove_when_done", main_auto_remove_when_done );
	gnome_config_set_bool( "open_popup", main_open_popup_on_dnd );
	gnome_config_set_int( "update_time",main_update_time );
	gnome_config_set_string( "pppd", main_pppd );
	gnome_config_pop_prefix();

	gnome_config_push_prefix( "/"PACKAGE"/url_conf/" );
	gnome_config_set_bool( "auto_by_default", url_auto_download_by_default );
	gnome_config_set_bool(" auto_rename_by_default", url_auto_rename_by_default );
	gnome_config_set_int( "max", url_max_simultaneous );
	gnome_config_set_int( "timeout", url_timeout );
	gnome_config_set_string( "default_dir", url_save_dir );
	gnome_config_set_bool( "url_auto_parse", url_auto_parse );
	gnome_config_pop_prefix();
	
//	gsfdm_historic_save_conf();
	
	gnome_config_sync();
}

/************************************************************************/
//	return the struct for proxy config
st_gsfdm_config_proxy const *gsfdm_config_get_proxy( void )
{	if( !proxy_data )
		gsfdm_config_load();
	
	return( proxy_data );
}

/************************************************************************/
gboolean	gsfdm_config_get_warm_emply_job( void )
{	return( main_warm_empty_job_stack ); }

/************************************************************************/
gboolean	gsfdm_config_prompt_before_removing( void )
{	return( main_prompt_before_removing ); }
	
/************************************************************************/
gboolean	gsfdm_config_auto_remove_when_done( void )
{	return( main_auto_remove_when_done ); }

/************************************************************************/
gboolean	gsfdm_config_get_auto_rename( void )
{	return( url_auto_rename_by_default ); }

/************************************************************************/
// return whether to switch auto download by default on or not
gboolean	gsfdm_config_get_auto_download( void )
{	return( url_auto_download_by_default ); }

/************************************************************************/
gboolean	gsfdm_config_get_auto_parse( void )
{	return( url_auto_parse ); }

/************************************************************************/
gboolean	gsfdm_config_open_popup_on_dnd( void )
{	return( main_open_popup_on_dnd ); }

/************************************************************************/
guint		gsfdm_config_get_update_time( void )
{	return( main_update_time ); }

/************************************************************************/
guint		gsfdm_config_max_simultaneous( void )
{	
	if( !url_max_simultaneous )
		return( G_MAXINT );
	
	return( url_max_simultaneous );
}

/************************************************************************/
guint		gsfdm_config_get_timeout_on_download( void )
{	return( url_timeout ); }

/************************************************************************/
gchar const *gsfdm_config_get_default_save_dir( void )
{	return( url_save_dir ); }

/************************************************************************/
gchar const *gsfdm_config_get_pppd( void )
{	return( main_pppd ); }

