/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		Header for file manipulation : func and boxes

/************************************************************************/

#include	<gnome.h>
#include	<signal.h>
#include	<dirent.h>
#include	<sys/stat.h>
#include	<sys/wait.h>
#include	"gsfdm_fileop.h"

/************************************************************************/
//	debug flag
/************************************************************************/
#ifdef HAVE_DEBUG
#undef	GSFDM_FILE_DIR_PARSING				// debug the get_file_from_dir
#undef DEBUG_FILE_TYPE						// debug get_file_type
#undef	DEBUG_FILE_DUMP_PARSING_ANSWER		// get_file_name from url
#endif

/************************************************************************/
//	static variables definition
/************************************************************************/
#define		GSFDM_FILEOP_ERROR_CODE	-1		// used for communication between state of the automates

/************************************************************************/
//	functions definition
/************************************************************************/

/************************************************************************/
// return the type of the file ; just prevent use of low level
// function in the code ( create a cleaner code )
/************************************************************************/
GsfdmFileType	gsfdm_file_get_file_type( gchar const *file )
{
struct stat	v;
	
	#ifdef DEBUG_FILE_TYPE
	g_print( "debug file : get type of %s\n", file );
	#endif

	if( stat( file, &v )	)
		return( GSFDM_FILE_TYPE_ERROR );
	
	#ifdef DEBUG_FILE_TYPE
	g_print( "debug file : %s is %d --\n", file, GSFDM_FILE_MASK_TYPE & v.st_mode );
	#endif
	
	return( GSFDM_FILE_MASK_TYPE & v.st_mode );
}

/************************************************************************/
// return stat from a file, storing data in an internal struct
struct stat const *gsfdm_file_get_file_stat_fast( gchar const *file, gboolean real )
{
static struct stat	st;

	if( real )
	{	if( lstat( file, &st ) )				// don't fallow symb
		return( NULL );
	
	} else {
		if( stat( file, &st ) )				// don't fallow symb
		return( NULL );
	}

	return( &st );
}
	
/************************************************************************/
// return( a new GList with (g_new) of all readable file ( for user ) in this directory
GList	*gsfdm_file_get_file_from_dir( gchar const *dir_name )
{	// copy paste from my other soft...
GList			*list = NULL;
DIR				*dir;
struct dirent	*file;
gchar			*tmp;

	#ifdef GSFDM_FILE_DIR_PARSING
	g_print( "debug file : getting file from dir : %s\n", dir_name );
	#endif

	if( !dir_name ) return( NULL );
	if( !( dir = opendir( dir_name ) ) ) return( NULL );	// unknow dir
	
	// read and check all files in dir
	while( ( file = readdir( dir ) ) )
	{	// discard current and parent
		if ( !strcmp( file->d_name, "." ) || !strcmp( file->d_name, ".." ) ) continue;
		tmp = g_strconcat( dir_name , "/", file->d_name, NULL );
		
		if( gsfdm_file_get_file_type( tmp ) == GSFDM_FILE_TYPE_REGULAR )
		{	list = g_list_append( list, (gpointer)g_strdup( file->d_name ) );

			#ifdef GSFDM_FILE_DIR_PARSING
			g_print( "debug file : found %s in %s\n", file->d_name, dir_name );
			#endif
		}
		#ifdef GSFDM_FILE_DIR_PARSING
		else { g_print( "debug file : %s is NOT regular \n", file->d_name ); }
		#endif
		
		g_free( tmp );
	}
	closedir( dir );
	
	return( g_list_first( list ) );
}

/************************************************************************/
// return a g_new str matching the ext type of this url / file
// exytract also from url with inlined options
/************************************************************************/
gchar  *gsfdm_file_extract_ext( gchar const *url )
{
	static gchar *state_20( guint index, guint start );
	static gchar *state_21( guint index, guint start );
	static gchar *state_22( guint index, guint start );
	static gchar *state_23( guint index, guint start );
	static gchar *state_24( guint index, guint start );
	static gchar *state_25( guint index, guint start );
	#define parse_ext() state_20( 0, 0 )
	
	static gchar *state_20( guint index, guint start )
	{	switch( url[index] )
		{	case '/' :	return( state_21( ++index, 0 ) );
			case '?' :
			case '#' :
			case '\0' :	return( NULL );
			default :	return( state_22( ++index, 0 ) );
		}
	}
	
	static gchar *state_21( guint index, guint start )
	{	switch( url[index] )
		{	case '/' :	return( state_21( ++index, 0 ) );
			case '?' :
			case '#' :
			case '\0' :	return( NULL );
			default :	return( state_22( ++index, 0 ) );
		}
	}
	
	static gchar *state_22( guint index, guint start )
	{	switch( url[index] )
		{	case '/' :	return( state_21( ++index, 0 ) );
			case '?' :
			case '#' :
			case '\0' :	return( NULL );
			case '.' :	return( state_23( ++index, 0 ) );
			default :	return( state_22( ++index, 0 ) );
		}
	}

	static gchar *state_23( guint index, guint start )
	{	switch( url[index] )
		{	case '/' :	return( state_21( ++index, 0 ) );
			case '?' :
			case '#' :
			case '\0' :	return( NULL );
			case '.' :	return( state_23( ++index, index ) );
			default :	return( state_24( ++index, index ) );
		}
	}

	static gchar *state_24( guint index, guint start )
	{	switch( url[index] )
		{	case '/' :	return( state_21( ++index, 0 ) );
			case '?' :
			case '#' :
			case '\0' :			// found an ext
			{	gchar	*tmp = 	g_strndup( &url[start-1], ( index - start + 1 ) );
				g_strdown( tmp );
				return( tmp );
			}
			default :	return( state_24( ++index, start ) );
		}
	}
	
	if( !url ) return( NULL );
	return( parse_ext() );
}

/************************************************************************/
// return a g_new str matchuing the protocole ( http:// mms:// ... )
// if none ( i.e. hd file ), return "file://"
/************************************************************************/
gchar	*gsfdm_file_get_protocol( gchar const *url )
{
	static gchar *state_0( gint pos );
	static gchar *state_1( gint pos );
	static gchar *state_2( gint pos );
	
	/************************************************************************/
	//	state of the automate, not really tough : 2 no-ending states !
	static gchar *state_0( gint pos )
	{	switch( url[ pos ] )
		{	case '.' :
			case ':' :
			case '?' :
			case '@' :
			case '#' :
			case '\0' :	return( NULL );
			case '/' :	return( g_strdup( "" ) );			// null string, local path
			default :	return( state_1( ++pos ) );			// ok, continue
		}
	}
	
	static gchar *state_1( gint pos )
	{	switch( url[ pos ] )
		{	case '?' :
			case '@' :
			case '/' :
			case '#' :
			case '\0' :		return( NULL );					// error in string
			case ':' :		return( state_2( ++pos ) );		// found the beginning of the end...
			default :		return( state_1( ++pos ) );		// ok, continue
		}
	}

	static gchar *state_2( gint pos )
	{	switch( url[ pos ] )
		{	case '/' :		return( g_strndup( url, pos ) );	// found
			default :		return( NULL );					// not the char expected
		}
	}

	/************************************************************************/	
	if( !url ) return( NULL );
		
	#ifdef	DEBUG_FILE_DUMP_PARSING_ANSWER
	{ gchar	*answer = state_0( 0 );
	
		g_print( "debug file : get protocol from %s --> %s\n", url, answer );
		return( answer );
	}
	#else
	return( state_0( 0 ) );
	#endif
}

/************************************************************************/
//	return true if the url match a local protocol :
// proto could by a proto string or directly an url / file string
// proto local == 'file:', 'files:', '/' or NULL ( case returned by get_proto )
/************************************************************************/
gboolean	gsfdm_file_protocol_is_local( gchar const *proto )
{
	if( !proto )
		return( TRUE );
		
	if( !g_strcasecmp( proto, "file:" ) )
		return( TRUE );
	
	if( !g_strcasecmp( proto, "files:" ) )
		return( TRUE );
		
	if( proto[0] == '/' )
		return( TRUE );
	
	return( FALSE );
}

/************************************************************************/
// ... extract the file name
/************************************************************************/
gchar	*gsfdm_file_get_file_name( gchar const *url )
{
guint	state = 30, start = 0, index = 0;
gboolean	end = FALSE;
gchar		*answer = NULL;
	
	// nota : could have used str func but with an automate,
	// it's more versatile and cleaner...
	// little different, no recursive, just for fun
	
	do{
		switch( state )
		{	case 30 :
			{	switch( url[ index ] )
				{	case '.' :	state = 31; start = index;	break;
					case '/' :	state = 33;	break;
					case '?' :
					case '#' :
					case 0 :	end = TRUE; break;
					default :	state = 34;	start = index;
				}
				break;
			}
			
			case 31 :	
			{	switch( url[ index ] )
				{	case '.' :	state = 32;	break;
					case '/' :	state = 33;	break;
					case '?' :
					case '#' :
					case 0 :	end = TRUE;	break;
					default :	state = 34;
				}
				break;
			}
				
			case 32 :
			{	switch( url[ index ] )
				{	case '/' :	state = 33;	break;
					case '?' :
					case '#' :
					case 0 :	end = TRUE;	break;
					default :	state = 34;
				}
				break;
			}
			
			case 33 :
			{	switch( url[ index ] )
				{	case '/' :	break;
					case '.' :	state = 31;	start = index;
						break;
					case '?' :
					case '#' :
					case 0 :	end = TRUE; break;
					default :	state = 34;	start = index;
				}
				break;
			}
			
			case 34 :
			{	switch( url[ index ] )
				{	case '?' :
					case '#' :
					case 0 :	answer = g_strndup( &url[ start ], index - start );	end = TRUE;	break;
					case '/' :	state = 33;	break;
					default :
						break;
				}
				break;
			}
			
			default :
				end = TRUE;
		}
		
	} while( index++, !end );
	
	#ifdef	DEBUG_FILE_DUMP_PARSING_ANSWER
		g_print( "debug file : get_file_name from %s --> %s \n", url, answer );
	}
	#endif
	
	if( !url ) return( NULL );
	return( answer );
}

/************************************************************************/
// comput a new name for the file "url", into the rep "dest", if formated
// is true that meen url is hd format ( path ) and not like "http://ar/eu/h/machintruc?blabla"
gchar	*gsfdm_file_get_new_name( gchar const *url, gchar const *dest, gboolean formated )
{
	static gboolean state_10( guint index0, guint index );
	static gboolean state_11( guint index0, guint index );
	static gboolean state_12( guint index0, guint index );
	#define parse_string() ( state_10( 0, 0 ) )

gchar	*travail, *res, *base, *ext, *tmp;
guint	counter;

	static gboolean state_10( guint index0, guint index )
	{	switch( travail[index] )
		{	case '1' :
			case '2' :
			case '3' :
			case '4' :
			case '5' :
			case '6' :
			case '7' :
			case '8' :
			case '9' :
			case '0' :	return( state_11( index, index + 1 ) );
				break;
			case '\0' :	return( FALSE );
				break;
			default :	return( state_12( 0, ++index ) );
		}
	}
	
	static gboolean state_11( guint index0, guint index )
	{	switch( travail[index] )
		{	case '.' :
			case '\0' :
				base = g_strndup( travail, index0 );
				ext = g_strdup( &travail[index] );

				counter = 0;								// ascII to num
				while( index0 < index )
				{	counter *= 10;
					counter += ( travail[ index0 ] - '0' );
					index0++;
				}
				return( TRUE );								// match !!!

			case '1' :
			case '2' :
			case '3' :
			case '4' :
			case '5' :
			case '6' :
			case '7' :
			case '8' :
			case '9' :
			case '0' :	return( state_11( index0, ++index ) );
				break;
			default :	return( state_12( 0, ++index ) );
		}
	}
	
	static gboolean state_12( guint index0, guint index )
	{
		switch( travail[index] )
		{	case '.' :
			case '\0' :
				base = g_strndup( travail, index );
				ext = g_strdup( &travail[index] );
				counter = 0;
				return( TRUE );
				break;				
			case '1' :
			case '2' :
			case '3' :
			case '4' :
			case '5' :
			case '6' :
			case '7' :
			case '8' :
			case '9' :
			case '0' :	return( state_11( index, index + 1 ) );
				break;		
			default :	return( state_12( 0, ++index ) );
		}
	}
	
	/************************************************************************/
	if( gsfdm_file_get_file_type( dest ) != GSFDM_FILE_TYPE_DIR )
		return( 0 );
	
	if( !formated )
		travail = gsfdm_file_get_file_name( url );
	 else
		travail = g_strdup( url );

	// create base, counter and ext, false on error ( null string )
	if( !parse_string() )
	{	g_free( travail );	
		return( NULL );
	}
	
	// loop until a valid name was found
	while( 
		( res = g_strdup_printf( "%s%d%s", base, ++counter, ext ) ), 
		( tmp = g_strconcat( dest, res, NULL ) ),
		gsfdm_file_get_file_type( tmp ) != GSFDM_FILE_TYPE_ERROR )
	{	g_free( res );
		g_free( tmp );
	}
	
	g_free( travail );	
	g_free( base );
	g_free( ext );
	g_free( tmp );		
	return( res );
}

/************************************************************************/
// create the array from a command line string ; break on first command
// can't parse multiple command line ( separated with , or ; ) & is an end condition
// for a classic arg list, use this : 
//			gchar **l = (gchar **)parse_command_line_from_string( command )->pdata;
GPtrArray *parse_command_line_from_string( gchar const *str )
{
GPtrArray	*a;
gchar		*ed, *ed2, *st = (gchar *)str;

	if( !st ) 
		return( NULL );

	a = g_ptr_array_new();

	while( st[0] )
	{	
		// reach for end arg sequence or end of string test if at end of string :
		if( !( ed = strpbrk( st, "\t&, \"" ) ) )
		{	g_ptr_array_add( a, (gpointer)g_strdup( st ) );
			break;
		}
		
		// found break sequence, create new string
		if( ed != st )
			g_ptr_array_add( a, (gpointer)g_strndup( st,  ed - st ) );
		
		// this is a sub-string ???			
		if( ed[0] == '\"' )
		{	// found the closer or exit on error
			if( !( ed2 = strchr( ed+1, '\"' ) ) )
			{	// free all allocated memory
				g_ptr_array_free( a, TRUE );
				return( NULL );
			}

			// found, create new arg
			g_ptr_array_add( a, (gpointer)g_strndup( ed+1, ed2 - ed - 1 ) );
			
			st = ed2 + 1;										// new start for next search
			continue;
		}
		
		// test if halt on a break char
		if( ed[0] != ' ' )
			break;

		st = ed + 1;		
	}

return( a );
}

/************************************************************************/
// launch command like sytem(), except than don't freeze parent until end of child
// if freeze is FALSE, and use a fork instead using sh. End command with & to create
// it's own session
// return child pid or 0 on error
pid_t	system_ext( gchar const *command, gboolean freeze )
{
pid_t	pid, ppid = getpid();

	signal( SIGCHLD, SIG_IGN );								// prevent zombie

	if( !( pid = fork() ) )									// don't test for pid owerflow...
	{	gchar **list = (gchar **)parse_command_line_from_string( command )->pdata;

		setpgid( 0, ppid );
		if( strchr( command, '&' ) )						// test if should make child as is own process
			setsid();										// set to background

		execvp( list[0], list );							// start the command
		g_print( "Can't launch application...\n" );
		_exit( 1 );
	}

	if( freeze && !strchr( command, '&' ) )
		waitpid( pid, NULL, WUNTRACED );

	return( pid );
}



