/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		Code for historic

/************************************************************************/

#include	<gnome.h>
#include	<stdio.h>
#include	<config.h>
#include	<unistd.h>
#include	"gsfdm_historic.h"
#include	"gsfdm_fileop.h"
#include	"gsfdm_message.h"

/************************************************************************/
//	debug support ( compiling flag )
/************************************************************************/
#ifdef HAVE_DEBUG
#define DEBUG_CONFIG_CREATE_HISTORY
#define DEBUG_HISTORIC_PUSH_URL
#define DEBUG_HISTORIC_LOAD
#define DEBUG_HISTORIC_DLG
#endif

/************************************************************************/
//	static functions definition
/************************************************************************/
static void		gsfdm_historic_add_entry( st_gsfdm_historic_entry *e );
static void		gsfdm_historic_remove_entry( gchar const *url );
static void		gsfdm_historic_remove_all_entry( void );
static st_gsfdm_historic_entry const *gsfdm_historic_found_entry_by_url( gchar const *url );
static st_gsfdm_historic_entry const *gsfdm_historic_found_entry_by_date( void );
static void		gsfdm_historic_foreach_entry( GHFunc func, gpointer data );
static guint	gsfdm_historic_get_nb_entry( void );

static void gsfdm_historic_clear( GnomeDialog *d );
static void gsfdm_historic_clear_sel( GnomeDialog *d );
static void gsfdm_historic_down( GnomeDialog *d );
static void gsfdm_historic_sort( GtkCList *w, guint c );

static void	gsfdm_historic_add_to_dlg( st_gsfdm_historic_entry *e );
static void	gsfdm_historic_remove_from_dlg( st_gsfdm_historic_entry *e );

/************************************************************************/
//	static variables definition
/************************************************************************/
#define	HISTORIC_FILE_NAME	"/gsfdm/gsfdm.hst"

#define HISTORIC_BP_CLEAR 0
#define HSITORIC_BP_CLEAR_SEL 1
#define HISTORIC_BP_DOWN 2
#define HISTORIC_BP_CLOSE 3

#define HISTORIC_COL_DATE 0
#define HISTORIC_COL_URL 1

#define HISTORIC_DATE_TEMPLATE "%Y/%m/%d - %H:%M"

//#define HISTORIC_ADD_DATE 0x5			// no specific reason, 0x5 ; class by date
//#define HISTORIC_ADD_ALPHA 0xA			// class by name... default

static	GtkBox	*bw = NULL;
	
static GHashTable	*list = NULL;

static gboolean	historic_use_history;
static gboolean	historic_use_specific;
static gchar	*historic_file_name;
static guint	historic_length;
static GsfdmHistoricMode historic_policy;
static FILE		*fd_historic_file = NULL;

static GnomeDialog	*dlg = NULL;
static GtkCList	*list_widget;

/************************************************************************/
//	functions definition
/************************************************************************/
// there are conveniences functions for hash manip, in case of change...
static void		gsfdm_historic_add_entry( st_gsfdm_historic_entry *e )
{	
	// don't check whether e == e in hash
	gsfdm_historic_remove_entry( gsfdm_historic_st_get_url( e ) );	
	g_hash_table_insert( list, (gpointer)g_strdup( gsfdm_historic_st_get_url( e ) ), (gpointer)e );
}

/************************************************************************/
static void		gsfdm_historic_remove_entry( gchar const *url )
{
	g_hash_table_remove( list, (gconstpointer)url );
}

/************************************************************************/
static void		gsfdm_historic_remove_all_entry( void )
{
	static gboolean callback( gpointer key, gpointer value, gpointer user_data )
	{	st_historic_entry_destroy( value );
		return( TRUE );
	}
	
	g_hash_table_foreach_remove( list, (GHRFunc)callback, NULL );
}

/************************************************************************/
static st_gsfdm_historic_entry const *gsfdm_historic_found_entry_by_url( gchar const *url )
{	
gpointer	p = g_hash_table_lookup( list, (gconstpointer)url );
	
	if( p )
		return( ST_GSFDM_HISTORIC_ENTRY( p ) );
	
	return( NULL );
}

/************************************************************************/
static st_gsfdm_historic_entry const *gsfdm_historic_found_entry_by_date( void )
{
	return( NULL );
}

/************************************************************************/
static void		gsfdm_historic_foreach_entry( GHFunc func, gpointer data )
{
	g_hash_table_foreach( list, func, data );
}

/************************************************************************/
static guint		gsfdm_historic_get_nb_entry( void )
{
	return( g_hash_table_size( list ) );
}

/************************************************************************/
static	void	check_bp_switcher( GtkToggleButton *bp, gpointer to_switch )
{	*( (gboolean *)to_switch ) = gtk_toggle_button_get_active( bp );
}

/************************************************************************/
static	void		gsfdm_historic_spin_guint_switcher( GtkSpinButton *spin, gpointer to_change )
{	*( (gint *)to_change ) = gtk_spin_button_get_value_as_int( spin );
}

/************************************************************************/
static void gsfdm_historic_radio_callback( GtkToggleButton *togglebutton, gpointer data )
{	if( togglebutton->active )
		historic_policy = ( GsfdmHistoricMode )data;
}

/************************************************************************/
static void	filesel_ok_callback( GtkWidget *d )
{
gchar	*file = gtk_file_selection_get_filename( GTK_FILE_SELECTION( d ) );
FILE	*fd;
	
	if( file )
	{	// first, test if exit, prompt if agree to truncate
		switch( gsfdm_file_get_file_type( file ) )
		{	case GSFDM_FILE_TYPE_REGULAR :
				if( gnome_dialog_run_and_close( GNOME_DIALOG( 
					gnome_question_dialog_parented( _( "Will erase this file, Agree ??" ),
					NULL, NULL, GTK_WINDOW( d ) ) ) ) )
					break;							// user not agree

			// the case of error is : don't exist yet, access denied
			case GSFDM_FILE_TYPE_ERROR :
				// able to open it ??
				if( ( fd = fopen( file, "w+" ) ) )
				{	// close current, open new
					fclose( fd_historic_file );
					fd_historic_file = fd;
					historic_file_name = g_strdup( file );
					
					// update the config widget
					gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( d ), "config_entry_widget" ) ), file );
					break;
				}
				
			default :
				gtk_widget_show( gnome_warning_dialog_parented( _( "Unsupported file type, Abort" ), GTK_WINDOW( d ) ) );
		}
	}
	
	gtk_widget_destroy( d );
}

/************************************************************************/
static void	new_file_name_callback( GtkWidget *parent )
{
GtkWidget	*d;
	
	d = gtk_file_selection_new( _( "New History file name, selec or enter new name" ) );
 	gtk_window_set_position( GTK_WINDOW( d ), GTK_WIN_POS_MOUSE );
	gtk_object_set_data( GTK_OBJECT( d ), "config_entry_widget", (gpointer)parent );
	gtk_signal_connect_object( GTK_OBJECT( GTK_FILE_SELECTION( d )->ok_button ), "clicked", GTK_SIGNAL_FUNC( filesel_ok_callback ), GTK_OBJECT( d ) );
	
	gtk_widget_show( GTK_WIDGET( d ) );
}

/************************************************************************/
// return widget for configuration
GtkWidget	*gsfdm_historic_get_config_page( void )
{
GtkBox			*bh;
GtkWidget		*c, *w, *f;
GtkRadioButton	*exit_radio;

	if( !bw )
	{		
		#ifdef	DEBUG_CONFIG_CREATE_HISTORY
		g_print( "debug : going to create the hist page : %d %s\n", historic_use_history, historic_file_name );
		#endif
		
		bw = GTK_BOX( gtk_vbox_new( FALSE, 5 ) );
		
		gtk_box_pack_start( bw, ( c = gtk_check_button_new_with_label( _( "Use history" ) ) ), FALSE, FALSE, 0 );
		gtk_signal_connect_after( GTK_OBJECT( c ), "toggled", GTK_SIGNAL_FUNC( check_bp_switcher ), (gpointer)(&historic_use_history) );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ), historic_use_history  );
	
		gtk_box_pack_start( bw, GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 5 );
		gtk_box_pack_start( bh, ( c = gtk_check_button_new_with_label( _( "Use Specific history file :" ) ) ), FALSE, FALSE, 0 );
		gtk_signal_connect_after( GTK_OBJECT( c ), "toggled", GTK_SIGNAL_FUNC( check_bp_switcher ), (gpointer)(&historic_use_specific) );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ), historic_use_specific );
		
		gtk_box_pack_end( bh, ( c = gtk_entry_new( ) ), FALSE, FALSE, 5 );
		if( historic_use_history )
			gtk_entry_set_text( GTK_ENTRY( c ), historic_file_name );
		gtk_signal_connect_object( GTK_OBJECT( c ), "button-press-event", GTK_SIGNAL_FUNC( new_file_name_callback ), NULL );
		
		// length limit
		w = gtk_spin_button_new( GTK_ADJUSTMENT( gtk_adjustment_new( (gfloat )historic_length, 0.0, 5000.0, 10.0, 100.0, 0.0 ) ), 0, 0 );
		gtk_spin_button_set_numeric( GTK_SPIN_BUTTON( w ), TRUE );	
		gtk_spin_button_set_update_policy( GTK_SPIN_BUTTON( w ), GTK_UPDATE_IF_VALID );
		gtk_signal_connect_after( GTK_OBJECT( w ), "changed", GTK_SIGNAL_FUNC( gsfdm_historic_spin_guint_switcher ), 
			(gpointer)( &historic_length ) );
		
		gtk_box_pack_start( bw, GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 0 );
		gtk_box_pack_start( bh, gtk_label_new( _( "Maximun length ( urls )" ) ), FALSE, FALSE, 5 );
		gtk_box_pack_end( bh, w, FALSE, FALSE, 5 );

		// frame for mode	
		gtk_box_pack_start( bw, GTK_WIDGET( f = gtk_frame_new( _( "When Url Match :" ) ) ), FALSE, FALSE, 0 );
		gtk_container_add( GTK_CONTAINER( f ), GTK_WIDGET( bh = GTK_BOX( gtk_vbox_new( FALSE, 5 ) ) ) );
			
		// the radio part
		gtk_box_pack_start( GTK_BOX( bh ), GTK_WIDGET( exit_radio = GTK_RADIO_BUTTON( 
			gtk_radio_button_new_with_label( NULL, _( "Don't care... Add" ) ) ) ), FALSE, FALSE, 0 );
		gtk_signal_connect( GTK_OBJECT( exit_radio ) , "toggled", GTK_SIGNAL_FUNC( gsfdm_historic_radio_callback ), (gpointer)GSFDM_HISTORIC_REPLACE );

		gtk_box_pack_start( GTK_BOX( bh ), GTK_WIDGET( exit_radio = GTK_RADIO_BUTTON( 
			gtk_radio_button_new_with_label( gtk_radio_button_group( exit_radio ), 
			_( "Ask What to do..." ) ) ) ), FALSE, FALSE, 0 );
		gtk_signal_connect( GTK_OBJECT( exit_radio ) , "toggled", GTK_SIGNAL_FUNC( gsfdm_historic_radio_callback ), (gpointer)GSFDM_HISTORIC_ASK );

		gtk_box_pack_start( GTK_BOX( bh ), GTK_WIDGET( exit_radio = GTK_RADIO_BUTTON( 
			gtk_radio_button_new_with_label( gtk_radio_button_group( exit_radio ), 
			_( "Skip..." ) ) ) ), FALSE, FALSE, 0 );
		gtk_signal_connect( GTK_OBJECT( exit_radio ) , "toggled", GTK_SIGNAL_FUNC( gsfdm_historic_radio_callback ), (gpointer)GSFDM_HISTORIC_SKIP );
		
		gtk_widget_show_all( GTK_WIDGET(  bw ) );
	}
	
	return( GTK_WIDGET( bw ) );
}

#define HISTORIC_FILE_LINE_LENGTH 1024
/************************************************************************/
// load configuration from saved conf
void		gsfdm_historic_load_conf( void )
{
	gnome_config_push_prefix( "/"PACKAGE"/historic/" );
	historic_use_history = gnome_config_get_bool( "historic_use_history=TRUE" );

	if( ( historic_use_specific = gnome_config_get_bool( "historic_use_specific=FALSE" ) ) )
	{
		// get a file name, test for right access on its directory
		historic_file_name = gnome_config_get_string( "historic_file_name="HISTORIC_FILE_NAME );
		
		// if can't create the file
		if( !( fd_historic_file = fopen( historic_file_name, "a+" ) ) )
		{	g_free( historic_file_name );
			historic_file_name = g_strconcat( g_get_home_dir(), "/"HISTORIC_FILE_NAME, NULL );
			
			gsfdm_message_push_message( 
			_( "Failing to open log file, coming back to default : ~/"HISTORIC_FILE_NAME ), FALSE, TRUE, FALSE );
		}
		
	} else {
		historic_file_name = g_strconcat( g_get_home_dir(), "/"HISTORIC_FILE_NAME, NULL );
	}
	
	if( !( fd_historic_file = fopen( historic_file_name, "a+" ) ) )
	{	gsfdm_message_push_message( _( "Fail to create default histoty file, Use of history disabled..." ), FALSE, TRUE, FALSE );
		historic_use_history = FALSE;						// fail to create, write
	}
	
	historic_length = gnome_config_get_int( "historic_length=100" );
	historic_policy = gnome_config_get_int( g_strdup_printf( "historic_policy=%d", GSFDM_HISTORIC_ASK ) );
	gnome_config_pop_prefix();
	
	// create table for history row
	list = g_hash_table_new( g_str_hash, g_str_equal );
	
	#ifdef DEBUG_HISTORIC_LOAD
	g_print( "---- debug historic : load from %s %d\n", historic_file_name, fd_historic_file );
	#endif
	
	// fill with saved history
	if( fd_historic_file && historic_use_history )
	{	gchar	*ptr, *str = g_new( gchar, HISTORIC_FILE_LINE_LENGTH );
		size_t	size = HISTORIC_FILE_LINE_LENGTH;
		st_gsfdm_historic_entry *e;

		rewind( fd_historic_file );					// opened in append mode : at the end
		while( getline( &str, &size, fd_historic_file )  != -1 )
		{	e = st_historic_entry_new();
			
			// getline don't remove '\n', search, in case or previously error
			if( ( ptr = strchr( str, '\n' ) ) )
				*ptr = 0;

			gsfdm_historic_st_get_url( e ) = g_strdup( str );

			// get the date,
			if( getline( &str, &size, fd_historic_file ) == -1 )
			{	 // well, there an error
				st_historic_entry_destroy( e );
				break;
			}
			
			// convert into numeric,
			gsfdm_historic_st_get_date( e ) = strtol( str, NULL, 10 );
			gsfdm_historic_add_entry( e );
			
			#ifdef DEBUG_HISTORIC_LOAD
			g_print( "---- debug historic : load %s\n", gsfdm_historic_st_get_url( e ) );
			#endif
		}
		
		g_free( str );
	}
}

/************************************************************************/
// ...
void		gsfdm_historic_save_conf( void )
{
	gnome_config_push_prefix( "/"PACKAGE"/historic/" );
	
	gnome_config_set_bool( "historic_use_history", historic_use_history );
	gnome_config_set_bool( "historic_use_specific", historic_use_specific );
	gnome_config_set_string( "historic_file_name", historic_file_name );
	gnome_config_set_int( "historic_length", historic_length );
	gnome_config_set_int( "historic_policy", historic_policy );
	
	if( fd_historic_file )
		fclose( fd_historic_file );
	
	gnome_config_pop_prefix();
	gnome_config_sync();
	
	// sync historic file : cut if too big
	if( fd_historic_file )
	{	
		if( gsfdm_historic_get_nb_entry() > historic_length )
		{	// shit... must cut on disk...
			gchar	*file = g_strconcat( historic_file_name, ".tmp", NULL );
			FILE	*tmp = fopen( file, "w" );
			gchar	*str = g_new( gchar, HISTORIC_FILE_LINE_LENGTH );
			size_t	size = HISTORIC_FILE_LINE_LENGTH;
			guint	line, to_skip = gsfdm_historic_get_nb_entry() - historic_length;
			
			// first, skip some lines
			for( line = 0; line < to_skip ; line++ )
			{	// doesn't need test, we have read ones yet
				getline( &str, &size, fd_historic_file );
				getline( &str, &size, fd_historic_file );
			}
			
			// now copy to tmp
			while( getline( &str, &size, fd_historic_file ) != -1 )
			{	fputs( str, tmp );
				getline( &str, &size, fd_historic_file );
				fputs( str, tmp );
			}
			
			// final : loop files
			fclose( tmp );
			unlink( historic_file_name );
			rename( file, historic_file_name );
			
			g_free( file );
			g_free( str );
		}
		
		fclose( fd_historic_file );
	}
}

/************************************************************************/
// add url to historic, return FALSE if can't i.e. cancel add of this url
gboolean	gsfdm_historic_push_url( gchar *url )
{
st_gsfdm_historic_entry *e;
	
	// don't use history, don't care, don't add
	if( !historic_use_history )
		return( TRUE );
	
	#ifdef DEBUG_HISTORIC_PUSH_URL
	g_print( "---- debug historic : push %s - hist is %d\n", url, gsfdm_historic_get_nb_entry() );
	#endif
	
	// check if on list
	if( ( e = gsfdm_historic_found_entry_by_url( url ) ) )
	{	// ... in list : action...
		
		#ifdef DEBUG_HISTORIC_PUSH_URL
		g_print( "---- debug historic : %s in historic\n", url );
		#endif
		
		switch( historic_policy )
		{	// remove old, add new ; add remove first in dlg
			case GSFDM_HISTORIC_REPLACE :
				gsfdm_historic_remove_from_dlg( e );
				
				#ifdef DEBUG_HISTORIC_PUSH_URL
				g_print( "---- debug historic : overide %s\n", url );
				#endif

				break;
			
			// ask user
			case GSFDM_HISTORIC_ASK :
			{	gchar	*tmp = g_strconcat( url, _( "Found in History, Overide ???" ), NULL );
				GnomeDialog	*d = GNOME_DIALOG( gnome_ok_cancel_dialog( tmp, NULL, NULL ) );
			
				gtk_window_set_position( GTK_WINDOW( d ), GTK_WIN_POS_MOUSE );
				
				if( !gnome_dialog_run_and_close( d ) )
				{	// ok, overide
					g_free( tmp );
					break;
				}		
					
				// don't overide, skip ---> to GSFDM_HISTORIC_SKIP
				g_free( tmp );
			}
			
			// don't add
			case GSFDM_HISTORIC_SKIP :
				#ifdef DEBUG_HISTORIC_PUSH_URL
				g_print( "---- debug historic : skip %s\n", url );
				#endif
				
				return( FALSE );
				break;
		}	
	}
	
	e = st_historic_entry_new( );
	gsfdm_historic_st_get_url( e ) = g_strdup( url );
	gsfdm_historic_st_get_date( e ) = time( NULL );
	
	gsfdm_historic_add_entry( e );
	gsfdm_historic_add_to_dlg( e );
	
	// add to historic file
	if( fd_historic_file )
	{	fprintf( fd_historic_file, "%s\n%ld\n", gsfdm_historic_st_get_url( e ), gsfdm_historic_st_get_date( e ) );
		fflush( fd_historic_file );
	}
		
	#ifdef DEBUG_HISTORIC_PUSH_URL
	g_print( "---- debug historic : ADDED %s\n", url );
	#endif

	return( TRUE );
}

/************************************************************************/
// add to dlg, finding poss according to dlg spec
static void	gsfdm_historic_add_to_dlg( st_gsfdm_historic_entry *e )
{
static gchar str[30];
gchar	*txt[] = { NULL, NULL, NULL };
guint	row;
time_t	t;

	if( !dlg )	return;
	
	t = gsfdm_historic_st_get_date( e );
	strftime( str, 30, HISTORIC_DATE_TEMPLATE, localtime( &t ) );
	txt[ HISTORIC_COL_DATE ] = str;
	txt[ HISTORIC_COL_URL ] = gsfdm_historic_st_get_url( e );
	
	row = gtk_clist_prepend( list_widget, txt );
	gtk_clist_set_row_data( list_widget, row, (gpointer)gsfdm_historic_st_get_date( e ) );
	
	#ifdef DEBUG_HISTORIC_DLG
	g_print( "---- debug historic : %s in dlg\n", gsfdm_historic_st_get_url( e ) );
	#endif
}

/************************************************************************/
// remove from dlg ; doesn't care about entry list
static void	gsfdm_historic_remove_from_dlg( st_gsfdm_historic_entry *e )
{
guint	row;
gchar	*txt;
	
	if( !dlg )	return;
	
	for( row = 0 ; row < list_widget->rows ; row++ )
	{	gtk_clist_get_text( list_widget, row, HISTORIC_COL_URL, &txt );
		
		if( !strcmp( txt, gsfdm_historic_st_get_url( e ) ) )
		{	gtk_clist_remove( list_widget, row );
			return;
		}
	}
	
}

/************************************************************************/
static void gsfdm_historic_real_clear( GnomeDialog *d )
{	gsfdm_historic_remove_all_entry();
	gtk_clist_clear( list_widget );
	gtk_widget_destroy( GTK_WIDGET( d ) );
}

/************************************************************************/
static void gsfdm_historic_clear( GnomeDialog *d )
{
GnomeDialog	*d2;
	
	d2 = GNOME_DIALOG( gnome_ok_dialog( _( "Really remove all history entry ???" ) ) );
	gnome_dialog_button_connect( d2, 0 , GTK_SIGNAL_FUNC( gsfdm_historic_real_clear ), NULL );
	gnome_dialog_button_connect_object( d2, 1, GTK_SIGNAL_FUNC( gtk_widget_destroy ), GTK_OBJECT( d2 ) );
	gnome_dialog_close_hides( d2, FALSE );
}

/************************************************************************/
static void gsfdm_historic_clear_sel( GnomeDialog *d )
{
GList	*l, *tmp;
gchar	*txt;
	
	if( !list_widget->selection )
	{	GtkWidget	*d2 = gnome_ok_dialog_parented( _( " Select something first..." ), GTK_WINDOW( d ) );
		
		gtk_widget_show( d2 );
		return;
	}
	
	// remove all selected, begin at the end - preserve index - make a copy,
	// when removing row, glist adr bound to change
	l = g_list_last( tmp = g_list_copy( list_widget->selection ) );
	
	gtk_clist_freeze( list_widget );
	while( l )
	{	gtk_clist_get_text( list_widget, (guint)( l->data ), HISTORIC_COL_URL, &txt );
		gsfdm_historic_remove_entry( txt );
		gtk_clist_remove( list_widget, (guint)( l->data ) );
		l = g_list_previous( l );
	}
	g_list_free( tmp );
	gtk_clist_thaw( list_widget );
}

/************************************************************************/
static void gsfdm_historic_down( GnomeDialog *d )
{	gtk_adjustment_set_value( list_widget->vadjustment, list_widget->vadjustment->upper );
}

/************************************************************************/
static void fill_list_widget( gpointer key, gpointer value, gpointer user_data )
{	gsfdm_historic_add_to_dlg( ST_GSFDM_HISTORIC_ENTRY( value ) );
}

static void gsfdm_historic_sort( GtkCList *w, guint c )
{
	gtk_clist_set_sort_column( w, c );
	gtk_clist_sort( w );
}  

/************************************************************************/
// popup the historic 
void		gsfdm_historic_open_historic( void )
{
gchar	*title[] = { N_( "Date" ), N_( "Url" ), NULL };
GtkScrolledWindow	*sc;

	if( !dlg )
	{	dlg = GNOME_DIALOG( gnome_dialog_new( _( "History" ), 
			GNOME_STOCK_PIXMAP_CLEAR, GNOME_STOCK_PIXMAP_TRASH, GNOME_STOCK_BUTTON_DOWN, GNOME_STOCK_BUTTON_CLOSE, NULL ) );
		gnome_dialog_close_hides( dlg, TRUE );
		gtk_window_set_policy( GTK_WINDOW( dlg ), TRUE, TRUE, TRUE );
 		gnome_dialog_set_default( dlg, HISTORIC_BP_CLOSE );
		
		gnome_dialog_button_connect( dlg, HISTORIC_BP_CLEAR , GTK_SIGNAL_FUNC( gsfdm_historic_clear ), NULL );
		gnome_dialog_button_connect( dlg, HSITORIC_BP_CLEAR_SEL, GTK_SIGNAL_FUNC( gsfdm_historic_clear_sel ), NULL );
		gnome_dialog_button_connect( dlg, HISTORIC_BP_DOWN, GTK_SIGNAL_FUNC( gsfdm_historic_down ), NULL );
		gnome_dialog_button_connect_object( dlg, HISTORIC_BP_CLOSE, GTK_SIGNAL_FUNC( gtk_widget_hide ), GTK_OBJECT( dlg ) );
		
		gtk_scrolled_window_set_policy( ( sc = GTK_SCROLLED_WINDOW( gtk_scrolled_window_new( NULL, NULL ) ) ),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
//		gtk_widget_set_usize( GTK_WIDGET( sc ), 150, 100 );		

		// set clist policy and events
		list_widget = GTK_CLIST( gtk_clist_new_with_titles( 2, title ) );
		gtk_container_add( GTK_CONTAINER( sc ), GTK_WIDGET( list_widget ) );
		gtk_clist_set_column_auto_resize( list_widget, HISTORIC_COL_DATE, TRUE );
		gtk_clist_set_column_auto_resize( list_widget, HISTORIC_COL_URL, TRUE );
		gtk_clist_set_selection_mode( list_widget, GTK_SELECTION_EXTENDED );
		gtk_clist_set_sort_type( list_widget, GTK_SORT_ASCENDING );
		gtk_clist_set_sort_column( list_widget, HISTORIC_COL_DATE );
		gtk_clist_set_auto_sort( list_widget, TRUE );
		
		gtk_signal_connect( GTK_OBJECT( list_widget ), "click-column", GTK_SIGNAL_FUNC( gsfdm_historic_sort ), NULL );
		
		gtk_box_pack_start( GTK_BOX( dlg->vbox ), GTK_WIDGET( sc ), TRUE, TRUE, 5 );
				
		// fill list with current historic
		gtk_clist_freeze( list_widget );
		gsfdm_historic_foreach_entry( (GHFunc)fill_list_widget, NULL );
		gtk_clist_thaw( list_widget );
		
		gtk_widget_show_all( dlg->vbox );
	}
	
	gtk_widget_show( GTK_WIDGET( dlg ) );
}
