/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		Header for plugin operations

/************************************************************************/

#include	<gnome.h>
#include	<gnome_path.h>
#include	<unistd.h>
#include	"gsfdm_info.h"
#include	"main.h"
#include	"gsfdm_url.h"
#include	"gsfdm_message.h"
#include	"gsfdm_plugin.h"

/************************************************************************/
//	static function definition
/************************************************************************/
static gboolean name_callback( GtkWidget *e, gpointer data );
static void path_callback( GnomePath *p, GnomePathChangeEnum change, gpointer user_data );
static void auto_dl_callback( GtkToggleButton *bp, gpointer d );
static void dock_callback( GtkButton *bp, gpointer d );

// apparently, there no function to parse a time in second into 
// a string : hh:mm:ss ... funny
static gchar const *make_time( glong time );

/************************************************************************/
//	static variables definition
/************************************************************************/
static GtkTable	*w = NULL;						// main container
static GtkLabel	*host;							// host from downloading
static GtkEntry	*name;							// name of file
static GnomePath *path;							// where to write
static GtkProgress	*progress;					// data retrieved
static GtkLabel	*estimated;						// time remain
static GtkLabel	*pid;							// pid of child
static GtkLabel *time_w;							// time from start

static GnomePixmap	*icon;						// mime type
static GnomePixmap	*state;						// state type
static GtkLabel		*auto_dl_text;				// auto down switch
static GtkButton	*auto_dl_bp;

static GtkButton	*dock;						// dock, undock button

#include	"../pixmaps/gsfdm.xpm"
#include	"../pixmaps/wait.xpm"
#include	"../pixmaps/running.xpm"
#include	"../pixmaps/done.xpm"
#include	"../pixmaps/partial.xpm"
#include	"../pixmaps/dead.xpm"
#include	"../pixmaps/unable.xpm"

// id of current displayed entry ; because every "module" read data from
// the same struct, the url_list "module", that is the real owner of these data
// could have removed an entry, so destroyed this struct --> incorrect mem data
// if read directly
static guint		current_url = 0;	// id of current url

// data for the path box
GtkFileSelection	*path_sel = NULL;

#define	GSFDM_INFO_PROGRESS_FORMAT	"%%p - %.2fKb / %.2fKb"

/************************************************************************/
//	functions definition
/************************************************************************/

/************************************************************************/
// return the top widget of the info box
/************************************************************************/
GtkWidget const	*gsfdm_info_get_dock( void )
{
GtkWidget	*f;
	
	// if no widget, create ( and initialize ) the box first
	if( !w )
	{
		w = GTK_TABLE( gtk_table_new( 9, 3, FALSE ) );
		// tip of the day : try without and you'll understand the use of this line
		gtk_widget_ref( GTK_WIDGET( w ) );

		gtk_table_set_row_spacings( w, 5 );
		gtk_table_set_col_spacings( w, 5 );

		gtk_table_attach( w, GTK_WIDGET( icon = GNOME_PIXMAP( gnome_pixmap_new_from_xpm_d( gsfdm_xpm ) ) ), 0, 1, 0, 1, 0,0,0,0 );
		gtk_table_attach( w, gtk_label_new( _( "Save In" ) ), 0, 1, 1, 2, 0,0,0,0 );
		gtk_table_attach( w, gtk_label_new( _( "Host" ) ), 0, 1, 2, 3, 0,0,0,0 );
		gtk_table_attach( w, GTK_WIDGET( name = GTK_ENTRY( gtk_entry_new() ) ), 1, 2, 0, 1, GTK_FILL | GTK_EXPAND,0,0,0 );
		gtk_table_attach( w, GTK_WIDGET( path = GNOME_PATH( gnome_path_new() ) ), 1, 2, 1, 2, GTK_FILL | GTK_EXPAND,0,0,0 );
		gtk_table_attach( w, GTK_WIDGET( host = GTK_LABEL( gtk_label_new( "" ) ) ), 1, 2, 2, 3, GTK_FILL | GTK_EXPAND,0,0,0 );
		
		gtk_table_attach( w, GTK_WIDGET( state = GNOME_PIXMAP( gnome_pixmap_new_from_xpm_d( gsfdm_xpm ) ) ), 2, 3, 0, 1, 0,0,0,0 );
		gtk_widget_set_sensitive( GTK_WIDGET( state ), FALSE );
		f = gtk_frame_new( "pid" );
		gtk_container_add( GTK_CONTAINER( f ), GTK_WIDGET( pid = GTK_LABEL( gtk_label_new( "" ) ) ) );
		gtk_widget_set_usize( GTK_WIDGET( f ), 48, -1 );
		gtk_table_attach( w, f, 2, 3, 1, 2, 0,0,0,0 );
		
		gtk_table_attach( w, gtk_hseparator_new(), 0, 3, 3, 4, GTK_FILL | GTK_EXPAND, 0,0,0 );
		
		gtk_table_attach( w, gtk_label_new( _( "Download" ) ), 0, 1, 4, 5, 0,0,0,0 );
		gtk_table_attach( w, gtk_label_new( _( "Estimed" ) ), 0, 1, 5, 6, 0,0,0,0 );
		gtk_table_attach( w, gtk_label_new( _( "Time" ) ), 0, 1, 6, 7, 0,0,0,0 );
		gtk_table_attach( w, GTK_WIDGET( progress = GTK_PROGRESS( gtk_progress_bar_new() ) ), 1, 3, 4, 5, GTK_FILL | GTK_EXPAND, 0,0,0 );
		gtk_table_attach( w, GTK_WIDGET( estimated = GTK_LABEL( gtk_label_new( "xx:xx:xx" ) ) ), 1, 3, 5, 6, 0,0,0,0 );
		gtk_table_attach( w, GTK_WIDGET( time_w = GTK_LABEL( gtk_label_new( "" ) ) ), 1, 3, 6, 7, 0,0,0,0 );
		
		gtk_table_attach( w, gtk_hseparator_new(), 0, 3, 7, 8, GTK_FILL | GTK_EXPAND,0,0,0 );
		
		// why a button instead a check button ??? : simple , with a butoon i can set the text i.e. the 
		// state WITHOUT sending the clicked signal... try you soon know why this pissed me off...
		gtk_table_attach( w, GTK_WIDGET( auto_dl_bp = GTK_BUTTON( 
			gtk_button_new_with_label( _( "Auto-Download is OFF" ) ) ) ), 0, 2, 8, 9, GTK_FILL | GTK_EXPAND,0,0,0 );
		auto_dl_text = GTK_LABEL( gtk_container_children( GTK_CONTAINER( auto_dl_bp ) )->data );
	
		gtk_table_attach( w, GTK_WIDGET( dock = GTK_BUTTON( gnome_pixmap_button(
			gnome_stock_pixmap_widget( NULL, GNOME_STOCK_PIXMAP_CONVERT ), _( "Un/Dock" ) ) ) ), 2, 3, 8, 9, 0,0,0,0 );
						
		// connect signal
		gtk_signal_connect( GTK_OBJECT( name ), "button-press-event", GTK_SIGNAL_FUNC( name_callback ), NULL );
		gtk_signal_connect( GTK_OBJECT( path ), "path_changed", GTK_SIGNAL_FUNC( path_callback ), (gpointer)w );

		gtk_signal_connect( GTK_OBJECT( auto_dl_bp ), "clicked", GTK_SIGNAL_FUNC( auto_dl_callback ), NULL );
		gtk_signal_connect( GTK_OBJECT( dock ), "clicked", GTK_SIGNAL_FUNC( dock_callback ), NULL );

		// set the default state
		gtk_widget_set_sensitive( GTK_WIDGET( name ), FALSE );
		gtk_widget_set_sensitive( GTK_WIDGET( path ), FALSE );
		gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), FALSE );
	 	gtk_entry_set_editable( name, FALSE );	
		
		// well, the total size is not always no, must change the config each time...
//		gtk_progress_set_format_string( progress, GSFDM_INFO_PROGRESS_FORMAT );
		gtk_progress_set_show_text( progress, TRUE );
				
		gtk_widget_show_all( GTK_WIDGET( w ) );
	}
	
	return( GTK_WIDGET( w ) );
}

/************************************************************************/
// display info for this id, if 0, clear
/************************************************************************/
void	gsfdm_info_display_info( gint entry_id )
{
st_gsfdm_url_entry	const *s = gsfdm_url_get_url_data( entry_id );
gchar				*tmp;
gchar 				**icon_data;
	
	// test if this id exit, perhaps...
	if( !s )
		return;											// something wrong...
	
	// ok, get
	current_url = entry_id;

	// fill all field
	gtk_entry_set_text( name, gsfdm_url_st_get_name( s ) );
	gtk_label_set_text( host, gsfdm_url_st_get_host( s ) );
	gnome_path_set_path( path, gsfdm_url_st_get_path( s ), FALSE );
	
	if( gsfdm_url_st_get_auto_dl( s ) )
		gtk_label_set_text( auto_dl_text, _( "Auto-Download is ON" ) );
	else
		gtk_label_set_text( auto_dl_text, _( "Auto-Download is OFF" ) );
	
	if( gsfdm_url_st_get_pid( s ) )
	{	gtk_label_set_text( pid, ( tmp = g_strdup_printf( "%i", gsfdm_url_st_get_pid( s ) ) ) );
		g_free( tmp );
	}	
	else
	{	gtk_label_set_text( pid, "" );
	}
	
	// set the progess and other size info
	gsfdm_info_inform_on_update( entry_id );
	
	// set the mime type icon
	icon_data = gsfdm_plugin_call( gsfdm_url_st_get_plug_id( s ), PLUG_GET_ICON, NULL );
	if( icon_data )
		gnome_pixmap_load_xpm_d( icon, icon_data );
	else
		gnome_pixmap_load_xpm_d( icon, gsfdm_xpm );
	
	// set the state icon, and fix the state of editable widget
	switch( gsfdm_url_st_get_state( s ) ) {
		case GSFDM_URL_STATE_WAIT :
			gnome_pixmap_load_xpm_d( state, wait_xpm );
			gtk_widget_set_sensitive( GTK_WIDGET( name ), TRUE );
			gtk_widget_set_sensitive( GTK_WIDGET( path ), TRUE );
			gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), TRUE );
			break;
		case GSFDM_URL_STATE_RUNNING :
			gnome_pixmap_load_xpm_d( state, running_xpm );
			gtk_widget_set_sensitive( GTK_WIDGET( name ), FALSE );
			gtk_widget_set_sensitive( GTK_WIDGET( path ), FALSE );
			gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), FALSE );
			break;
		case GSFDM_URL_STATE_DONE :
			gnome_pixmap_load_xpm_d( state, done_xpm );
			gtk_widget_set_sensitive( GTK_WIDGET( name ), FALSE );
			gtk_widget_set_sensitive( GTK_WIDGET( path ), FALSE );
			gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), FALSE );
			break;
		case GSFDM_URL_STATE_DEAD :
			gnome_pixmap_load_xpm_d( state, dead_xpm );
			gtk_widget_set_sensitive( GTK_WIDGET( name ), TRUE );
			gtk_widget_set_sensitive( GTK_WIDGET( path ), TRUE );
			gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), TRUE );
			break;
		case GSFDM_URL_STATE_PARTIAL :
			gnome_pixmap_load_xpm_d( state, partial_xpm );
			gtk_widget_set_sensitive( GTK_WIDGET( name ), FALSE );
			gtk_widget_set_sensitive( GTK_WIDGET( path ), FALSE );
			gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), TRUE );
			break;
		case GSFDM_URL_STATE_UNABLE :
			gnome_pixmap_load_xpm_d( state, unable_xpm );
			gtk_widget_set_sensitive( GTK_WIDGET( name ), TRUE );
			gtk_widget_set_sensitive( GTK_WIDGET( path ), TRUE );
			gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), TRUE );
			break;
		default :
	}
	
	// set the progess and other size info
	gsfdm_info_inform_on_update( entry_id );
}

/************************************************************************/
// infor that this id is no more selected
/************************************************************************/
void	gsfdm_info_undisplay_info( gint entry_id )
{
	if( !current_url )
		return;
	
	if( entry_id != current_url )
		return;											// don't care of this id

	current_url = 0;

	// ok, clean all widgets
	gtk_entry_set_text( name, "" );
	gnome_path_set_path( path, "", FALSE );
	
	gtk_label_set_text( host, "" );
	gtk_label_set_text( estimated, "xx:xx:xx" );
	gtk_label_set_text( pid, "" );
	gtk_label_set_text( time_w, "" );
	gnome_pixmap_load_xpm_d( state, gsfdm_xpm );
	gnome_pixmap_load_xpm_d( icon, gsfdm_xpm );
	gtk_progress_set_percentage( progress, 0.0 );
	
	// and unactive all
	gtk_widget_set_sensitive( GTK_WIDGET( name ), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( path ), FALSE );
	gtk_widget_set_sensitive( GTK_WIDGET( auto_dl_bp ), FALSE );
}

/************************************************************************/
// infor the info "block" that the state, data for this id has just changed
/************************************************************************/
void	gsfdm_info_inform_for_change( gint entry_id )
{
	if( !current_url )
		return;
	if( entry_id != current_url )
		return;											// don't care of this id
	
	gsfdm_info_display_info( entry_id );				// ok update all field, according to new state / value
}

/************************************************************************/
// infor the info "block" that this id has been deleted from list
/************************************************************************/
void	gsfdm_info_inform_for_deleted( gint entry_id )
{
	if( !current_url )	return;
	if( current_url == entry_id )
		gsfdm_info_undisplay_info( entry_id );
}

/************************************************************************/
// infor the info "block" that the running filed for this id has been updated
/************************************************************************/
void	gsfdm_info_inform_on_update( gint entry_id )
{
gchar				*tmp;
st_gsfdm_url_entry	const *s;
	
	if( !current_url )	return;
		
	// is it displayed ???
	if( current_url != entry_id )
		return;											// don't care
	
	s = gsfdm_url_get_url_data( entry_id );

	// perhaps must set the host name...
	if( gsfdm_url_st_get_host( s ) )
		gtk_label_set_text( host, gsfdm_url_st_get_host( s ) );

	// set the progess. 
	// set the string each time instead of using adj. upper may be unknow
	gtk_progress_set_format_string( progress, 
		( tmp = g_strdup_printf( GSFDM_INFO_PROGRESS_FORMAT, (gfloat)gsfdm_url_st_get_size( s ) / 1024.0,
		(gfloat)gsfdm_url_st_get_total( s ) / 1024.0 ) ) );
	g_free( tmp );
	
	// there aparently a fucking bug with 1.0
	if( gsfdm_url_st_get_percent( s ) < 1.0 )
		gtk_progress_set_percentage( progress, gsfdm_url_st_get_percent( s ) );
	else
		gtk_progress_set_percentage( progress, 0.9999 );
	
	gtk_label_set_text( time_w, make_time( gsfdm_url_st_get_time( s ) ) );

	if( ( gsfdm_url_st_get_state( s ) == GSFDM_URL_STATE_RUNNING ) && gsfdm_url_st_get_percent( s ) != 0.0 )
		gtk_label_set_text( estimated, make_time( (glong)
			( (gfloat)( gsfdm_url_st_get_time( s ) / gsfdm_url_st_get_percent( s ) ) ) ) );
	else
		gtk_label_set_text( estimated, "xx:xx:xx" );
}

/************************************************************************/
//	callback when ckick to change the name of the file...
/************************************************************************/
static gboolean name_callback( GtkWidget *e, gpointer data )
{
// simple callback to a gnome_request_dialog
static void callback( gchar *string, gpointer data ) { *(gchar **)data = string; }
gchar 				*c_name = NULL, *tmp, *tmp2;
st_gsfdm_url_entry	const *s;

	if( !current_url )
	{	gtk_signal_emit_stop_by_name( GTK_OBJECT( e ), "button_press_event" );
		return( TRUE );
	}

	if( !( s = gsfdm_url_get_url_data( current_url ) ) )
	{	current_url = 0;
		gtk_signal_emit_stop_by_name( GTK_OBJECT( e ), "button_press_event" );
		return( TRUE );
	}	
	
	// test first if allowed to change the name
	if( !gsfdm_url_can_edit( s ) )
	{	gtk_signal_emit_stop_by_name( GTK_OBJECT( e ), "button_press_event" );
		return( TRUE );											// can't edit for now
	}
		
	// open a request box, 
	if( !gnome_dialog_run_and_close( GNOME_DIALOG( gnome_request_dialog( FALSE, _( "Enter a new file name" ),
		gsfdm_url_st_get_name( s ), 256, (GnomeStringCallback)callback, (gpointer)&c_name, 
        GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( e ) ) ) ) ) ) )
	{	// ok, continue, still want to change the name
		tmp = g_strconcat( gsfdm_url_st_get_path( s ), "/", c_name, NULL );
		
		// test if the file exist, prompt for overwrite
		switch( access( tmp, F_OK ) )
		{	// file exist, ask if overwrite
			case 0 :
				// ovewrite
				if( !gnome_dialog_run_and_close( GNOME_DIALOG( 
					gnome_question_dialog_parented( _( "FILE exist OVERWRITE ???" ), NULL, NULL, 
					GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( e ) ) ) ) ) ) )
				{	unlink( tmp );							// remove the file
					tmp2 = g_strdup_printf( "File %s deleted for further write\n", tmp );
						gsfdm_message_push_message( tmp2, FALSE, FALSE, TRUE );
					g_free( tmp2 );
				// exit
				} else {
					g_free( c_name );
					break;
				}
			
			// don't exist, change
			default:
				g_free( gsfdm_url_st_get_name( s ) );		// perform an "hard change"
				gsfdm_url_st_get_name( s ) = c_name;		// could, id locked
				gsfdm_url_data_changed( current_url );		// inform "url block" of change
		}

		g_free( tmp );

	// abort name change
	} else {
		if( c_name ) g_free( c_name );
	}
	
	gsfdm_url_release_edit( s );
	gtk_signal_emit_stop_by_name( GTK_OBJECT( e ), "button_press_event" );
	return( TRUE );
}

/************************************************************************/
//	callback when click to change the destination dir for this file
/************************************************************************/
static void path_callback( GnomePath *p, GnomePathChangeEnum change, gpointer user_data )
{
st_gsfdm_url_entry	const *s;

	if( !current_url )
		return;
	
	s = gsfdm_url_get_url_data( current_url );
	
	switch( change )
	{	case GNOME_PATH_CHANGE_TO_NEW :
		case GNOME_PATH_CHANGE_TO_CHILD :
		case GNOME_PATH_CHANGE_TO_PARENT :
			g_free( gsfdm_url_st_get_path( s ) );
			gsfdm_url_st_get_path( s ) = gnome_path_get_path( p );
			break;
        case GNOME_PATH_CHANGE_NO_CHANGE :
	}
}

/************************************************************************/
//	callback when change the auto download flag
/************************************************************************/
static void auto_dl_callback( GtkToggleButton *bp, gpointer d )
{
st_gsfdm_url_entry	const *s;
	
	if( !current_url )	return;
		
	s = gsfdm_url_get_url_data( current_url );	
	
	if( gsfdm_url_st_get_auto_dl( s ) )
		gtk_label_set_text( auto_dl_text, _( "Auto-Download is OFF" ) );
	else
		gtk_label_set_text( auto_dl_text, _( "Auto-Download is ON" ) );

	gsfdm_url_st_get_auto_dl( s ) = !gsfdm_url_st_get_auto_dl( s );

	gsfdm_url_data_changed( current_url );
}

/************************************************************************/
// call for a dock, undock op
/************************************************************************/
static void dock_callback( GtkButton *bp, gpointer d )
{	gsfdm_main_dock_op( MAIN_DOCK_INFO );
}

/************************************************************************/
//	return a const string that match the time (sec) into a readable form
/************************************************************************/
static gchar const *make_time( glong time )
{
static gchar	t[10];
guint		hour, min, sec;
	
	hour = (guint)( time / 3600 );
	min = (guint)( ( time - ( hour * 3600 ) ) / 60 );
	sec = (guint)( time - ( min * 60 ) );
	
	snprintf( t, 10, "%02u:%02u:%02u", hour, min, sec );
	
	return( t );
}
