/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		Header for opening and manipulate the log file

/************************************************************************/

#include	<gnome.h>
#include	<unistd.h>
#include	<config.h>
#include	"gsfdm_log.h"
#include	"gsfdm_message.h"
#include	"gsfdm_fileop.h"

/************************************************************************/
//	debug
/************************************************************************/
#ifdef HAVE_DEBUG
#undef	DEBUG_CONFIG_CREATE_LOG_BOX
#undef	DEBUG_CONFIG_INIT_LOG
#undef	DEBUG_LOG_PUSH_DATA
#endif

/************************************************************************/
//	static variables definition
/************************************************************************/
#define	GSFDM_LOG_DEFAULT_LOG_FILE_NAME "/gsfdm/gsfdm.log"

static	gboolean	log_initialized = FALSE;
static	gboolean	use_log_file = TRUE;
static	gboolean	use_specific_log_file = FALSE;
static	gchar		*user_log_file = NULL;
static	gchar		*log_file_name = NULL;

static	FILE		*fd_file_name = NULL;

static	GnomeDialog	*log_dlg = NULL;
static	GnomeLess	*less_widget;

/************************************************************************/
//	functions definition
/************************************************************************/

/************************************************************************/
// init param, open log file... save some test, some cpu time
/************************************************************************/
gboolean gsfdm_log_init( void )
{
	log_initialized = TRUE;
	
	#ifdef	DEBUG_CONFIG_INIT_LOG
	g_print( "debug : initialise log data\n");
	#endif
	
	log_file_name = g_strconcat( g_get_home_dir(), GSFDM_LOG_DEFAULT_LOG_FILE_NAME, NULL );
	
	// load config from file
	gnome_config_push_prefix( "/"PACKAGE"/log/" );
	use_log_file = gnome_config_get_bool( "use_log_file=true" );
	use_specific_log_file = gnome_config_get_bool( "use_specific_log_file=false" );
	user_log_file = gnome_config_get_string( 
		g_strconcat( "user_log_file=", g_get_home_dir(), GSFDM_LOG_DEFAULT_LOG_FILE_NAME, NULL ) );
	gnome_config_pop_prefix();
	
	if( use_specific_log_file )
		log_file_name = user_log_file;
	
	// open the log file for writting ( append )
	if( !( fd_file_name = fopen( log_file_name, "a+" ) ) )
	{	gsfdm_message_push_message( 
			_( "Failing to open log file, coming back to default : ~/"GSFDM_LOG_DEFAULT_LOG_FILE_NAME ), FALSE, TRUE, FALSE );
		
		log_file_name = g_strconcat( g_get_home_dir(), GSFDM_LOG_DEFAULT_LOG_FILE_NAME, NULL );
		if( !( fd_file_name = fopen( log_file_name, "a+" ) ) )
		{	gsfdm_message_push_message( _( "Fail to create default log file, Use of log file disabled..." ), FALSE, TRUE, FALSE );
			use_log_file = FALSE;						// fail to create, write
			return( FALSE );
		}
	}
	
	#ifdef	DEBUG_CONFIG_INIT_LOG
	g_print( "debug : log initilalize with %d %d %s - %s\n", use_log_file,use_specific_log_file, user_log_file, log_file_name );
	#endif
	
	return( TRUE );
}

/************************************************************************/
// close ( e.i. close stream ) and save config
void gsfdm_log_close( void )
{
	gnome_config_push_prefix( "/"PACKAGE"/log/" );
	gnome_config_set_bool( "use_log_file", use_log_file );
	gnome_config_set_bool( "use_specific_log_file", use_specific_log_file );
	gnome_config_set_string( "user_log_file", user_log_file );
	gnome_config_pop_prefix();
	gnome_config_sync();
	
	log_initialized = FALSE;
	fclose( fd_file_name );
}

/************************************************************************/
static	void	check_bp_switcher( GtkToggleButton *bp, gpointer to_switch )
{	*( (gboolean *)to_switch ) = gtk_toggle_button_get_active( bp );
}

/************************************************************************/
static void	filesel_ok_callback( GtkWidget *dlg )
{
gchar	*file = gtk_file_selection_get_filename( GTK_FILE_SELECTION( dlg ) );
FILE	*fd;
	
	if( file )
	{	// first, test if exit, prompt if agree to truncate
		switch( gsfdm_file_get_file_type( file ) )
		{	case GSFDM_FILE_TYPE_REGULAR :
				if( gnome_dialog_run_and_close( GNOME_DIALOG( 
					gnome_question_dialog_parented( _( "Will erase this file, Agree ??" ),
					NULL, NULL, GTK_WINDOW( dlg ) ) ) ) )
					break;							// user not agree

			// the case of error is : don't exist yet, access denied
			case GSFDM_FILE_TYPE_ERROR :
				// able to open it ??
				if( ( fd = fopen( file, "w+" ) ) )
				{	// close current, open new
					fclose( fd_file_name );
					fd_file_name = fd;
					log_file_name = g_strdup( file );
					
					// update the config widget
					gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( dlg ), "config_entry_widget" ) ), file );
					break;
				}
				
			default :
				gtk_widget_show( gnome_warning_dialog_parented( _( "Unsupported file type, Abort" ), GTK_WINDOW( dlg ) ) );
		}
	}
	
	gtk_widget_destroy( dlg );
}

/************************************************************************/
static void	new_file_name_callback( GtkWidget *parent )
{
GtkWidget	*dlg;
	
	dlg = gtk_file_selection_new( _( "New log file name, selec or enter new name" ) );
 	gtk_window_set_position( GTK_WINDOW( dlg ), GTK_WIN_POS_MOUSE );
	gtk_object_set_data( GTK_OBJECT( dlg ), "config_entry_widget", (gpointer)parent );
	gtk_signal_connect_object( GTK_OBJECT( GTK_FILE_SELECTION( dlg )->ok_button ), "clicked", GTK_SIGNAL_FUNC( filesel_ok_callback ), GTK_OBJECT( dlg ) );
	
	gtk_widget_show( GTK_WIDGET( dlg ) );
}

/************************************************************************/
GtkWidget	*gsfdm_log_get_config_page( void )
{
GtkBox	*bh, *bv;
GtkWidget	*c;
	
	if( !log_initialized )
		gsfdm_log_init();
	
	#ifdef	DEBUG_CONFIG_CREATE_LOG_BOX
	g_print( "debug : going to create the log page : %d %d %s\n", use_log_file, use_specific_log_file, user_log_file );
	#endif
	
	bv = GTK_BOX( gtk_vbox_new( FALSE, 5 ) );
	
	gtk_box_pack_start( bv, ( c = gtk_check_button_new_with_label( _( "Use a log File" ) ) ), FALSE, FALSE, 0 );
	gtk_signal_connect_after( GTK_OBJECT( c ), "toggled", GTK_SIGNAL_FUNC( check_bp_switcher ), (gpointer)(&use_log_file) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ), use_log_file );

	gtk_box_pack_start( bv, GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 5 ) ) ), FALSE, FALSE, 5 );
	gtk_box_pack_start( bh, ( c = gtk_check_button_new_with_label( _( "Use Specific log file :" ) ) ), FALSE, FALSE, 0 );
	gtk_signal_connect_after( GTK_OBJECT( c ), "toggled", GTK_SIGNAL_FUNC( check_bp_switcher ), (gpointer)(&use_specific_log_file) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ), use_specific_log_file );
	
	gtk_box_pack_end( bh, ( c = gtk_entry_new( ) ), FALSE, FALSE, 5 );
	if( user_log_file )
		gtk_entry_set_text( GTK_ENTRY( c ), user_log_file );
	gtk_signal_connect_object( GTK_OBJECT( c ), "button-press-event", GTK_SIGNAL_FUNC( new_file_name_callback ), NULL );
	
	gtk_widget_show_all( GTK_WIDGET( bv ) );
	
	return( GTK_WIDGET( bv ) );
}

/************************************************************************/
// add a line to the log file
/************************************************************************/
void	gsfdm_log_push_line( gchar const *data )
{
	if( !use_log_file )
		return;
	
	if( !log_initialized )
		if( !gsfdm_log_init() )
			return;							// can't
		
	#ifdef	DEBUG_LOG_PUSH_DATA
	g_print( "debug ; log ; writing %s into %s\n", data, log_file_name );
	#endif
		
	// ok, push the data at the end
	fputs( data, fd_file_name );
	fputc( '\n', fd_file_name );
	fflush( fd_file_name );
		 
	// perhaps add to log dlg
	if( log_dlg )
	{	gtk_text_insert( GTK_TEXT( less_widget->text ), NULL, NULL, NULL, data, strlen( data ) );
	}
}

/************************************************************************/
static void log_clear( GnomeDialog *dlg )
{
	// clear widget, and file
	gnome_less_clear( less_widget );
	
	fclose( fd_file_name );
	fd_file_name = fopen( log_file_name, "w+" );
}

/************************************************************************/
static void log_down( GnomeDialog *dlg )
{
	gtk_adjustment_set_value( less_widget->text->vadj, less_widget->text->vadj->upper );
}

#define LOG_BP_CLEAR 0
#define LOG_BP_DOWN 1
#define LOG_BP_CLOSE 2
/************************************************************************/
// open the log file dlg
/************************************************************************/
void	gsfdm_log_open( void )
{
	if( !log_initialized && use_log_file )
		if( !gsfdm_log_init() )
			return;
	
	if( !log_dlg )
	{	log_dlg = GNOME_DIALOG( gnome_dialog_new( _( "Log file content" ), 
			GNOME_STOCK_PIXMAP_CLEAR, GNOME_STOCK_BUTTON_DOWN, GNOME_STOCK_BUTTON_CLOSE, NULL ) );
		gnome_dialog_close_hides( log_dlg, TRUE );
		gtk_window_set_policy( GTK_WINDOW( log_dlg ), TRUE, TRUE, TRUE );
 		gnome_dialog_set_default( log_dlg, LOG_BP_CLOSE );
		
		gnome_dialog_button_connect( log_dlg,LOG_BP_CLEAR , GTK_SIGNAL_FUNC( log_clear ), NULL );
		gnome_dialog_button_connect( log_dlg, LOG_BP_DOWN, GTK_SIGNAL_FUNC( log_down ), NULL );
		gnome_dialog_button_connect_object( log_dlg, LOG_BP_CLOSE, GTK_SIGNAL_FUNC( gtk_widget_hide ), GTK_OBJECT( log_dlg ) );
		
		gtk_box_pack_start( GTK_BOX( log_dlg->vbox ), GTK_WIDGET( less_widget = GNOME_LESS( gnome_less_new() ) ), FALSE, FALSE, 5 );
		gtk_text_set_word_wrap( GTK_TEXT( less_widget->text ), FALSE );
		gtk_text_set_line_wrap( GTK_TEXT( less_widget->text ), FALSE );
		gtk_widget_show( GTK_WIDGET( less_widget ) );

		gnome_less_show_file( less_widget, log_file_name );   
	}
	
	gtk_widget_show( GTK_WIDGET( log_dlg ) );
}
