/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		code for url / list manipulation

/************************************************************************/

#include	<gnome.h>
#include	<gdk/gdktypes.h>
#include	<unistd.h>
#include	<stdio.h>
#include	"gsfdm_url.h"
#include	"main.h"
#include	"gsfdm_info.h"
#include	"gsfdm_message.h"
#include	"gsfdm_plugin.h"
#include	"gsfdm_fileop.h"
#include	"gsfdm_config.h"
#include	"gsfdm_edit.h"
#include	"gsfdm_historic.h"

/************************************************************************/
//	define for debugging
/************************************************************************/
#ifdef HAVE_DEBUG
#define DEBUG_URL_ID
#define DEBUG_URL_MAIN_LOOP
//#define DEBUG_URL_COMMUNICATION					// always debug, whether pppd or not
//#define DEBUG_NO_UPDATE_CONFIG
#define	DEBUG_URL_PUSH
#define DEBUG_URL_LOAD_CONFIG
#define DEBUG_URL_ADD_CONFIG
#define DEBUG_URL_RM_CONFIG
#define DEBUG_URL_UPDATE_CONFIG
#define DEBUG_URL_REFRESH_CONFIG
#define	DEBUG_URL_ADD_DATA
#define DEBUG_URL_PUSH_URL
#define DEBUG_URL_REMOVE
#define DEBUG_URL_VECTOR
#define DEBUG_URL_START
#define DEBUG_URL_STOP
#define DEBUG_URL_SELEC_RECEIVE
#endif

/************************************************************************/
//	static function definition
/************************************************************************/
static gboolean gsfdm_url_main_loop( gpointer data );
static gboolean gsfdm_check_pppd( void );				// true = can start...

static void url_select_callback( GtkCList *cl, gint r, gint c, GdkEventButton *b, gpointer d );
static void url_unselect_callback( GtkCList *cl, gint r, gint c, GdkEventButton *b, gpointer d );

static gint url_click_cell_callback( GtkWidget *cl, GdkEventButton *e, gpointer d );
static void url_click_column_callback( GtkCList *cl, gint c, gpointer p );

static void url_click_up( GtkWidget *bp );
static void url_click_down( GtkWidget *bp );

static void dock_callback( GtkButton *bp, gpointer d );

// callback for copy / paste actions
static gint gsfdm_url_selection_clear( GtkWidget *widget, GdkEventSelection *event );
static void gsfdm_url_selection_handle( GtkWidget *widget, GtkSelectionData *selection_data, guint info, guint time_stamp );
static void gsfdm_url_selection_received( GtkWidget *widget, GtkSelectionData *selection_data );

// callback for dnd support
static void	gsfdm_url_drop_received( GtkWidget *cl, GdkDragContext *c, gint x, gint y,
    GtkSelectionData *s, guint i, guint t, gpointer d );
static void gsfdm_url_drag_send( GtkWidget *cl, GdkDragContext *context,
	GtkSelectionData *data, guint info, guint time, gpointer d );

// function for auto saving the url list content
#define gsfdm_url_add_to_config( id ) ( gsfdm_url_update_to_config( id ) )
static void gsfdm_url_update_to_config( guint id );
static void gsfdm_url_remove_from_config( guint id );
static void gsfdm_url_refresh_in_config( guint id );

// start downloading the url matching this id ; stop
static void gsfdm_url_start_id( guint id );
static void gsfdm_url_stop_id( guint id );

static void		gsfdm_url_percent_changed( gint id );
static void	gsfdm_url_id_state_changed( guint id );

// remove this url from the list ; return true on success
// alway remove, even with running process
static gboolean	gsfdm_url_remove_url( gint id );

// directly return data ( const ) of the next ( in fact first with auto_dl on )
// url in state WAIT
static st_gsfdm_url_entry *gsfdm_url_get_next_job( void );

// add a new entry from a disk file ; return true on success
static gboolean	gsfdm_url_push_url_from_file( gchar const *file, gboolean auto_dl, gboolean auto_rn );
// same but from an url ( http://...  mms://... ) ; return true on success
static gboolean	gsfdm_url_push_url_from_url( gchar const *url, gboolean auto_dl, gboolean auto_rn );
// add but from a struct : internal use a startup from config
static gboolean	gsfdm_url_push_url_from_data( st_gsfdm_url_entry *s );

static void	gsfdm_url_edit_url( void );
static void	gsfdm_url_select_all( void );
static void	gsfdm_url_select_none( void );

/************************************************************************/
//	static variables
/************************************************************************/
// data of context menu
static GnomeUIInfo	context_data[] = {
    GNOMEUIINFO_ITEM_STOCK( N_( "Start" ), NULL, &gsfdm_url_start, GNOME_STOCK_PIXMAP_EXEC ),
  	GNOMEUIINFO_ITEM_STOCK( N_( "Stop" ), NULL,	&gsfdm_url_stop, GNOME_STOCK_PIXMAP_STOP ),
	GNOMEUIINFO_SEPARATOR,
  	GNOMEUIINFO_ITEM_STOCK( N_( "Initialize" ), NULL, &gsfdm_url_init, GNOME_STOCK_PIXMAP_CLEAR ),
	GNOMEUIINFO_ITEM_STOCK( N_( "Restart" ), NULL,	&gsfdm_url_restart, GNOME_STOCK_PIXMAP_REFRESH ),
  	GNOMEUIINFO_ITEM_STOCK( N_( "Delete" ), NULL, &gsfdm_url_delete, GNOME_STOCK_PIXMAP_CLEAR ),
	GNOMEUIINFO_SEPARATOR,
   	GNOMEUIINFO_MENU_COPY_ITEM( &gsfdm_url_copy_url_list, NULL ),
	GNOMEUIINFO_MENU_PASTE_ITEM( &gsfdm_url_paste_url_list, NULL ),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_CLEAR_ITEM( &gsfdm_url_clear_url_list, NULL ),
	GNOMEUIINFO_MENU_SELECT_ALL_ITEM( &gsfdm_url_select_all, NULL ),
  	GNOMEUIINFO_ITEM_NONE( N_( "Select None" ), NULL, &gsfdm_url_select_none  ),
	
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK( N_( "Edit Source" ), NULL, &gsfdm_url_edit_url, GNOME_STOCK_PIXMAP_PROPERTIES ),
    GNOMEUIINFO_END
	};
   
// define how to display % in the list column
#define URL_PERCENT_FORMAT "%3.2f%%"
	
static GtkBox		*w = NULL;			// main container
static GtkCList		*list;				// the list of url
static GtkButton	*dock;
static GtkWidget	*app_up, *app_down;

#include	"../pixmaps/wait_low.xpm"
#include	"../pixmaps/running_low.xpm"
#include	"../pixmaps/done_low.xpm"
#include	"../pixmaps/partial_low.xpm"
#include	"../pixmaps/dead_low.xpm"
#include	"../pixmaps/unable_low.xpm"
#include	"../pixmaps/download.xpm"
#include	"../pixmaps/gsfdm.xpm"
#define		MINI_ICON_HEIGHT 24
	
static GdkPixmap	*running_low_pix, *done_low_pix, *partial_low_pix, *dead_low_pix, *wait_low_pix, *unable_low_pix;
static GdkBitmap	*running_low_bit, *done_low_bit, *partial_low_bit, *dead_low_bit, *wait_low_bit, *unable_low_bit;
	
// variable for job control
GArray		*job_stack = NULL;			// list of id currently running, NULL if none

// the timer is never stopped. I did first but this use very few cpu time.
// what ever the time is, the timer is use for update of data, not to manage :
// when as start / stop ... commands are send to plug at once
gint		timer_id;					// id of timer used
gboolean	stack_empty_flag = FALSE;

#define URL_BUFFER_LINE_LENGTH 1024
#define URL_BUFFER_TO_PLUG 5000

/************************************************************************/
//	functions definition
/************************************************************************/
// destroy ( free ) all data from a st_gsfdm_url_entry
void	gsfdm_url_st_destroy( st_gsfdm_url_entry *s )
{
	if( gsfdm_url_st_get_url( s ) )
		g_free( gsfdm_url_st_get_url( s ) );
	if( gsfdm_url_st_get_name( s ) )
		g_free( gsfdm_url_st_get_name( s ) );
	if( gsfdm_url_st_get_path( s ) )
		g_free( gsfdm_url_st_get_path( s ) );
	if( gsfdm_url_st_get_host( s ) )
		g_free( gsfdm_url_st_get_host( s ) );
	g_free( s );	
}

/************************************************************************/
// return a new unique id, parse the list first ( after tle fill by config )
// for finding the upper value, apply a mask in case of overflow
static gint gsfdm_url_get_new_id( void )
{
static guint	id = 0;
	
	// if 0 : this is the first call, look in clist first
	if( !id )
	{	guint		row;
		
		for( row=0 ; row < list->rows ; row++ )
			id = MAX( id, gsfdm_url_st_get_id( gtk_clist_get_row_data( list, row ) ) );
		
		#ifdef DEBUG_URL_ID
		g_print( "debug url : STARTUP ID IS : %d\n", id );
		#endif
	}
	
	id++;
	id &=0x7FFF;				// MAX is a macro not a function ( think why

	#ifdef DEBUG_URL_ID
	g_print( "debug url : CREATE new id : %d", MAX( id, 1 ) );
	#endif
	
	return( ( id = MAX( id, 1 ) ) );		// you MUST do this way )
}

/************************************************************************/
// true if can edit data for this entry, if true set edited flag to true ( lock entry... )
/************************************************************************/
gboolean	gsfdm_url_can_edit( st_gsfdm_url_entry *s )
{
	if( !gsfdm_url_st_get_edit( s ) && ( gsfdm_url_st_get_state( s ) != GSFDM_URL_STATE_RUNNING ) )
	{	gsfdm_url_st_get_edit( s ) = TRUE;
		return( TRUE );
	}
		
	return( FALSE );
}

/************************************************************************/
void		gsfdm_url_release_edit( st_gsfdm_url_entry *s )
{	gsfdm_url_st_get_edit( s ) = FALSE;
}

/************************************************************************/
//	return ( and create the main url container )
/************************************************************************/
#define	GSFDM_URL_COL_STATE 0
#define	GSFDM_URL_COL_PERCENT 1
#define	GSFDM_URL_COL_NAME 2
GtkWidget const	*gsfdm_url_get_dock( void )
{
gchar	*title[] = { N_( "S" ), "%", N_( "Name" ), NULL };
static	GtkTargetEntry default_target[] = {
	{ "STRING", 0, GDK_TARGET_STRING },
	{ "gtk-clist-drag-reorder", 0, 0 } };
#define DEFAULT_TARGET_SIZE 2
GtkScrolledWindow	*sc;
GtkBox	*b;

	// if no widget, create ( and initialize ) the box first
	if( !w )
	{	// the main container : a scroll widget list
		w = GTK_BOX( gtk_vbox_new( FALSE, 5 ) );

		// tip of the day : try without and you'll understand the use of this line
		gtk_widget_ref( GTK_WIDGET( w ) );
	
		gtk_scrolled_window_set_policy( ( sc = GTK_SCROLLED_WINDOW( gtk_scrolled_window_new( NULL, NULL ) ) ),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
		gtk_widget_set_usize( GTK_WIDGET( sc ), 150, 100 );		

		// set clist policy and events
		list = GTK_CLIST( gtk_clist_new_with_titles( 3, title ) );
		gtk_container_add( GTK_CONTAINER( sc ), GTK_WIDGET( list ) );
		gtk_clist_set_column_auto_resize( list, GSFDM_URL_COL_PERCENT, TRUE );
		gtk_clist_set_column_auto_resize( list, GSFDM_URL_COL_NAME, TRUE );
		gtk_clist_set_selection_mode( list, GTK_SELECTION_EXTENDED );
		gtk_clist_set_row_height( GTK_CLIST( list ), MINI_ICON_HEIGHT );
		gtk_clist_set_column_width( GTK_CLIST( list ),GSFDM_URL_COL_STATE, MINI_ICON_HEIGHT );
		
		gtk_signal_connect( GTK_OBJECT( list ), "button_press_event", GTK_SIGNAL_FUNC( url_click_cell_callback ), NULL );
		gtk_signal_connect( GTK_OBJECT( list ), "click-column", GTK_SIGNAL_FUNC( url_click_column_callback ), NULL );
		gtk_signal_connect( GTK_OBJECT( list ), "select-row", GTK_SIGNAL_FUNC ( url_select_callback ), NULL );
		gtk_signal_connect( GTK_OBJECT( list ), "unselect-row", GTK_SIGNAL_FUNC ( url_unselect_callback ), NULL );

		gtk_box_pack_start( w, GTK_WIDGET( sc ), TRUE, TRUE, 5 );
		gtk_widget_set_usize( GTK_WIDGET( sc ), 150, -1 );

		gtk_box_pack_start( w, GTK_WIDGET( b = GTK_BOX( gtk_hbox_new( FALSE, 0 ) ) ), FALSE, FALSE, 0 );
		gtk_box_pack_end( b, GTK_WIDGET( dock = GTK_BUTTON( gnome_pixmap_button(
			gnome_stock_pixmap_widget( NULL, GNOME_STOCK_PIXMAP_CONVERT ), _( "Un/Dock" ) ) ) ), FALSE, FALSE, 5 ); 
		gtk_signal_connect( GTK_OBJECT( dock ), "clicked", GTK_SIGNAL_FUNC( dock_callback ), NULL );

		// move url
		gtk_box_pack_start( b, ( app_up = gnome_stock_button( GNOME_STOCK_PIXMAP_UP ) ), FALSE, FALSE, 5 );
		gtk_widget_set_sensitive( app_up, FALSE );
		gtk_signal_connect( GTK_OBJECT( app_up ), "clicked", GTK_SIGNAL_FUNC( url_click_up ), NULL );
		gtk_box_pack_start( b, ( app_down = gnome_stock_button( GNOME_STOCK_PIXMAP_DOWN ) ), FALSE, FALSE, 5 );
		gtk_widget_set_sensitive( app_down, FALSE );
		gtk_signal_connect( GTK_OBJECT( app_down ), "clicked", GTK_SIGNAL_FUNC( url_click_down ), NULL );

		// add a context menu
		gnome_popup_menu_attach( gnome_popup_menu_new( context_data ), GTK_WIDGET( list ), NULL );

		// add copy / paste support
  		gtk_selection_add_target( GTK_WIDGET( list ), GDK_SELECTION_PRIMARY, GDK_SELECTION_TYPE_STRING, 1 );
		gtk_signal_connect( GTK_OBJECT( list ), "selection_clear_event", GTK_SIGNAL_FUNC( gsfdm_url_selection_clear ), NULL );
		gtk_signal_connect( GTK_OBJECT( list ), "selection_get", GTK_SIGNAL_FUNC( gsfdm_url_selection_handle ), NULL );
		gtk_signal_connect( GTK_OBJECT( list ), "selection_received", GTK_SIGNAL_FUNC( gsfdm_url_selection_received ), NULL );
			
		// add dnd support
    	gtk_drag_dest_set( GTK_WIDGET( list ), GTK_DEST_DEFAULT_ALL, default_target, DEFAULT_TARGET_SIZE, GDK_ACTION_COPY );
    	gtk_signal_connect( GTK_OBJECT( list ), "drag_data_received", GTK_SIGNAL_FUNC( gsfdm_url_drop_received ), NULL );

		gtk_drag_source_set( GTK_WIDGET( list ), GDK_BUTTON1_MASK, default_target, DEFAULT_TARGET_SIZE, GDK_ACTION_COPY );
    	gtk_signal_connect( GTK_OBJECT( list ), "drag_data_get", GTK_SIGNAL_FUNC( gsfdm_url_drag_send ), NULL );


//		gtk_signal_connect( GTK_OBJECT( list ), "row_move", GTK_SIGNAL_FUNC( gsfdm_url_row_move ), NULL );


		// create a set of variables from pixmap, faster than load each time
		gdk_imlib_data_to_pixmap( wait_low_xpm, &wait_low_pix, &wait_low_bit );
		gdk_imlib_data_to_pixmap( running_low_xpm, &running_low_pix, &running_low_bit );
		gdk_imlib_data_to_pixmap( done_low_xpm, &done_low_pix, &done_low_bit );
		gdk_imlib_data_to_pixmap( partial_low_xpm, &partial_low_pix, &partial_low_bit );
		gdk_imlib_data_to_pixmap( dead_low_xpm, &dead_low_pix, &dead_low_bit );
		gdk_imlib_data_to_pixmap( unable_low_xpm, &unable_low_pix, &unable_low_bit );

//		timer_id = gtk_timeout_add( gsfdm_config_get_update_time() * 1000, (GtkFunction)gsfdm_url_main_loop, NULL );

		job_stack = g_array_new( TRUE, FALSE, sizeof( guint ) );

		gtk_widget_show_all( GTK_WIDGET( w ) );
	}
	
	return( GTK_WIDGET( w ) );
}

// *****************************************************************************
// just send the entry content to gsfdm_url
static void gsfdm_add_url_ok_callback( GnomeDialog *dlg )
{
gchar	*url = gtk_entry_get_text( GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( dlg ), "entry_widget" ) ) );
	
	// add only if something, could let the url protocol parser show the error ( what it were bound to do )
	// but this way prevent user from a borring warning dlg
	if( strlen( url ) )	
		gsfdm_url_push_url( url,
			gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( gtk_object_get_data( GTK_OBJECT( dlg ), "dl_widget" ) ) ),
			gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( gtk_object_get_data( GTK_OBJECT( dlg ), "rn_widget" ) ) ) );
	
	gtk_widget_destroy( GTK_WIDGET( dlg ) );
}

// *****************************************************************************
//	add an url by direct string
// *****************************************************************************
GnomeDialog	*gsfdm_url_open_url( void )
{
GnomeDialog	*url_dlg;
GtkBox		*bh, *bv;
GtkWidget	*e, *cd, *cr;
	
	url_dlg = GNOME_DIALOG( gnome_dialog_new( _( "Adding URL" ), GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL ) );
	gnome_dialog_close_hides( url_dlg, TRUE );
	gnome_dialog_set_parent( url_dlg, GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( w ) ) ) );
		
	gnome_dialog_button_connect_object( url_dlg, 0, GTK_SIGNAL_FUNC( gsfdm_add_url_ok_callback ), GTK_OBJECT( url_dlg ) );
	gnome_dialog_button_connect_object( url_dlg, 1, GTK_SIGNAL_FUNC( gtk_widget_destroy ), GTK_OBJECT( url_dlg ) );
	gtk_box_set_spacing( GTK_BOX( url_dlg->vbox ), 5 );
	gnome_dialog_set_default( url_dlg, 0 );

	gtk_box_pack_start( GTK_BOX( url_dlg->vbox ), GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 0 ) ) ) , FALSE, FALSE, 5 );
		
	gtk_box_pack_start( GTK_BOX( bh ), gnome_pixmap_new_from_xpm_d( download_xpm ), FALSE, FALSE, 5 );
	gtk_box_pack_start( GTK_BOX( bh ), GTK_WIDGET( bv = GTK_BOX( gtk_vbox_new( FALSE, 0 ) ) ) , FALSE, FALSE, 5 );
		
	gtk_box_pack_start( GTK_BOX( bv ), ( e = gtk_entry_new() ), FALSE, FALSE, 5 );
	gtk_box_pack_start( GTK_BOX( bv ), ( cd = gtk_check_button_new_with_label( _( "Auto Download" ) ) ), FALSE, FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( bv ), ( cr = gtk_check_button_new_with_label( _( "Auto Rename" ) ) ), FALSE, FALSE, 0 );
		
	// coold use static global variables but this way is better
	gtk_object_set_data( GTK_OBJECT( url_dlg ), "entry_widget", (gpointer)( e ) );
	gtk_object_set_data( GTK_OBJECT( url_dlg ), "dl_widget", (gpointer)( cd ) );
	gtk_object_set_data( GTK_OBJECT( url_dlg ), "rn_widget", (gpointer)( cr ) );
	gnome_dialog_editable_enters( url_dlg, GTK_EDITABLE( e ) );
	gtk_widget_grab_focus( e );

	gtk_widget_show_all( GTK_WIDGET( url_dlg ) );
	
	// configure the auto dl check button, could be easier for user
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( cd ), gsfdm_config_get_auto_download() );
	
	// configure the rename
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( cr ), gsfdm_config_get_auto_rename() );

	gtk_widget_show( GTK_WIDGET( url_dlg ) );
	return( url_dlg );
}

/************************************************************************/
//	open the dlg and fix to this url ; used by paste and dnd
/************************************************************************/
void gsfdm_url_popup_url( gchar const *url )
{
GnomeDialog	*dlg = gsfdm_url_open_url();
	
	gtk_entry_set_text( GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( dlg ), "entry_widget" ) ), url );
}

/************************************************************************/
// call for a dock, undock op
/************************************************************************/
static void dock_callback( GtkButton *bp, gpointer d )
{	gsfdm_main_dock_op( MAIN_DOCK_URL_LIST );
}

/************************************************************************/
static void fix_up_button( void )
{	// button active if found an item not in a block starting at row 0...
GList	*l = list->selection;
gboolean	state = TRUE;			// suppose could be activated
gboolean	block = FALSE;
guint		last = 0;
	
	if( !l )
	{	gtk_widget_set_sensitive( GTK_WIDGET( app_up ), FALSE );
		return;
	}
	
	do
	{	if( (guint)l->data == 0 )
		{	block = TRUE;			// found start...
			state = FALSE;
			continue;
		}
		
		if( block )
		{	last++;					// suppose to be
			if( (guint)( l->data ) != last )
			{	// this row is selected and not consecutive with the "start block"
				// can use the up button
				state = TRUE;
				break;
			}
		}
		
	} while( l = g_list_next( l ), l );
	
	gtk_widget_set_sensitive( GTK_WIDGET( app_up ), state );
}

/************************************************************************/
static void fix_down_button( void )
{	// same that up but from the end
GList		*l = list->selection;
gboolean	state = TRUE;			// suppose could be activated
gboolean	block = FALSE;
guint		last = ( list->rows - 1 );
	
	if( !l )
	{	gtk_widget_set_sensitive( GTK_WIDGET( app_down ), FALSE );
		return;
	}
		
	l = g_list_last( l );
	do
	{	if( (guint)l->data == ( list->rows - 1 ) )
		{	block = TRUE;			// fuound start...
			state = FALSE;
			continue;
		}
		
		if( block )
		{	last--;					// suppose to be
			
			if( (guint)( l->data ) != last )
			{	// this row is selected and not consecutive with the "start block"
				// can use the up button
				state = TRUE;
				break;
			}
		}
		
	} while( l = g_list_previous( l ), l );
	
	gtk_widget_set_sensitive( GTK_WIDGET( app_down ), state );
}

/************************************************************************/
// move selection one step up ( arrow direction, not list )
/************************************************************************/
static void url_click_up( GtkWidget *bp )
{
GList	*l, *tmp, *tmp2;
gboolean	flag;
	
	tmp = l = g_list_copy( list->selection );
	
	gtk_clist_freeze( list );
	while( l )
	{	// can move one step up only if not the fist ( 0 ), and previours not selected (
		// in this case that meens previous could have been moved );
		if( (guint)( l->data ) != 0 )
		{	tmp2 = list->selection;
			flag = TRUE;
			
			while( tmp2 )
			{	if( (guint)( tmp2->data ) == ( (guint)( l->data ) - 1 ) )
				{	flag = FALSE;
					break;
				}
				tmp2 = g_list_next( tmp2 );
			}
			
			if( flag )
				gtk_clist_swap_rows( list, (guint)( l->data ), ( (guint)( l->data ) - 1 ) );
		}
		l = g_list_next( l );
	}
	g_list_free( tmp );

	// fix state of move buttons
	fix_up_button();
	fix_down_button();
	
	// focus on the first selected row
	// if < 0, gtk_adjust will take care of that
	if( list->vadjustment->upper > list->vadjustment->value )
	{	gtk_adjustment_set_value( list->vadjustment, 
		(float)( ( (gfloat)( (guint)list->selection->data + 1 ) / 
			(gfloat)list->rows )*list->vadjustment->upper - list->vadjustment->page_size ) );
		gtk_adjustment_value_changed( list->vadjustment );
	}
	
	gtk_clist_thaw( list );
}

/************************************************************************/
/************************************************************************/
static void url_click_down( GtkWidget *bp )
{
GList	*l, *tmp, *tmp2;
gboolean	flag;
	
	tmp = l = g_list_copy( list->selection );
	l = g_list_last( l );
	
	gtk_clist_freeze( list );
	while( l )
	{	// can move one step up only if not the last , and next not selected (
		// in this case that meens previous could have been moved );
		if( (guint)( l->data ) != ( list->rows - 1 ) )
		{	tmp2 = list->selection;
			flag = TRUE;
			
			while( tmp2 )
			{	if( (guint)( tmp2->data ) == ( (guint)( l->data ) + 1 ) )
				{	flag = FALSE;
					break;
				}
				tmp2 = g_list_next( tmp2 );
			}
			
			if( flag )
				gtk_clist_swap_rows( list, (guint)( l->data ), ( (guint)( l->data ) + 1 ) );
		}
		l = g_list_previous( l );
	}	
	g_list_free( tmp );

	// fix state of move buttons
	fix_up_button();
	fix_down_button();
	
	// focus on the last selected row
	if( list->vadjustment->upper > list->vadjustment->value )
	{	gtk_adjustment_set_value( list->vadjustment, (float)( ( 
			(gfloat)( (guint)g_list_last( list->selection )->data ) / 
			(gfloat)list->rows )*list->vadjustment->upper ) );
		gtk_adjustment_value_changed( list->vadjustment );
	}
	
	gtk_clist_thaw( list );
}

/************************************************************************/
//	callback when a row is selected
/************************************************************************/
static void url_select_callback( GtkCList *cl, gint r, gint c, GdkEventButton *b, gpointer d )
{
st_gsfdm_url_entry	*s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, r ) );
	
	// fix state of move buttons
	fix_up_button();
	fix_down_button();

	gsfdm_info_display_info( gsfdm_url_st_get_id( s ) );
}

/************************************************************************/
//	callback when a row of the list is unselected
/************************************************************************/
static void url_unselect_callback( GtkCList *cl, gint r, gint c, GdkEventButton *b, gpointer d )
{
st_gsfdm_url_entry	*s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, r ) );
	
	// fix state of move buttons
	fix_up_button();
	fix_down_button();
	
	gsfdm_info_undisplay_info( gsfdm_url_st_get_id( s ) );
}

/************************************************************************/
//	intercept all mouse button event first : needed for popup menu
/************************************************************************/
static gint url_click_cell_callback( GtkWidget *cl, GdkEventButton *e, gpointer d )
{
gint	r, c;
	
	// intersept only left button
	if( e->button != 1 )
		return( TRUE );
	
	// test if events on a valid cell
	if( !gtk_clist_get_selection_info( GTK_CLIST( cl ), (gint)e->x, (gint)e->y, &r, &c ) )
		return( TRUE );									// NO!!!!
	
	if( e->type != GDK_2BUTTON_PRESS )
		return( TRUE );
	
	// double left click, force downloading of THIS job
	gsfdm_url_start_id( gsfdm_url_st_get_id( gtk_clist_get_row_data( GTK_CLIST( cl ), r ) ) );
	
	// don't change the focus ; it appear than a FALSE or TRUE return value don't run...
	gtk_signal_emit_stop_by_name( GTK_OBJECT( cl ), "button_press_event" );
	return( TRUE );

}

/************************************************************************/
//	when click on the tittle of one column
/************************************************************************/
static void url_click_column_callback( GtkCList *cl, gint c, gpointer p )
{

	
	
	
	
}

/************************************************************************/
// create the vector with id
static void gsfdm_url_create_vector(  guint *argc, gchar **argv[] )
{
guint	row;
gchar	**tab;
	
	tab = g_new0( gchar *, ( list->rows + 1 ) );
	for( row = 0 ; row < list->rows ; row++ )
	{	tab[ row ] = g_strdup_printf( "%d", 
			gsfdm_url_st_get_id( ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row  ) ) ) );
	}
	
	*argv = tab;
	*argc = list->rows;
	
	#ifdef DEBUG_URL_VECTOR
	g_print( "debug url : create vector of %d size\n", list->rows  );
	#endif
}

/************************************************************************/
static void	gsfdm_url_destroy_vector( gchar *argv[] )
{
gchar	**a = argv;
guint	index = 0;
	
	while( a[ index ] )
	{	g_free( a[ index++ ] ); }
	
	g_free( argv );
}

/************************************************************************/
//	load the config and start all "stuff"
/************************************************************************/
void	gsfdm_url_load_config( void )
{
static gchar		*text[] = { NULL, NULL, NULL, NULL };
st_gsfdm_url_entry	*s = NULL;
gint				size, index = 0;
gchar				**argv, *section;
guint				row;
glong				file_size;

	#ifdef DEBUG_URL_LOAD_CONFIG
	g_print( "debug url : loading configurztion\n" );
	#endif

	// create th e timer for auto download loop -- and update loop	
	timer_id = gtk_timeout_add( gsfdm_config_get_update_time() * 1000, (GtkFunction)gsfdm_url_main_loop, NULL );

	// fill url list with previously saved
	
	// get the vector
	gnome_config_push_prefix( "/"PACKAGE"/url_list/" );
	gnome_config_get_vector( "list", &size, &argv );
	gnome_config_pop_prefix();

	if( !argv )
	{
		#ifdef DEBUG_URL_LOAD_CONFIG
		g_print( "---- debug url : load successful with nothing...\n" );
		#endif
		return;
	}
	
	// there something to add
	gtk_clist_freeze( list );
	while( index < size )
	{	s = gsfdm_url_st_new();

		section = g_strdup_printf( "/"PACKAGE"/url_%s/", argv[ index ] );

		// in case of error in config
		if( !gnome_config_has_section( section ) )
		{	g_free( section );
			index++;
			continue;
		}
		
		// section found, parse
		gnome_config_push_prefix( section );
		
		// create struct from saved info
		// test for plugin is done at every start of downloading
		gsfdm_url_st_get_url( s ) = gnome_config_get_string( "url" );		
		gsfdm_url_st_get_id( s ) = gnome_config_get_int( "id" );
		gsfdm_url_st_get_name( s ) = gnome_config_get_string( "name" );
		gsfdm_url_st_get_path( s ) = gnome_config_get_string( "path" );
		gsfdm_url_st_get_pid( s ) = 0;
		gsfdm_url_st_get_auto_dl( s ) = gnome_config_get_bool( "auto" );
		gsfdm_url_st_get_auto_rn( s ) = gnome_config_get_bool( "auto_rn" );
		gsfdm_url_st_get_time( s ) = gnome_config_get_int_with_default( "time=0", NULL );
		gsfdm_url_st_get_state( s ) = gnome_config_get_int( "state" );
		gsfdm_url_st_get_total( s ) = 0;		// just in case. Don't save it. Prevent change
		
		switch( gsfdm_url_st_get_state( s ) )
		{	case GSFDM_URL_STATE_RUNNING :
				gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_PARTIAL;
				// must check whether it's still ok
			case GSFDM_URL_STATE_PARTIAL :
			{	gchar	*tmp = g_strconcat( gsfdm_url_st_get_path( s ), "/", gsfdm_url_st_get_name( s ), NULL );
				
				switch( gsfdm_file_get_file_type( tmp ) )
				{	case GSFDM_FILE_TYPE_ERROR :		// not in disk 
					{	gchar *tmp2;
					
						tmp2 = g_strconcat( _( "Unable to found partial file : " ), tmp, _( " ... switch to WAIT state" ), NULL );
						gsfdm_message_push_message( tmp2, FALSE, FALSE, TRUE );
						g_free( tmp2 );
						
						gsfdm_url_st_get_percent( s ) = 0.0;
						gsfdm_url_st_get_size( s ) = 0;
						gsfdm_url_st_get_auto_dl( s ) = FALSE;
						gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_WAIT;
						break;
					}
					
					case GSFDM_FILE_TYPE_REGULAR :
						// get size from disk	
						gsfdm_url_st_get_percent( s ) = gnome_config_get_float_with_default( "percent=0.0", NULL ); // must be right ( ?? )
						gsfdm_url_st_get_size( s ) = ( gsfdm_file_get_file_stat_fast( tmp, FALSE )->st_size >> 10 );
						break;
					
					default :							// big error
					{	gchar *tmp2;
					
						tmp2 = g_strconcat( _( "WARNING" ), tmp, _( " exist but is NOT a regular file" ), NULL );
						gsfdm_message_push_message( tmp2, FALSE, FALSE, TRUE );
						g_free( tmp2 );
						
						gsfdm_url_st_get_percent( s ) = 0;
						gsfdm_url_st_get_auto_dl( s ) = FALSE;
						gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_DEAD;
					}
				}
				
				g_free( tmp );
				break;
			}
					
			case GSFDM_URL_STATE_DONE : 
				gsfdm_url_st_get_percent( s ) = 1.0;
				break;
			
			default :
				gsfdm_url_st_get_percent( s ) = gnome_config_get_float_with_default( "percent=0.0", NULL );
		}
			
		gsfdm_url_st_get_plug_id( s ) = gsfdm_plugin_get_id( gsfdm_url_st_get_url( s ), FALSE );
		
		// set percent and size...
		text[ GSFDM_URL_COL_PERCENT ] = g_strdup_printf( URL_PERCENT_FORMAT, gsfdm_url_st_get_percent( s ) * 100.0 );		
		text[ GSFDM_URL_COL_NAME ] = gsfdm_url_st_get_name( s );
		row = gtk_clist_append( list, text );    
		gtk_clist_set_row_data( list, row, (gpointer)s );
		g_free( text[ GSFDM_URL_COL_PERCENT ] );
		
		// change icon and fix size and %
			switch( gsfdm_url_st_get_state( s ) )
			{	case GSFDM_URL_STATE_RUNNING :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, running_low_pix, running_low_bit );
					break;
				case GSFDM_URL_STATE_DONE :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, done_low_pix, done_low_bit );
					break;
				case GSFDM_URL_STATE_DEAD :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, dead_low_pix, dead_low_bit );
					file_size = 0;
					break;
				case GSFDM_URL_STATE_PARTIAL :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, partial_low_pix, partial_low_bit );
					break;
				default :
					gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_WAIT;
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, wait_low_pix, wait_low_bit );
					file_size = 0;
			}
	
		g_free( section );
		gnome_config_pop_prefix();
		index++;
	}
	
	gsfdm_url_destroy_vector( argv );
	gtk_clist_thaw( list );
	
	// if autodl on start a loop ????
	
	#ifdef DEBUG_URL_LOAD_CONFIG
	g_print( "---- debug url : load successful\n" );
	#endif
}

/************************************************************************/
// there are internal functions that save the url list at regular time
// in case of segfault
static void gsfdm_url_update_to_config( guint id )
{
gchar	**argv;
guint	size;
gchar	*section;
st_gsfdm_url_entry const *s;
	
	#ifdef DEBUG_URL_UPDATE_CONFIG
	g_print( "---- debug url : update config for %d...\n",id  );
	#endif
	
	#ifdef	DEBUG_NO_UPDATE_CONFIG
	g_print( "---- debug url : skip update config\n"  );
	return;
	#endif
	
	// create the vector
	gnome_config_push_prefix( "/"PACKAGE"/url_list/" );
	gsfdm_url_create_vector( &size, &argv );
	gnome_config_set_vector( "list", size, argv );
	gnome_config_pop_prefix();
	gsfdm_url_destroy_vector( argv );
	
	// create new section ( but first, try a remove, just in case of garbage )
	section = g_strdup_printf( "/"PACKAGE"/url_%d/", id );
	gnome_config_clean_section( section );
	
	gnome_config_push_prefix( section );
	s = gsfdm_url_get_url_data( id );
	gnome_config_set_int( "id", gsfdm_url_st_get_id( s ) );
	gnome_config_set_string( "url", gsfdm_url_st_get_url( s ) );
	gnome_config_set_string( "name", gsfdm_url_st_get_name( s ) );
	gnome_config_set_string( "path", gsfdm_url_st_get_path( s ) );
	gnome_config_set_int( "pid", gsfdm_url_st_get_pid( s ) );
	gnome_config_set_bool( "auto", gsfdm_url_st_get_auto_dl( s ) );
	gnome_config_set_bool( "auto_rn", gsfdm_url_st_get_auto_rn( s ) );
	gnome_config_set_int( "state", gsfdm_url_st_get_state( s ) );
	gnome_config_set_int( "time", gsfdm_url_st_get_time( s ) );
	
	gnome_config_pop_prefix();
	gnome_config_sync();
	g_free( section );
}

/************************************************************************/
static void gsfdm_url_remove_from_config( guint id )
{
gchar	**argv;
gint	size;
gchar	*section;

	#ifdef DEBUG_URL_UPDATE_CONFIG
	g_print( "---- debug url : RM config for %d...\n",id  );
	#endif

	#ifdef	DEBUG_NO_UPDATE_CONFIG
	g_print( "---- debug url : skip update config\n"  );
	return;
	#endif
	
	// create the vector
	gnome_config_push_prefix( "/"PACKAGE"/url_list/" );
	gsfdm_url_create_vector( &size, &argv );
	gnome_config_set_vector( "list", size, argv );
	gnome_config_pop_prefix();
	gsfdm_url_destroy_vector( argv );
	
	// create new section ( but first, try a remove, just in case of garbage )
	section = g_strdup_printf( "/"PACKAGE"/url_%d/", id );
	gnome_config_clean_section( section );
	gnome_config_sync();
	g_free( section );	
}

/************************************************************************/
// called when changing state or updating setting ( name, path ... )
static void gsfdm_url_refresh_in_config( guint id )
{
gchar	*section;
st_gsfdm_url_entry const *s = gsfdm_url_get_url_data( id );

	#ifdef DEBUG_URL_REFRESH_CONFIG
	g_print( "---- debug url : refresh state of %d...\n",id  );
	#endif
	
	#ifdef	DEBUG_NO_UPDATE_CONFIG
	g_print( "---- debug url : skip update config\n"  );
	return;
	#endif
	
	// create new section ( but first, try a remove, just in case of garbage )
	section = g_strdup_printf( "/"PACKAGE"/url_%d/", id );
	if( gnome_config_has_section( section ) )
	{	gnome_config_push_prefix( section );
		gnome_config_set_int( "pid", gsfdm_url_st_get_pid( s ) );
		gnome_config_set_float( "percent", gsfdm_url_st_get_percent( s ) );
		gnome_config_set_int( "state", gsfdm_url_st_get_state( s ) );
		gnome_config_set_int( "time", gsfdm_url_st_get_time( s ) );
	//	gnome_config_set_string( "name", gsfdm_url_st_get_name( s ) );
	//	gnome_config_set_string( "url", gsfdm_url_st_get_url( s ) );
	//	gnome_config_set_string( "path", gsfdm_url_st_get_path( s ) );
	//	gnome_config_set_bool( "auto", gsfdm_url_st_get_auto_dl( s ) );
	//	gnome_config_set_bool( "auto_rn", gsfdm_url_st_get_auto_rn( s ) );
		gnome_config_pop_prefix();
		gnome_config_sync();
	}
	
	g_free( section );	
}

/************************************************************************/
//	save the state and data of all url
/************************************************************************/
void	gsfdm_url_save_config( void )
{
	// do nothing ( saved every time ), just for compatibility in my way of
	// creating soft : a "block" --> a file --> a default set of functions
}


/************************************************************************/
// true = can start... there're pppd 
/************************************************************************/
static gboolean gsfdm_check_pppd( void )
{
static gchar *ppp = NULL;

	// watch if ppp is running...
	if( !ppp )
		ppp = g_strconcat( "/var/run/", gsfdm_config_get_pppd(), ".pid", NULL );

	#ifndef DEBUG_URL_COMMUNICATION
	// if no pppd, delay download...
	if( access( ppp, X_OK ) )
		return( FALSE );
	#endif

	return( TRUE );							// always start, usefull when use "pingpong" ( watch plug sources )
}

/************************************************************************/
//	the main loop, start job, get data from child...
/************************************************************************/
static gboolean gsfdm_url_main_loop( gpointer data )
{
time_t			tm = time( NULL );	// just for save some cpu time (!)
st_gsfdm_url_entry const *s;
GsfdmUrlState	new_state;
guint			index = 0;
	
//	#ifdef DEBUG_URL_MAIN_LOOP
//	g_print( "debug url : entree main loop ; pile est %d \n", job_stack->len );
//	#endif

	// watch if ppp is running...
	if( !gsfdm_check_pppd() )
		return( TRUE );

//g_print( "UPDATE STACK %d\n", job_stack->len );

	// update data for all running process
	while( index < job_stack->len )
	{	
//g_print( "UPDATE pour %d\n", g_array_index( job_stack, guint, index ) );
		
		// maybe a "shadow" job	
		if( !( s = gsfdm_url_get_url_data( g_array_index( job_stack, guint, index ) ) ) )
		{	g_array_remove_index_fast( job_stack, index );
			continue;
		}

		// get the change or an error
		if( ( ( new_state = (GsfdmUrlState)gsfdm_plugin_call( gsfdm_url_st_get_plug_id( s ), 
			PLUG_UPDATE_JOB, ST_GSFDM_URL_ENTRY_SHARED( s ), NULL ) ) & GSFDM_URL_STATE_MASK ) == GSFDM_URL_STATE_ERROR )
			new_state =  GSFDM_URL_STATE_DEAD;
			
		// if change occur, update "last_update_time" field
		if( new_state && GSFDM_URL_CHANGE_MASK )
			gsfdm_url_st_get_last_update_time( s ) = tm;
		
		// check for timeout ; if return 0, there no time out
		if( gsfdm_config_get_timeout_on_download() )
		{	if( difftime( tm, gsfdm_url_st_get_last_update_time( s ) ) >= ( gsfdm_config_get_timeout_on_download() ) )
			{//	new_state = GSFDM_URL_STATE_PARTIAL;
		
				#ifdef DEBUG_URL_MAIN_LOOP
				g_print( "debug url : timeout for %s\n", gsfdm_url_st_get_name( s ) );
				#endif
			}
		}

		gsfdm_url_st_get_time( s ) += ( tm - gsfdm_url_st_get_last_time( s ) );
		gsfdm_url_st_get_last_time( s ) = tm;

		// if no change, skip
		if( !( new_state & GSFDM_URL_CHANGE_MASK ) )
		{	index++;	
			continue;
		}
		
		gsfdm_url_st_get_state( s ) = ( new_state & GSFDM_URL_STATE_MASK );
		
		// now, what should i do with these info ???
		switch( gsfdm_url_st_get_state( s ) )
		{	case GSFDM_URL_STATE_RUNNING :
				gsfdm_url_percent_changed( gsfdm_url_st_get_id( s ) );
				index++;
				break; 
				
			// if partial, like done, dead but also infor user
			case GSFDM_URL_STATE_PARTIAL :
			{	gchar		*mess = g_strconcat( _( "Premature end of file :\n" ), gsfdm_url_st_get_name( s ), NULL );
				GtkWidget	*d = gnome_warning_dialog_parented( mess, GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( w ) ) ) );
				
				gtk_window_set_position( GTK_WINDOW( d ), GTK_WIN_POS_MOUSE );
				gtk_widget_show( d );				
				g_free( mess );

				gsfdm_url_st_get_pid( s ) = 0;
				gsfdm_url_release_edit( s );
				gsfdm_url_id_state_changed( gsfdm_url_st_get_id( s ) );

				g_array_remove_index_fast( job_stack, index );		// remove from stack
			
				gsfdm_message_push_code( MESS_PARTIAL_LINK, (gpointer)s );
				break;
			}
			
			case GSFDM_URL_STATE_DONE :
				gsfdm_url_st_get_pid( s ) = 0;
				gsfdm_url_release_edit( s );
				gsfdm_url_id_state_changed( gsfdm_url_st_get_id( s ) );

				g_array_remove_index_fast( job_stack, index );		// remove from stack
			
				gsfdm_message_push_code( MESS_FINISH_DOWNLOAD, (gpointer)s );
			
				// auto parse - add new references - ???
				if( gsfdm_config_get_auto_parse() )
				{	// now parse data, want 'network' data
					// open stream on file, create a memory stream for anwser...
					gchar		*file = g_strconcat( gsfdm_url_st_get_path( s ), "/", gsfdm_url_st_get_name( s ), NULL );
					FILE		*input = fopen( file, "r" );
					FILE		*output;
					gchar		*ext;
					guint		plug_id = gsfdm_plugin_get_id( file, TRUE );
					
					// need plug of downloaded file....
					if( !plug_id )
					{	g_free( file );
						break;
					}
					
					// some test, just in case...
					if( !( input = fopen( file, "r" ) ) )
					{	g_free( file );
						break;
					}
					
					output = fmemopen( NULL, URL_BUFFER_TO_PLUG, "w+" );
					ext = gsfdm_file_extract_ext( file );
					
					if( gsfdm_plugin_call( plug_id, PLUG_PARSE_FILE, input, output, ST_GSFDM_URL_ENTRY_SHARED( s ), ext ) )
					{	// ok, add every 'line' as url 	
						gchar	*str = g_new0( gchar, URL_BUFFER_LINE_LENGTH );		// isn't it enough ???
						size_t	size = URL_BUFFER_LINE_LENGTH;
						
						rewind( output );
						// add evey line, call gsfdm_url_push_url, in case of 'local' ref..
						while( getline( &str, &size,output  ) != -1 )
						{	// last line is emply.. sometime
							if( !strlen( str ) )
								break;
							
							gsfdm_url_push_url( str, gsfdm_config_get_auto_download(), gsfdm_config_get_auto_rename() );
						}
						
						g_free( str );
					}
					
					g_free( file );
					g_free( ext );
					fclose( input );
					fclose( output );
				}
		
				// auto clean the list ???
				if( gsfdm_config_auto_remove_when_done() )
					gsfdm_url_remove_url( gsfdm_url_st_get_id( s ) );
				break;
			
			case GSFDM_URL_STATE_DEAD :
				gsfdm_message_push_code( MESS_DEAD_LINK, (gpointer)s );
				gsfdm_url_st_get_pid( s ) = 0;
				gsfdm_url_release_edit( s );
				gsfdm_url_id_state_changed( gsfdm_url_st_get_id( s ) );
				g_array_remove_index_fast( job_stack, index );		// remove from stack
				break;
			
			case GSFDM_URL_STATE_UNABLE :			
			default :
				gsfdm_message_push_code( MESS_ERROR, (gpointer)s );
				gsfdm_url_st_get_pid( s ) = 0;
				gsfdm_url_release_edit( s );
				gsfdm_url_id_state_changed( gsfdm_url_st_get_id( s ) );

				g_array_remove_index_fast( job_stack, index );		// remove from stack
		}
	}

	// already at limit up ???
	if( gsfdm_url_nb_are_working() >= gsfdm_config_max_simultaneous() )
		return( TRUE );		

	// look if could start new download
	if( gsfdm_main_get_allow_auto_dl() )
	{	// add new job
		if( ( s = gsfdm_url_get_next_job() ) )
		{
			#ifdef DEBUG_URL_MAIN_LOOP
			g_print( "debug url : main loop auto add new job : %s\n", gsfdm_url_st_get_name( s ) );
			#endif

			// add a new job
			gsfdm_url_start_id( gsfdm_url_st_get_id( s ) );
			return( TRUE );
		}	
	}

	if( !stack_empty_flag && !job_stack->len )
	{	gfsdm_main_job_stack_empty();			// infor of stack empty, perhaps display a dlg
		stack_empty_flag = TRUE;

		#ifdef DEBUG_URL_MAIN_LOOP
		g_print( "debug url : main loop - stack empty\n" );
		#endif
	}

	return( TRUE );				
}

/************************************************************************/
// # of job(s) running --> 0 if at rest
/************************************************************************/
guint	gsfdm_url_nb_are_working( void )
{
	return( job_stack->len );
}

/************************************************************************/
//	change the timing of update function
/************************************************************************/
void	gsfdm_url_set_update_time( guint sec )
{
	gtk_timeout_remove( timer_id );
	timer_id = gtk_timeout_add( sec * 1000,(GtkFunction)gsfdm_url_main_loop, NULL );
}

/************************************************************************/
// start downloading the url matching this id ; stop
/************************************************************************/
static void gsfdm_url_start_id( guint id )
{
static GnomeDialog			*dlg = NULL;
static GtkLabel				*label;
st_gsfdm_url_entry const	*s = gsfdm_url_get_url_data( id );
gchar						*full_path, *tmp;
	
	if( !s ) return;								// id unknow
	
	#ifdef DEBUG_URL_START
	g_print( "debug url : start id %d - %s\n", id, gsfdm_url_st_get_name( s ) );
	#endif
	
	// going to make change on state.... also check state == RUNNING
	if( !gsfdm_url_can_edit( s ) )
	{	
		#ifdef DEBUG_URL_START
		g_print( "debug url : can't start id %d - %s\n", id, gsfdm_url_st_get_name( s ) );
		#endif
		
		return;
	}
	// must check if plug_id exist ( yet )
	if( !gsfdm_plugin_check_for_plugin( gsfdm_url_st_get_plug_id( s ) ) )
	{	gtk_widget_show( gnome_warning_dialog_parented( 
			_( "Plug in charge of this file is unknow or disabled\nDownload delayed" ),
			GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( w ) ) ) ) );
		return;
	}

	// if no pppd, dealy download and infor user wxith a simple message
	if( !gsfdm_check_pppd() )
	{	gsfdm_message_push_code( MESS_NO_PPPD, NULL );
		return;
	}
		
	// first, look for target dir.. and rigth	
	if( access( gsfdm_url_st_get_path( s ), W_OK ) )
	{	// don't exist or write error, can create it ???
		if( mkdir( gsfdm_url_st_get_path( s ), S_IRWXU ) )
		{	// no, so if exist can't access it anyway
			g_free( gsfdm_url_st_get_path( s ) );
			gsfdm_url_st_get_path( s ) = g_strdup( g_get_home_dir() );	// no
			gsfdm_message_push_code( MESS_PATH_CHANGED, (gpointer)s  );

			// update config
			gsfdm_url_update_to_config( gsfdm_url_st_get_id( s ) );
		}
	}
	
	// look whether file exist, in case prompt or rename
	if( gsfdm_url_st_get_state( s ) != GSFDM_URL_STATE_PARTIAL )
	{	full_path = g_strconcat( gsfdm_url_st_get_path( s ), "/", gsfdm_url_st_get_name( s ), NULL );
		
		switch( gsfdm_file_get_file_type( full_path ) )
		{	case GSFDM_FILE_TYPE_REGULAR :		// file exist....
			{	// direct rename
				if( gsfdm_url_st_get_auto_rn( s ) )
				{	gchar *new_name;
							
					new_name = gsfdm_file_get_new_name( gsfdm_url_st_get_name( s ), gsfdm_url_st_get_path( s ), TRUE );			
					tmp = g_strdup_printf( "File : %s  was automaticaly renamed as : %s\n", 
						gsfdm_url_st_get_name( s ), new_name );
					gsfdm_message_push_message( tmp, FALSE, FALSE, FALSE );
					g_free( tmp );
		
					g_free( gsfdm_url_st_get_name( s ) );
					gsfdm_url_st_get_name( s ) = new_name;	
					
					// update config
					gsfdm_url_update_to_config( gsfdm_url_st_get_id( s ) );
					break;	
				}
				
				// prompt user
				if( !dlg )
				{	GtkBox	*bh;
					
					dlg = GNOME_DIALOG( gnome_dialog_new( _( "File exist" ), 
						GNOME_STOCK_BUTTON_OK, "auto rename", GNOME_STOCK_BUTTON_CANCEL, NULL ) );
					gnome_dialog_close_hides( dlg, TRUE );
					gnome_dialog_set_close( dlg, TRUE );
					gnome_dialog_set_parent( dlg, GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( w ) ) ) );
					gtk_box_set_spacing( GTK_BOX( dlg->vbox ), 5 );
				
					gtk_box_pack_start( GTK_BOX( dlg->vbox ), GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 0 ) ) ) , FALSE, FALSE, 5 );
			
					gtk_box_pack_start( GTK_BOX( bh ), gnome_pixmap_new_from_xpm_d( gsfdm_xpm ), FALSE, FALSE, 5 );
					gtk_box_pack_start( GTK_BOX( bh ), GTK_WIDGET( label = GTK_LABEL( gtk_label_new( "" ) ) ), FALSE, FALSE, 5 );
			
					gtk_widget_show_all( GTK_WIDGET( dlg->vbox ) );			
				}
					
				tmp = g_strdup_printf( _( "%s\n already in list. Replace ??" ), gsfdm_url_st_get_name( s ) );
				gtk_label_set_text( label, tmp );
	
				switch( gnome_dialog_run_and_close( dlg ) )
				{	case 0 :			// ok button : replace
						unlink( full_path );
						break;
					
					case 1 :			// rename
					{	gchar		*new_name;
						
						new_name = gsfdm_file_get_new_name( gsfdm_url_st_get_name( s ), gsfdm_url_st_get_path( s ), TRUE );
						g_free( gsfdm_url_st_get_name( s ) );
						gsfdm_url_st_get_name( s ) = new_name;
						break;
					}
					
					default :
						gsfdm_message_push_message( tmp, FALSE, TRUE, FALSE );
						g_free( tmp );
						gsfdm_url_st_destroy( ST_GSFDM_URL_ENTRY( s ) );
						return;					// aborting
				}

				g_free( tmp );
			}
			
			// could write throught socket and fifo ?????
			case GSFDM_FILE_TYPE_DIR :
			case GSFDM_FILE_TYPE_SOCKET :
			case GSFDM_FILE_TYPE_C_DEVICE :
			case GSFDM_FILE_TYPE_B_DEVICE :
			case GSFDM_FILE_TYPE_FIFO :
			{	GtkWidget	*d;
				
				tmp = g_strconcat( _( "Can't start download of :\n" ),gsfdm_url_st_get_name( s ),  NULL );
				d = gnome_error_dialog( tmp );
				gtk_window_set_position( GTK_WINDOW( d ), GTK_WIN_POS_MOUSE );
				gtk_widget_show( d );
				g_free( tmp );
				gsfdm_url_st_get_auto_dl( s ) = FALSE;
				break;
			}
			
			// all right
			default :
		}
		
		g_free( full_path );
	}
	
	// launch
	if( !( gsfdm_url_st_get_pid( s ) = (guint)gsfdm_plugin_call( 
		gsfdm_url_st_get_plug_id( s ), PLUG_START_JOB, ST_GSFDM_URL_ENTRY_SHARED( s ), gsfdm_config_get_proxy(), NULL ) ) )
		return;									// can't start...
	
	gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_RUNNING;
	gsfdm_url_st_get_last_time( s ) = time( NULL );
	gsfdm_url_st_get_last_update_time( s ) = gsfdm_url_st_get_last_time( s );

	// add to list
	g_array_append_val( job_stack, id );	
	stack_empty_flag = FALSE;
	
	#ifdef DEBUG_URL_START
	g_print( "debug url : %d - %s started ; plug %d, timer %d every %d sec\n", 
		id, gsfdm_url_st_get_name( s ), gsfdm_url_st_get_plug_id( s ), timer_id, gsfdm_config_get_update_time() );
	#endif
	
	// there lot of graphic changes to do
	// update config done further
	gsfdm_url_id_state_changed( id );
	gsfdm_message_push_code( MESS_START_DOWNLOAD, (gpointer)s );
}

/************************************************************************/
static void gsfdm_url_stop_id( guint id )
{
st_gsfdm_url_entry const *s = gsfdm_url_get_url_data( id );
guint	i;
	
	if( !s ) return;								// id unknow
		
	#ifdef DEBUG_URL_STOP
	g_print( "debug url : stop id %d - %s\n", id, gsfdm_url_st_get_name( s ) );
	#endif

	// stop it only if running !!!
	if( gsfdm_url_st_get_state( s ) != GSFDM_URL_STATE_RUNNING )
		return;

	gsfdm_url_st_get_auto_dl( s ) = FALSE;			// stopped so no auto dl
	
	// inform plug to kill THIS download
	gsfdm_plugin_call( gsfdm_url_st_get_plug_id( s ), PLUG_STOP_JOB, ST_GSFDM_URL_ENTRY_SHARED( s ) );
	gsfdm_url_st_get_pid( s ) = 0;
	gsfdm_url_release_edit( s );
	gsfdm_url_st_get_last_time( s ) = 0;
	gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_PARTIAL;

	//##########################################################
	// remove from job stack
	for( i = 0 ; i < job_stack->len ; i++ )
	{		if( g_array_index( job_stack, guint, i ) == id )
			{	g_array_remove_index_fast( job_stack, i );
				break;
			}
	}
	
	gsfdm_message_push_code( MESS_DOWNLOAD_STOPPED, (gpointer)s );

	// there lot of graphic changes to do
	// update config to match new state ( filled by PLUG_STOP_JOB )
	gsfdm_url_id_state_changed( id );
}

/************************************************************************/
// remove this url from the list ; return true on success
// alway remove, even with running process
/************************************************************************/
static gboolean	gsfdm_url_remove_url( gint id )
{	
st_gsfdm_url_entry	const *s;
guint				row;
	
	// there an error
	s = gsfdm_url_get_url_data( id );

	// stop process
	gsfdm_url_stop_id( id );			// in case of running...
	
	// if editing, cancel, else, remove whatever the state is
	if( gsfdm_url_st_get_edit( s ) )
		return( FALSE );

	#ifdef DEBUG_URL_REMOVE
	g_print( "debug url : remove %d - %s\n", id, gsfdm_url_st_get_name( s ) );
	#endif 
	
	// remove from list ; scan list for matchin data
	gtk_clist_freeze( list );
	for( row = 0 ; row < list->rows ; row++ )
	{	
		if( gsfdm_url_st_get_id( gtk_clist_get_row_data( list, row ) ) == id )
		{
			// in case currently displayed...
			gsfdm_info_inform_for_deleted( id );

			// clean saved list
			gsfdm_url_remove_from_config( id );

			gsfdm_url_st_destroy( ST_GSFDM_URL_ENTRY( s ) );
			gtk_clist_remove( list, row );
			break;						// found, removed
		}
	}
 	gtk_clist_thaw( list );

	// don't remove from job_stack ; this was done by stop function
	return( TRUE );
}

/************************************************************************/
// try to remove the url matching url ( not the name ) from the
// list ; if force = TRUE and running force stop else return false
gboolean	gsfdm_url_remove( gchar const *url, gboolean force )
{
st_gsfdm_url_entry	*s;
guint				row;
	
	// fist, found id
	for( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		if( !strcmp( url, gsfdm_url_st_get_url( s ) ) )
		{	// found remove if force and/or not running
			if( force )
				gsfdm_url_release_edit( s );
			
			if( gsfdm_url_can_edit( s ) )
			{	gsfdm_url_remove_url( gsfdm_url_st_get_id( s ) );
				return( TRUE );
			}
				
			break;
		}
	}

	return( FALSE );
}

/************************************************************************/
// stop download for this url, true if found
// switch to auto of if not running
gboolean	gsfdm_url_stop_url( gchar const *url )
{
st_gsfdm_url_entry	*s;
guint				row;
	
	// fist, found id
	for( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		if( !strcmp( url, gsfdm_url_st_get_url( s ) ) )
		{	// found 
			
			// just switch ????
			if ( gsfdm_url_st_get_state( s ) != GSFDM_URL_STATE_RUNNING )
			{	gsfdm_url_st_get_auto_dl( s ) = FALSE;
				gsfdm_info_inform_for_change( gsfdm_url_st_get_id( s ) );
				return( TRUE );
			}
			
			// more, must stop
			gsfdm_url_stop_id( gsfdm_url_st_get_id( s ) );
			return( TRUE );
		}
	}

	return( FALSE );
}

/************************************************************************/
// infor gsfdm_url to kill all process ( stop downloading )
// involve formcing auto_download to false
void	gsfdm_url_stop_downloading( void )
{
	// pass through the running_job list and stop each
	while( job_stack->len )
	{	gsfdm_url_stop_id( g_array_index( job_stack, guint, 0 ) );
		g_array_remove_index_fast( job_stack, 0 );
	}
}

/************************************************************************/
// check whether is local or net
/************************************************************************/
gboolean	gsfdm_url_push_url( gchar const *file, gboolean auto_dl, gboolean auto_rn )
{
gchar		*pt, *tmp = g_strdup( file );
gboolean	state;
	
	#ifdef DEBUG_URL_PUSH_URL
	g_print( "debug url : data to add : %s - %d %d\n", file, auto_dl, auto_rn );
	#endif
	
	// if done when reading content of a file, could have some '\n' remain...
	if( ( pt = strchr( tmp, '\n' ) ) )
		pt[0] = 0;
	
	// now remove all white spaces at the end, and begining...
	pt = tmp + strlen( tmp );
	while( *pt == ' ' )				// count down while white space
	{	pt -= 1;
	}
	pt[1] = 0;
	
	pt = tmp;						// now remove at beginning
	while( *pt == ' ' )
	{	pt += 1;
	}
	
	if( pt != tmp )
	{	gchar	*tmp2 = g_strdup( pt );
		g_free( tmp );
		tmp = tmp2;
	}	
	
	if( gsfdm_file_protocol_is_local( tmp ) )
		state = gsfdm_url_push_url_from_file( tmp, auto_dl, auto_rn );
	else
		state = gsfdm_url_push_url_from_url( tmp, auto_dl, auto_rn );
	
	g_free( tmp );
	return( state );
}

/************************************************************************/
// add a new entry from a disk file ; return true on success
/************************************************************************/
static gboolean	gsfdm_url_push_url_from_file( gchar const *file, gboolean auto_dl, gboolean auto_rn )
{
st_gsfdm_url_entry *s;
guint		plug_id;
	
	#ifdef	DEBUG_URL_PUSH
	g_print( "debug url : received file to do : %s - %d %d\n", file, auto_dl, auto_rn );
	#endif	DEBUG_URL_PUSH
	
	// test type of file, add only regular file...
	
	if( gsfdm_file_get_file_type( file ) != GSFDM_FILE_TYPE_REGULAR )
	{	gsfdm_message_push_code( MESS_DISMISS, (gpointer)file );
		return( FALSE );
	}
	
	// first ask what to use to download, if no id, can download, abort...
	if( ( plug_id = gsfdm_plugin_get_id( file, FALSE ) ) )
	{	// plug found...
		// now parse data, want 'network' data
		// open stream on file, create a memory stream for anwser...
		FILE		*input = fopen( file, "r" );
		FILE		*output = fmemopen( NULL, URL_BUFFER_TO_PLUG, "w+" );
		gchar		*ext = gsfdm_file_extract_ext( file );
		
		if( gsfdm_plugin_call( plug_id, PLUG_PARSE_FILE, input, output, NULL, ext ) )
		{	// ok, add every 'line' as url 	
			gchar	*str = g_new0( gchar, URL_BUFFER_LINE_LENGTH );		// isn't it enough ???
			size_t	size = URL_BUFFER_LINE_LENGTH;
			
			rewind( output );
			// add evey line, call gsfdm_url_push_url, in case of 'local' ref..
			while( getline( &str, &size,output  ) != -1 )
			{	// last line is emply.. sometime
				if( !strlen( str ) )
					break;
				
				gsfdm_url_push_url( str, auto_dl, auto_rn );
			}
			
			g_free( str );
			g_free( ext );
			fclose( input );
			fclose( output );
			return( TRUE );
		}
		
		g_free( ext );
		fclose( input );
		fclose( output );
		// error when parsing...
	}
		
	// warm user and directly add an entry...
	s = gsfdm_url_st_new();
	gsfdm_url_st_get_url( s ) = g_strdup( file );
	gsfdm_url_st_get_name( s ) = gsfdm_file_get_file_name( file );
	gsfdm_url_st_get_path( s ) = g_strdup( gsfdm_config_get_default_save_dir() );
	gsfdm_url_st_get_auto_dl( s ) = FALSE;
	gsfdm_url_st_get_auto_rn( s ) = auto_rn;
	gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_WAIT;
		
	gsfdm_message_push_code( MESS_NO_PLUGIN, s );
	return( gsfdm_url_push_url_from_data( s ) );
}

/************************************************************************/
// same but from an url ( http://...  mms://... ) ; return true on success
/************************************************************************/
static gboolean	gsfdm_url_push_url_from_url( gchar const *url, gboolean auto_dl, gboolean auto_rn )
{
st_gsfdm_url_entry *s = gsfdm_url_st_new();

	#ifdef	DEBUG_URL_PUSH
	g_print( "debug url : received url to do : %s - %d %d\n", url, auto_dl, auto_rn );
	#endif	DEBUG_URL_PUSH
	
	// first ask what to use to download, if no id, can download, abort...
	if( !( gsfdm_url_st_get_plug_id( s ) = gsfdm_plugin_get_id( url, FALSE ) ) )
	{	gsfdm_url_st_destroy( s );
		return( FALSE );
	}
	
	// some url have no filename, using default page when sending data ; 
	// need a name to save on disk
	gsfdm_url_st_get_url( s ) = g_strdup( url );
	if( !( gsfdm_url_st_get_name( s ) = gsfdm_file_get_file_name( url ) ) )
		gsfdm_url_st_get_name( s ) = g_strdup( "index.html" );
	gsfdm_url_st_get_path( s ) = g_strdup( gsfdm_config_get_default_save_dir() );
	gsfdm_url_st_get_auto_dl( s ) = auto_dl;
	gsfdm_url_st_get_auto_rn( s ) = auto_rn;
	gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_WAIT;
	
	// data are all right, add to list
	return( gsfdm_url_push_url_from_data( s ) );
}

/************************************************************************/
// add but from a struct : internal use a startup from config
/************************************************************************/
static gboolean	gsfdm_url_push_url_from_data( st_gsfdm_url_entry *s )
{
static gchar		*text[] = { NULL, NULL, NULL, NULL };
st_gsfdm_url_entry	*s2;
gint				row;
gchar				*tmp;

	#ifdef	DEBUG_URL_ADD_DATA
	g_print( "debug url : parse data for %s\n", gsfdm_url_st_get_url( s ) );
	#endif

	// check if this url is already in list, no other solution : must check with every row...
	for( row = 0 ; row < list->rows ; row++ )
	{	s2 = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		if( !strcmp( gsfdm_url_st_get_url( s ), gsfdm_url_st_get_url( s2 ) ) )
		{	gsfdm_message_push_message( _( "Url already in list" ), FALSE, FALSE, FALSE );
			gsfdm_url_st_destroy( s );
			return( FALSE );
		}
	}

	// ask for history
	if( !gsfdm_historic_push_url( gsfdm_url_st_get_url( s ) ) )
	{	gsfdm_url_st_destroy( s );		
		return( FALSE );		
	}
	
	#ifdef	DEBUG_URL_ADD_DATA
	g_print( "---- debug url : parsed\n" );
	#endif
	
	// add ,struc to list, first, new id, what ever the data are filled with
	// --> and id MUST be unique
	gsfdm_url_st_get_id( s ) = gsfdm_url_get_new_id();
	text[ GSFDM_URL_COL_PERCENT ] = g_strdup_printf( URL_PERCENT_FORMAT, gsfdm_url_st_get_percent( s ) * 100.0 );	
	text[ GSFDM_URL_COL_NAME ] = gsfdm_url_st_get_name( s );
	row = gtk_clist_append( list, text );    
	gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, wait_low_pix, wait_low_bit );

	gtk_clist_set_row_data( list, row, (gpointer)s );
	g_free( text[ GSFDM_URL_COL_PERCENT ] );
	
	// focus on the last selected row	
/*	if( list->vadjustment->upper > list->vadjustment->value )
	{	gtk_adjustment_set_value( list->vadjustment, (float)( ( (gfloat)( row ) / 
			(gfloat)list->rows )*list->vadjustment->upper ) );
		gtk_adjustment_value_changed( list->vadjustment );
	}
*/	
	//###############################################################
	// add to config file
	gsfdm_url_add_to_config( gsfdm_url_st_get_id( s ) );
	
	// send message
	tmp = g_strdup_printf( _( "Adding file %s to list... ( %s )" ), gsfdm_url_st_get_name( s ), gsfdm_url_st_get_url( s ) );
	gsfdm_message_push_message( tmp, FALSE, FALSE, FALSE );

	return( TRUE );
}

/************************************************************************/
// return a new GList of (gint)id of selected url
/************************************************************************/
GList	*gsfdm_url_return_current_selection( void )
{
GList	*l = NULL, *tmp = list->selection;
	
	while( tmp )
	{	l = g_list_append( l, (gpointer)gsfdm_url_st_get_id( gtk_clist_get_row_data( list, (guint)tmp->data ) ) );
		tmp = g_list_next( tmp );
	}
	
	return( l );
}

/************************************************************************/
void	gsfdm_url_switch_auto_dl()
{
GList		*l;
gboolean	auto_dl;
st_gsfdm_url_entry	*s = NULL;

	l = list->selection;
	if( !l )
		return;						// nothing to do : no selection
	
	// get the state to fix
	auto_dl  = gsfdm_url_st_get_auto_dl( gtk_clist_get_row_data( list, (guint)l->data ) );
	l = g_list_next( l );
	
	while( l )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, (guint)l->data ) );
			
		if( gsfdm_url_st_get_auto_dl( s ) != auto_dl )
		{	gsfdm_url_st_get_auto_dl( s ) = auto_dl;
			gsfdm_url_update_to_config( gsfdm_url_st_get_id( s ) );
		}
		
		l = g_list_next( l );
	}	
}

/************************************************************************/
// directly return data ( const ) of the next ( in fact first with auto_dl on )
// url in state WAIT ; NULL if none
/************************************************************************/
static st_gsfdm_url_entry *gsfdm_url_get_next_job( void )
{
st_gsfdm_url_entry	*s = NULL;
guint				row;
	
	for ( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		if( gsfdm_url_st_get_auto_dl( s ) && ( gsfdm_url_st_get_state( s ) == GSFDM_URL_STATE_WAIT ) )
			return( s );						// found next job
	}
	
	return( NULL );								// nothing to do
}

/************************************************************************/
// return a new GList of(gint)id of row matching the spec
/************************************************************************/
GList	*gsfdm_url_return_state( GsfdmUrlState state )
{
GList				*l = NULL;
st_gsfdm_url_entry	*s = NULL;
guint				row;

	for ( row = 0 ; row < list->rows ; row++ )	// scan all row
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		if( gsfdm_url_st_get_state( s ) == state )
			l = g_list_append( l, (gpointer)gsfdm_url_st_get_id( s ) );	// found one more
	}
	
	return( l );
}

/************************************************************************/
// return data for a specific id / url ; NULL if none
/************************************************************************/
st_gsfdm_url_entry const *gsfdm_url_get_url_data( gint id )
{
guint				row;
st_gsfdm_url_entry	*s;
	
	for( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		// there no many lines, so that do it...
		if( id == gsfdm_url_st_get_id( s ) )
			return( s );
	}
	
	return( NULL );
}

/************************************************************************/
// return a new str ( list text formated, \n ended ) of the source of all url
/************************************************************************/
gchar	*gsfdm_url_get_url_list( void )
{
st_gsfdm_url_entry	*s;
gchar	*tmp = NULL, *tmp2;
guint	row;
	
	for( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		// there no many lines, so that do it...
		if( gsfdm_url_st_get_url( s ) )
		{	tmp2 = g_strconcat( gsfdm_url_st_get_url( s ), "\n", tmp, NULL );
			g_free( tmp );
			tmp = tmp2;
		}
	}
	
	return( tmp );
}

/************************************************************************/
// infor url that data of this id have changed
/************************************************************************/
static void		gsfdm_url_percent_changed( gint id )
{
st_gsfdm_url_entry	*s = NULL;
guint				row;
gchar				*tmp;
	
	// directly scan into the clist for a row matching this id, need data AND row...
	for( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		if( gsfdm_url_st_get_id( s ) == id )
		{	// found, change the name
			
			gtk_clist_set_text( list, row, GSFDM_URL_COL_NAME, gsfdm_url_st_get_name( s ) );
			tmp = g_strdup_printf( URL_PERCENT_FORMAT, gsfdm_url_st_get_percent( s ) * 100.0 );
			gtk_clist_set_text( list, row, GSFDM_URL_COL_PERCENT, tmp );
			g_free( tmp );
			
			// don't care of state : this is an update callback not a change callback
			
			// infor info module, perharps it's displayed
			gsfdm_info_inform_on_update( id );
		}
	}
}

/************************************************************************/
// infor the state changed, fill with new state...
// take care of message and config
/************************************************************************/
static void	gsfdm_url_id_state_changed( guint id )
{
st_gsfdm_url_entry	*s = NULL;
guint				row;
gchar				*tmp;
	
	// directly scan into the clist for a row matching this id, need data AND row...
	for( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		if( gsfdm_url_st_get_id( s ) == id )
		{	// found, change the name
			gtk_clist_set_text( list, row, GSFDM_URL_COL_NAME, gsfdm_url_st_get_name( s ) );
			tmp = g_strdup_printf( URL_PERCENT_FORMAT, gsfdm_url_st_get_percent( s ) * 100.0 );
			gtk_clist_set_text( list, row, GSFDM_URL_COL_PERCENT, tmp );
			g_free( tmp );
			
			// change icon
			switch( gsfdm_url_st_get_state( s ) )
			{	case GSFDM_URL_STATE_WAIT :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, wait_low_pix, wait_low_bit );
					break;
				case GSFDM_URL_STATE_RUNNING :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, running_low_pix, running_low_bit );
					break;
				case GSFDM_URL_STATE_DONE :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, done_low_pix, done_low_bit );
					break;
				case GSFDM_URL_STATE_DEAD :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, dead_low_pix, dead_low_bit );
					break;
				case GSFDM_URL_STATE_PARTIAL :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, partial_low_pix, partial_low_bit );
					break;
				case GSFDM_URL_STATE_UNABLE :
					gtk_clist_set_pixmap( list, row, GSFDM_URL_COL_STATE, unable_low_pix, unable_low_bit );
					break;
				default :
					return;
			}
			
			// infor info module, perharps it's displayed
			gsfdm_info_inform_for_change( id );
			
			// refresh in saved list
			gsfdm_url_refresh_in_config( id );
			break;
		}
	}
}


/************************************************************************/
// infor the data ( name, path, auto dl... ) changed
// take care of message and config
/************************************************************************/
void	gsfdm_url_data_changed( guint entry_id )
{
st_gsfdm_url_entry	*s = NULL;
guint				row;
gchar				*tmp;
	
	// directly scan into the clist for a row matching this id, need data AND row...
	for( row = 0 ; row < list->rows ; row++ )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, row ) );
		
		if( gsfdm_url_st_get_id( s ) == entry_id )
		{	// found, change the name
			gtk_clist_set_text( list, row, GSFDM_URL_COL_NAME, gsfdm_url_st_get_name( s ) );
			tmp = g_strdup_printf( URL_PERCENT_FORMAT, gsfdm_url_st_get_percent( s ) * 100.0 );
			gtk_clist_set_text( list, row, GSFDM_URL_COL_PERCENT, tmp );
			g_free( tmp );
			
			gsfdm_url_update_to_config( entry_id );
			break;
		}
	}
	
}

/************************************************************************/
/************************************************************************/
//	function used by toolbar, menu and context
/************************************************************************/
/************************************************************************/
void	gsfdm_url_start( void )
{
GList	*l, *tmp = gsfdm_url_return_current_selection();
	
	l = tmp;
	while( l )
	{	gsfdm_url_start_id( (guint)(l->data ) );
		l = g_list_next( l );
	}
	g_list_free( tmp );
}

/************************************************************************/
void	gsfdm_url_stop( void )
{
GList	*l, *tmp = gsfdm_url_return_current_selection();
	
	l = tmp;
	while( l )
	{	gsfdm_url_stop_id( (guint)(l->data ) );
		l = g_list_next( l );
	}
	g_list_free( tmp );
}

/************************************************************************/
void 	gsfdm_url_init( void )
{
GList				*l, *tmp = gsfdm_url_return_current_selection();
st_gsfdm_url_entry  const *s;
gchar				*file;
	
	l = tmp;
	while( l )
	{	s = gsfdm_url_get_url_data( (guint)(l->data ) );
		
		if( gsfdm_url_st_get_edit( s ) )
		{	l = g_list_next( l );
			continue;	
		}
		
		gsfdm_url_stop_id( (guint)(l->data ) );
		
		// if stop a running job ( in other case, why using this function ??? )
		// the new state is partial and must destroy the file
		if( gsfdm_url_st_get_state( s ) == GSFDM_URL_STATE_PARTIAL )
		{	file = g_strconcat( gsfdm_url_st_get_path( s ), "/", gsfdm_url_st_get_name( s ), NULL );
			unlink( file );
			g_free( file );
		}
		
		gsfdm_url_st_get_state( s ) = GSFDM_URL_STATE_WAIT;
		gsfdm_url_st_get_percent( s ) = 0.0;
		gsfdm_url_st_get_size( s ) = 0;
		gsfdm_url_st_get_time( s ) = 0;
		gsfdm_url_st_get_last_time( s ) = 0;
		gsfdm_url_id_state_changed( gsfdm_url_st_get_id( s ) );
		
		l = g_list_next( l );
	}
	g_list_free( tmp );
	
/*	// if no timer create new ; if auto_dl is disabled, the loop will take care of that
	if( !timer_id )
	{	timer_id = gtk_timeout_add( gsfdm_config_get_update_time() * 1000, (GtkFunction)gsfdm_url_main_loop, NULL );
	}
*/
}

/************************************************************************/
void	gsfdm_url_restart( void )
{
GList	*l, *tmp = gsfdm_url_return_current_selection();

	l = tmp;
	while( l )
	{	gsfdm_url_stop_id( (guint)(l->data ) );
		gsfdm_url_start_id( (guint)(l->data ) );
		l = g_list_next( l );
	}
	g_list_free( tmp );
}

/************************************************************************/
void	gsfdm_url_delete( void )
{
static GnomeDialog	*dlg;
st_gsfdm_url_entry	const *s;
GList				*l, *tmp;
gboolean			flag = FALSE, prompt = gsfdm_config_prompt_before_removing();
	
	// create a dialog, just in case(s)
	// this is a question dialog, but with 3 buttons...
	if( !dlg )
	{	GtkBox	*bh;
		
		dlg = GNOME_DIALOG( gnome_dialog_new( _( "Question before removing URL" ), GNOME_STOCK_BUTTON_OK, 
			_( "Always" ), GNOME_STOCK_BUTTON_CANCEL, NULL ) );
		gnome_dialog_close_hides( dlg, TRUE );
		gnome_dialog_set_close( dlg, TRUE );
		gnome_dialog_set_parent( dlg, GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( w ) ) ) );
			
		gtk_box_set_spacing( GTK_BOX( dlg->vbox ), 5 );
		
		gtk_box_pack_start( GTK_BOX( dlg->vbox ), GTK_WIDGET( bh = GTK_BOX( gtk_hbox_new( FALSE, 0 ) ) ) , FALSE, FALSE, 5 );
		
		gtk_box_pack_start( GTK_BOX( bh ), gnome_pixmap_new_from_xpm_d( gsfdm_xpm ), FALSE, FALSE, 5 );
		gtk_box_pack_start( GTK_BOX( bh ), 
			gtk_label_new( _( "Some files running or scheduled.\nDelete anyway ?? \n( and perhaps stop downloading)" ) ), TRUE, FALSE, 5 );
		
		gtk_widget_show_all( GTK_WIDGET( dlg->vbox ) );
	}
	
	l = tmp = gsfdm_url_return_current_selection();
	
	while( l )
	{	s = gsfdm_url_get_url_data( (guint)( l->data ) );

		switch( gsfdm_url_st_get_state( s ) )
		{	case GSFDM_URL_STATE_WAIT :
			case GSFDM_URL_STATE_RUNNING :
			case GSFDM_URL_STATE_PARTIAL :

			if( prompt )
				{	if( !flag )
					{	switch( gnome_dialog_run_and_close( dlg ) )
						{	case 1 :			// yes for all	
								flag = TRUE;
							case 0 :			// ok for this one
								break;
							default :			// cancel
								g_list_free( tmp );
								return;
						}
					}
					
					// this is just to prevent kill of running job...
					if( gsfdm_url_st_get_state( s ) == GSFDM_URL_STATE_RUNNING )
						gsfdm_url_stop_id( gsfdm_url_st_get_id( s ) );
				}	
				
			default :
				gsfdm_url_remove_url( gsfdm_url_st_get_id( s ) );				
		}
		
		l = g_list_next( l );
	}
	
	g_list_free( tmp );
}

/************************************************************************/
void	gsfdm_url_clear_url_list( void )
{	
GList	*l, *s;
	
	// start by removing all dead entry
	s = l = gsfdm_url_return_state( GSFDM_URL_STATE_DEAD );
	while( l )
	{	gsfdm_url_remove_url( (gint)l->data );
		l = g_list_next( l );
	}
	g_list_free( s );
	
	// and all done 
	s = l = gsfdm_url_return_state( GSFDM_URL_STATE_DONE );
	while( l )
	{	gsfdm_url_remove_url( (gint)l->data );
		l = g_list_next( l );
	}
	g_list_free( s );
}

/************************************************************************/
static void	gsfdm_url_edit_url( void )
{
	if( !( list->selection ) )
		return;
	
	gsfdm_edit_entry( gsfdm_url_st_get_id( gtk_clist_get_row_data( list, (gint)( list->selection->data ) ) ) );
}


/************************************************************************/
static void	gsfdm_url_select_all( void )
{	gtk_clist_freeze( list );
	gtk_clist_select_all( list );
	gtk_clist_thaw( list );
}

/************************************************************************/
static void	gsfdm_url_select_none( void )
{	gtk_clist_freeze( list );
	gtk_clist_unselect_all( list );
	gtk_clist_thaw( list );
}

/************************************************************************/
/************************************************************************/
/************************************************************************/

// pointer to the selection ; could have use an object_set_data
static GString	*select_data = NULL;				// ptr for selection ( owning )

/************************************************************************/
// clean the selection - ( set by a copy call )
static gint gsfdm_url_selection_clear( GtkWidget *widget, GdkEventSelection *event )
{
	if( select_data )
	{	g_string_truncate( select_data, 0 );
	}
	
	return( TRUE );
}

/************************************************************************/
//	someone request data from selection : give
static void gsfdm_url_selection_handle( GtkWidget *widget, GtkSelectionData *selection_data, guint info, guint time_stamp )
{
	if( select_data )
		gtk_selection_data_set( selection_data, GDK_SELECTION_TYPE_STRING, 8, select_data->str, select_data->len );
}
	
/************************************************************************/
// export the current selection
void	gsfdm_url_copy_url_list( void )
{
st_gsfdm_url_entry	*s;
GList	*l = list->selection;
	
	// if already a selection, clean first
	if( select_data )
	{	g_string_truncate( select_data, 0 );
	} else {
		select_data = g_string_sized_new( 512 );
	}
	
	// comput the list
	while( l )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, (guint)l->data ) );		
		g_string_append( select_data, gsfdm_url_st_get_url( s ) );
 		g_string_append_c( select_data, '\n' );
		l = g_list_next( l );
	}
	
	// and post
	if( !gtk_selection_owner_set( GTK_WIDGET( list ), GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME ) )
	{	g_string_truncate( select_data, 0 );					// error when posting, abort
		return;
	}
}

/************************************************************************/
// receiving the selection : do the real job
static void gsfdm_url_selection_received( GtkWidget *widget, GtkSelectionData *selection_data )
{
	//	overflow is a counter 
	static	void parse_string( gchar *start_pos, guint overflow )
	{	gchar	*next_pos, *str;
	
		// if no '\n', this is end of string
		if( !( next_pos = strchr( start_pos, '\n' ) ) )
			next_pos = start_pos + strlen( start_pos );

		// in case of 'blank line'
		if( next_pos != start_pos )
		{	// the string that "suppose" to match an url
			str = g_strndup( start_pos, next_pos - start_pos );
	
//####################################################################
			if( gsfdm_config_open_popup_on_dnd() )
				gsfdm_url_popup_url( str );
			else
				gsfdm_url_push_url( str, gsfdm_config_get_auto_download(), gsfdm_config_get_auto_rename() );
		
			g_free( str );
			
			if( ( ++overflow ) >= 256 )
				return;
		}
		
		// reach the end of string ???
		if( next_pos[0] )
			parse_string( next_pos+1, overflow );
	}
		
	/************************************************************************/
	#ifdef DEBUG_URL_SELEC_RECEIVE
	g_print( "debug url : paste / dnd\n" );
	#endif
	
    // received null list ???
	if( selection_data->length < 1 )
		return;
	
	// get only if a string...
	if( selection_data->type != GDK_SELECTION_TYPE_STRING )
	{	gsfdm_message_push_message( _( "Paste / dnd a selection of type unknow..." ), FALSE, FALSE, FALSE );
		return;
	}
	
	// start parsing the sting as a lsit of string, separed by '\n'
	// use recursive loop, could have used a infini while with a break statement
	{	gchar	*data = g_new0( char, selection_data->length + 1 );
		
			memcpy( ( void *)data, (void *)selection_data->data, selection_data->length );
 
// liberer la selection pour eviter un auto clean de gtk ??????????
//		  gtk_selection_data_free( selection_data );
                                              
			parse_string( data, 0 );					// copy to be sure to parse a null ending string
			g_free( data );
	}
}

/************************************************************************/
void	gsfdm_url_paste_url_list( void )
{
	// request the data
	gtk_selection_convert( GTK_WIDGET( list ), GDK_SELECTION_PRIMARY, gdk_atom_intern( "STRING", FALSE ), GDK_CURRENT_TIME );	
}

/************************************************************************/
//	receive a drop into the list of url
/************************************************************************/
static void	gsfdm_url_drop_received( GtkWidget *cl, GdkDragContext *c, gint x, gint y,
    GtkSelectionData *sel, guint i, guint t, gpointer d )
{
	// same thing than pasting a list...
	gtk_signal_emit_by_name( GTK_OBJECT( cl ), "selection_received", sel );
}

/************************************************************************/
// export the selection as dnd
/************************************************************************/
static void gsfdm_url_drag_send( GtkWidget *cl, GdkDragContext *context,
	GtkSelectionData *data, guint info, guint time, gpointer d )
{
st_gsfdm_url_entry	*s;
GList	*l = list->selection;
	
	// don't drag from and to this window
	if( context->dest_window == context->source_window )
	{	gtk_drag_finish( context, FALSE, FALSE, time );
		return;
	}
	
	// if already a selection, clean first
	if( select_data )
	{	g_string_truncate( select_data, 0 );
	} else {
		select_data = g_string_sized_new( 512 );
	}
	
	// comput the list
	// well actually ( 0.1 ) this export only one url cause clist clear the 
	// selection when drag
	while( l )
	{	s = ST_GSFDM_URL_ENTRY( gtk_clist_get_row_data( list, (guint)l->data ) );		
		g_string_append( select_data, gsfdm_url_st_get_url( s ) );
 		g_string_append_c( select_data, '\n' );
		l = g_list_next( l );
	}
	
	gtk_selection_data_set( data, gdk_atom_intern( "STRING", TRUE ), 
		8, g_strdup( select_data->str ), select_data->len );	
}
