/************************************************************************/

//			Gsfdm : a gnome stream file downloader

//				E-mail : j.a.l @ free.fr

//		Header for url list operations

/************************************************************************/

#ifndef	GSFDM_URL_LIST
#define	GSFDM_URL_LIST

#include	<gnome.h>
#include	<config.h>

/************************************************************************/
//	structure and define definition
/************************************************************************/
typedef	enum {
	GSFDM_URL_STATE_WAIT,			// url entry is sceduled
	GSFDM_URL_STATE_RUNNING,		// url is downloading
	GSFDM_URL_STATE_DONE,			// downloaded
	GSFDM_URL_STATE_DEAD,			// answer is dead : can't download
	GSFDM_URL_STATE_PARTIAL,		// downloading has been stopped
	GSFDM_URL_STATE_UNABLE,			// there error, usually between gsfdm and plug
	GSFDM_URL_STATE_ERROR = -1,		// case return when request update to plugin
									// it don't know what you're looking for
	
	// only used for communication
	GSFDM_URL_STATE_MASK = 0x7fff,	// just bit for state
	GSFDM_URL_CHANGE_MASK = 0x8000	// just bit that state a change in data ( dif from blank update... )
	
} GsfdmUrlState;

// struct is split in two part, just the 1st is accessable by 
// plug in, better than a simple cast
// consider this part as public, other private ( at least read-only )
typedef struct {
	gchar	*url;				// string of the source : or url
	gchar	*name;				// name used for save
	gchar	*path;				// rep where saved
	gchar	*host;				// host of download

	// protected
	gfloat	percent;			// percentage of download
	glong	size;				// byte done
	gint	pid;				// pid of child running the download
	glong	total;				// total size, 0 if unknow so the use of 'percent'
} st_gsfdm_url_entry_shared;

#define ST_GSFDM_URL_ENTRY_SHARED( s ) ( (( st_gsfdm_url_entry_shared *)s) )
#define gsfdm_url_st_shared_get_url( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->url )
#define gsfdm_url_st_shared_get_name( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->name )
#define gsfdm_url_st_shared_get_path( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->path )
#define gsfdm_url_st_shared_get_host( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->host )
#define gsfdm_url_st_shared_get_percent( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->percent )
#define gsfdm_url_st_shared_get_size( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->size )
#define gsfdm_url_st_shared_get_total( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->total )
#define gsfdm_url_st_shared_get_pid( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->pid )

// define the data of an entry / url
typedef struct {
	st_gsfdm_url_entry_shared base;
	
	// private
	guint	id;					// internal unique identifiant
	guint	plug_id;			// plug used
	
	gboolean	auto_dl;		// auto-downloading
	gboolean	auto_rn;		// allow direct auto rename if already disk file exist
	GsfdmUrlState state;		// current state
	gboolean	edited;			// this is an internal flag to prevent start of downloading 
								// when currently changing a field, or to change when downloading is running
								// use fonction to change it ( but you don't need that... ) true = edited
	time_t	time;				// time running
	time_t	last_time;			// last time where time update occur ( internal use )
	time_t	last_update_time;	// last time where data were updated ( for timeout )
} st_gsfdm_url_entry;

#define ST_GSFDM_URL_ENTRY( s ) ( (( st_gsfdm_url_entry *)s) )

// accesors, a friend can't live without then ( and finaly he's on true ) ; "au fait, salut seb..."
#define gsfdm_url_st_get_id( s ) ( ST_GSFDM_URL_ENTRY( s )->id )
#define gsfdm_url_st_get_plug_id( s ) ( ST_GSFDM_URL_ENTRY( s )->plug_id )
#define gsfdm_url_st_get_url( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->url )
#define gsfdm_url_st_get_name( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->name )
#define gsfdm_url_st_get_path( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->path )
#define gsfdm_url_st_get_host( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->host )
#define gsfdm_url_st_get_percent( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->percent )
#define gsfdm_url_st_get_size( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->size )
#define gsfdm_url_st_get_total( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->total )
#define gsfdm_url_st_get_pid( s ) ( ST_GSFDM_URL_ENTRY_SHARED( s )->pid )
#define gsfdm_url_st_get_auto_dl( s ) ( ST_GSFDM_URL_ENTRY( s )->auto_dl )
#define gsfdm_url_st_get_auto_rn( s ) ( ST_GSFDM_URL_ENTRY( s )->auto_rn )
#define gsfdm_url_st_get_edit( s ) ( ST_GSFDM_URL_ENTRY( s )->edited )
#define gsfdm_url_st_get_state( s ) ( ST_GSFDM_URL_ENTRY( s )->state )
#define gsfdm_url_st_get_time( s ) ( ST_GSFDM_URL_ENTRY( s )->time )
#define gsfdm_url_st_get_last_time( s ) ( ST_GSFDM_URL_ENTRY( s )->last_time )
#define gsfdm_url_st_get_last_update_time( s ) ( ST_GSFDM_URL_ENTRY( s )->last_update_time )

// return a new struct with defaut value : all at zero ...
#define	gsfdm_url_st_new() ( g_new0( st_gsfdm_url_entry, 1 ) )

// destroy ( free ) all data from a st_gsfdm_url_entry
void	gsfdm_url_st_destroy( st_gsfdm_url_entry *s );

/************************************************************************/
// true if can edit data for this entry, if true set edited flag to true ( lock entry... )
/************************************************************************/
gboolean	gsfdm_url_can_edit( st_gsfdm_url_entry *s );
void		gsfdm_url_release_edit( st_gsfdm_url_entry *s );

/************************************************************************/
//	functions definition
/************************************************************************/
// return the top widget of the info box
GtkWidget const	*gsfdm_url_get_dock( void );

/************************************************************************/
//	load the config and start all "stuff"
/************************************************************************/
void	gsfdm_url_load_config( void );

/************************************************************************/
//	save the state and data of all url
/************************************************************************/
void	gsfdm_url_save_config( void );

/************************************************************************/
//	these functions work on the job control
/************************************************************************/
// # of job(s) running --> 0 if at rest
guint	gsfdm_url_nb_are_working( void );

/************************************************************************/
// infor gsfdm_url to kill all process ( stop downloading )
void	gsfdm_url_stop_downloading( void );

/************************************************************************/
// set the # of jobs at a same time
void	gsfdm_url_set_max_job( guint number );

/************************************************************************/
// inform of new update time out
void	gsfdm_url_set_update_time( guint sec );

// *****************************************************************************
//	add an url by direct string ; open a new dlg
GnomeDialog	*gsfdm_url_open_url( void );

/************************************************************************/
//	open a new dlg and fix to this url
void	gsfdm_url_popup_url( gchar const *url );

/************************************************************************/
// check whether is local or net
gboolean	gsfdm_url_push_url( gchar const *file, gboolean auto_dl, gboolean auto_rn );

/************************************************************************/
// try to remove the url matching url ( not the name ) from the
// list ; if force = TRUE and running force stop else return false
gboolean	gsfdm_url_remove( gchar const *url, gboolean force );

/************************************************************************/
// stop download for this url, true if found
// switch to auto off if not running
gboolean	gsfdm_url_stop_url( gchar const *url );

/************************************************************************/
//	these functions infor on job, selection ...
/************************************************************************/
// return data for a specific id / url ; NULL if none
// DON'T EDIT
st_gsfdm_url_entry const *gsfdm_url_get_url_data( gint id );

/************************************************************************/
// return a new str ( list text formated, \n ended ) of the source of all url
gchar	*gsfdm_url_get_url_list( void );

/************************************************************************/
// infor the data ( name, path, auto dl... ) changed
// take care of message and config
void	gsfdm_url_data_changed( guint id );

/************************************************************************/
void	gsfdm_url_switch_auto_dl();

/************************************************************************/
// return a new GList of (gint)id of selected url
// NULL if none
GList	*gsfdm_url_return_current_selection( void );

/************************************************************************/
// return a new GList of(gint)id of row matching the spec
// NULL if NONE
GList	*gsfdm_url_return_state( GsfdmUrlState state );

/************************************************************************/
// function shared in toolbar, menu and context menu
/************************************************************************/
// there use the selected list
void	gsfdm_url_copy_url_list( void );
void	gsfdm_url_paste_url_list( void );
void	gsfdm_url_clear_url_list( void );
void	gsfdm_url_start( void );
void	gsfdm_url_stop( void );
void 	gsfdm_url_init( void );
void	gsfdm_url_restart( void );
void	gsfdm_url_delete( void );

#endif	GSFDM_URL_LIST
